@@ -1,4 +1,4 @@
-# $Id: browser.tcl,v 1.52 2006/01/28 20:45:08 aleksey Exp $
+# $Id: browser.tcl 848 2006-12-29 12:03:05Z sergei $
 
 option add *JBrowser.fill          Black	widgetDefault
 option add *JBrowser.nscolor       MidnightBlue widgetDefault
@@ -41,6 +41,8 @@ proc browser::open {args} {
     set config(fill) 	      [option get $bw fill          JBrowser]
     set config(nscolor)       [option get $bw nscolor       JBrowser]
 
+    bind $bw <Destroy> [list [namespace current]::destroy_state $bw]
+
     frame $bw.navigate
     button $bw.navigate.back -text <- \
 	-command [list browser::history_move $bw 1]
@@ -65,16 +67,14 @@ proc browser::open {args} {
     set sw [ScrolledWindow $bw.sw]
 
     set tw [Tree $bw.tree -deltax 16 -deltay 18 -dragenabled 1 \
-		-draginitcmd browser::draginitcmd]
+		-draginitcmd [list browser::draginitcmd $bw]]
     $sw setwidget $tw
 
     pack $sw -side top -expand yes -fill both
     set browser(tree,$bw) $tw
     $tw bindText <Double-ButtonPress-1> [list browser::textaction $bw]
     $tw bindText <ButtonPress-3>        [list browser::textpopup $bw]
-    $tw bindText <Any-Enter>  [list browser::textballoon $bw enter  %X %Y]
-    $tw bindText <Any-Motion> [list browser::textballoon $bw motion %X %Y]
-    $tw bindText <Any-Leave>  [list browser::textballoon $bw leave  %X %Y]
+    balloon::setup $tw -command [list browser::textballoon $bw]
 
     bindscroll $tw.c
 
@@ -86,6 +86,8 @@ proc browser::open {args} {
     set browser(hist,$bw) {}
     set browser(histpos,$bw) 0
 
+    hook::run open_browser_post_hook $bw $sw $tw
+
     incr brwid
     browser::go $bw
 }
@@ -113,8 +115,6 @@ proc browser::go {bw} {
 
 	set browse_list [update_combo_list $browse_list $jid 20]
 	$bw.navigate.entry configure -values $browse_list
-	set custom::saved([namespace current]::browse_list) $browse_list
-	custom::store
 
 	jlib::send_iq get \
 	    [jlib::wrapper:createtag query \
@@ -223,35 +223,21 @@ image create photo ""
 
 proc browser::item_icon {category type} {
     switch -- $category {
-	service {
-	    switch -- $type {
-		jud        { return browser/jud }
-		aim        { return browser/aim }
-		icq        { return browser/icq }
-		msn        { return browser/msn }
-		x-gadugadu { return browser/gg }
-		x-weather  { return browser/weather }
-		yahoo      { return browser/yahoo }
-		default    { return "" }
-	    }
-	}
+	service -
+	gateway -
 	application {
-	    switch -- $type {
-		x-weather { return browser/weather }
-		default   { return "" }
+	    if {[lsearch -exact [image names] browser/$type] >= 0} {
+		return browser/$type
+	    } else {
+		return ""
 	    }
 	}
-	conference {
-	    return browser/conference
-	}
-	user {
-	    return browser/user
-	}
-	headline {
-	    return browser/message
-	}
 	default {
-	    return ""
+	    if {[lsearch -exact [image names] browser/$category] >= 0} {
+		return browser/$category
+	    } else {
+		return ""
+	    }
 	}
     }
 }
@@ -625,37 +611,25 @@ proc browser::browser_action {action bw node} {
     }
 }
 
-proc browser::textballoon {bw action X Y node} {
+proc browser::textballoon {bw node} {
     variable browser
 
     set tw $browser(tree,$bw)
-    set data [$tw itemcget $node -data]
-    set data2 [lassign $data type]
+    set data [lassign [$tw itemcget $node -data] \
+		      type jid category subtype name version]
     if {$type == "jid"} {
-	    lassign $data2 jid category type name version
+	return [list $bw:$node \
+		     [item_balloon_text \
+			  $bw $jid $category $subtype $name $version]]
     } else {
-	return
-    }
-
-    switch -- $action {
-	enter {
-	    balloon::set_text \
-		[item_balloon_text $bw $jid $category $type $name $version]
-	}
-	motion {
-	    balloon::on_mouse_move \
-		[item_balloon_text $bw $jid $category $type $name $version] \
-		$X $Y
-	}
-	leave {
-	    balloon::destroy
-	}
+	return [list $bw:$node ""]
     }
 }
 
-proc browser::draginitcmd {t node top} {
+proc browser::draginitcmd {bw t node top} {
+    set connid browser(connid,$bw)
     set data [$t itemcget $node -data]
-    set data2 [lassign $data type]
+    set data2 [linsert [lassign $data type] 0 $connid]
 
     if {$type == "jid"} {
 	if {[set img [$t itemcget $node -image]] != ""} {
@@ -669,7 +643,7 @@ proc browser::draginitcmd {t node top} {
 }
 
 proc browser::entrydropcmd {bw target source pos op type data} {
-    set jid [lindex $data 0]
+    set jid [lindex $data 1]
     goto $bw $jid
 }
 
@@ -693,3 +667,12 @@ proc browser::register_ns_handler {ns handler args} {
     }
 }
 
+# Destroy all (global) state assotiated with the given browser window.
+# Intended to be bound to a <Destroy> event handler for browser windows.
+proc browser::destroy_state {bw} {
+    variable browser
+
+    array unset browser *,$bw
+    array unset browser *,$bw,*
+}
+
