@@ -1,21 +1,18 @@
-# $Id: chats.tcl,v 1.153 2006/05/28 14:54:24 aleksey Exp $
+# $Id: chats.tcl 1024 2007-03-07 15:58:27Z sergei $
 
 package require textutil
 
-option add *Chat.theyforeground        red3   widgetDefault
-option add *Chat.meforeground          blue   widgetDefault
+option add *Chat.theyforeground        blue   widgetDefault
+option add *Chat.meforeground          red3   widgetDefault
+option add *Chat.highlightforeground   red3   widgetDefault
 option add *Chat.serverlabelforeground green  widgetDefault
 option add *Chat.serverforeground      violet widgetDefault
 option add *Chat.infoforeground        blue   widgetDefault
 option add *Chat.errforeground         red    widgetDefault
-option add *urlforeground              blue   widgetDefault
-option add *urlactiveforeground        red    widgetDefault
 option add *Chat.inputheight           3      widgetDefault
 
 
 namespace eval chat {
-    array set opened {}
-    set urlid 0
     set enrichid 0
     custom::defgroup Chat [::msgcat::mc "Chat options."] -group Tkabber
     custom::defvar options(smart_scroll) 0 \
@@ -24,10 +21,6 @@ namespace eval chat {
     custom::defvar options(stop_scroll) 0 \
 	[::msgcat::mc "Stop chat window autoscroll."] \
 	-type boolean -group Chat
-    custom::defvar options(emphasize) 1 \
-	[::msgcat::mc "Enable messages emphasize."] \
-	-type boolean -group Chat \
-	-command chat::switch_emphasize
     custom::defvar options(display_status_description) 1 \
 	[::msgcat::mc "Display description of user status in chat windows."] \
 	-type boolean -group Chat
@@ -36,51 +29,14 @@ namespace eval chat {
 	-type radio -group Chat \
 	-values [list normal [::msgcat::mc "Normal"] \
 		      chat [string trim [::msgcat::mc "Chat "]]]
+    custom::defvar options(gen_status_change_msgs) 0 \
+	[::msgcat::mc "Generate chat messages when chat peer\
+	    changes his/her status and/or status message"] \
+	-type boolean -group Chat
 
     custom::defvar open_chat_list {} [::msgcat::mc "List of users for chat."] \
 	    -group Hidden
 
-    set url_regexp {^
-		([^\w\d]*)
-		(
-
-		 (?:
-			(?: ftp|https?)://[-\w]+(\.\w[-\w]*)+
-		   |
-			(?: [a-z0-9][-a-z0-9]* \. )+
-			(?: com
-			  | edu
-			  | biz
-			  | gov
-			  | in(?:t|fo)
-			  | mil
-			  | net
-			  | org
-			  | name
-			  | aero
-			  | arpa
-			  | coop
-			  | museum
-			  | pro
-			  | travel
-			  | [a-z][a-z]
-			)
-		)
-		(?: : \d+ )?
-
-		(?:
-			/
-			[^.,?!:;"'<>()\[\]{}\s\x7F-\xFF]*
-			(?:
-				[.,?!:;]+ [^.,?!:;"'<>()\[\]{}\s\x7F-\xFF]+
-
-			)*
-
-		)?
-	     )
-	     ([^\w\d]*)$
-    }
-
 }
 
 set chat_width 50
@@ -111,7 +67,7 @@ proc chat::open_chat_dialog {} {
 
     label $gf.ljid -text [::msgcat::mc "JID:"]
     ecursor_entry [ComboBox $gf.jid -textvariable [namespace current]::open_chat_jid \
-	    -values $open_chat_list -width 35].e
+	    -values [linsert $open_chat_list 0 ""] -width 35].e
     
     grid $gf.ljid -row 0 -column 0 -sticky e
     grid $gf.jid  -row 0 -column 1 -sticky ew
@@ -152,8 +108,6 @@ proc chat::open_chat {gw} {
     }
 
     set open_chat_list [update_combo_list $open_chat_list $open_chat_jid 20]
-    set custom::saved([namespace current]::open_chat_list) $open_chat_list
-    custom::store
     open_to_user $connid $open_chat_jid
 }
 
@@ -171,7 +125,11 @@ proc chat::get_nick {connid jid type} {
 		set nick [roster::itemconfig $connid \
 			      [roster::find_jid $connid $jid] -name]
 		if {$nick == ""} {
-		    set nick [node_from_jid $jid]
+		    if {[node_from_jid $jid] != ""} {
+			set nick [node_from_jid $jid]
+		    } else {
+			set nick $jid
+		    }
 		}
 	    }
 	}
@@ -201,37 +159,31 @@ proc chat::get_jid {chatid} {
     return [lindex $chatid 1]
 }
 
-proc client:message {connid from id type is_subject subject body err thread priority x} {
-    debugmsg chat "MESSAGE: $connid; $from; $id $type; $is_subject; $subject; $body; $err;\
-$thread; $priority; $x"
+###############################################################################
 
-    if {![cequal [info commands ::ssj::encrypted:input] ""]} {
-	set xs {}
+proc client:message \
+     {connid from id type is_subject subject body err thread priority x} {
 
-	foreach xe $x {
-	    jlib::wrapper:splitxml $xe tag vars isempty chdata children
+    debugmsg chat "MESSAGE: $connid; $from; $id; $type; $is_subject;\
+		   $subject; $body; $err; $thread; $priority; $x"
 
-	    if {![cequal [jlib::wrapper:getattr $vars xmlns] \
-			 jabber:x:encrypted]} {
-		lappend xs $xe
-		continue
-	    } elseif {[cequal $chdata ""]} {
-# in case the sender didn't check the exit code from gpg...
-	    } elseif {[catch { ssj::encrypted:input $connid $from $chdata } body]} {
-		set body [format [::msgcat::mc ">>> Unable to decipher data: %s <<<"] $body]
-	    } else {
-		lappend xs $xe
-	    }
-	}
+    hook::run rewrite_message_hook connid from id type is_subject \
+				   subject body err thread priority x
 
-	set x $xs
-    }
+    debugmsg chat "REWRITTEN MESSAGE: $connid; $from; $id; $type; $is_subject;\
+		   $subject; $body; $err; $thread; $priority; $x"
 
-    chat::process_message \
-	$connid $from $id $type $is_subject $subject $body $err $thread $priority $x
+    hook::run process_message_hook $connid $from $id $type $is_subject \
+				   $subject $body $err $thread $priority $x
 }
 
-proc chat::process_message {connid from id type is_subject subject body err thread priority x} {
+###############################################################################
+
+proc chat::rewrite_message \
+     {vconnid vfrom vid vtype vis_subject vsubject \
+      vbody verr vthread vpriority vx} {
+    upvar 2 $vfrom from
+    upvar 2 $vtype type
     variable options
 
     if {$type == ""} {
@@ -239,19 +191,18 @@ proc chat::process_message {connid from id type is_subject subject body err thre
     }
 
     set from [tolower_node_and_domain $from]
-
-    hook::run process_message_hook \
-	$connid $from $id $type $is_subject $subject $body $err $thread $priority $x
 }
 
-proc chat::process_message_fallback {connid from id type is_subject subject body err thread priority x} {
+hook::add rewrite_message_hook [namespace current]::chat::rewrite_message 99
+
+###############################################################################
+
+proc chat::process_message_fallback \
+     {connid from id type is_subject subject body err thread priority x} {
     global font
-    variable opened
     variable chats
 
     set chatid [chatid $connid $from]
-    hook::run normalize_chat_id_hook chatid $type
-
 
     switch -- $type {
 	chat {
@@ -278,13 +229,20 @@ proc chat::process_message_fallback {connid from id type is_subject subject body
 	    }
 	}
 	error {
-	    if {$is_subject} {
-		set body "subject: "
-		restore_subject $chatid
+	    if {[is_groupchat $chatid]} {
+		if {$is_subject} {
+		    set body "subject: "
+		    restore_subject $chatid
+		} else {
+		    set body ""
+		}
+		append body [error_to_string $err]
 	    } else {
-		set body ""
+		if {$is_subject && $subject != ""} {
+		    set body "[::msgcat::mc {Subject:}] $subject\n$body"
+		}
+		set body "[error_to_string $err]\n$body"
 	    }
-	    append body [error_to_string $err]
 	}
 	default {
 	    debugmsg chat "MESSAGE: UNSUPPORTED message type '$type'"
@@ -302,10 +260,9 @@ proc chat::process_message_fallback {connid from id type is_subject subject body
 hook::add process_message_hook \
     [namespace current]::chat::process_message_fallback 99
 
+###############################################################################
 
 proc chat::window_titles {chatid} {
-    variable chats
-
     set connid [get_connid $chatid]
     set jid [get_jid $chatid]
     set chatname [roster::itemconfig $connid \
@@ -331,23 +288,19 @@ proc chat::reconnect_groupchats {connid} {
     variable chats
     global grouproster
 	
-    if {![info exists chats(opened)]} return
-
-    foreach chatid $chats(opened) {
-	if {[cequal [get_connid $chatid] $connid]} {
-	    if {[is_groupchat $chatid]} {
-		if {[info exists ::muc::muc_password($chatid)]} {
-		    set password $::muc::muc_password($chatid)
-		} else {
-		    set password ""
-		}
-		join_group [get_jid $chatid] \
-		    -nick [get_our_groupchat_nick $chatid] \
-		    -password $password \
-		    -connection $connid
+    foreach chatid [opened $connid] {
+	if {[is_groupchat $chatid]} {
+	    if {[info exists ::muc::muc_password($chatid)]} {
+		set password $::muc::muc_password($chatid)
 	    } else {
-		set chats(status,$chatid) connected
+		set password ""
 	    }
+	    muc::join_group $connid \
+			    [get_jid $chatid] \
+			    [get_our_groupchat_nick $chatid] \
+			    $password
+	} else {
+	    set chats(status,$chatid) connected
 	}
     }
 }
@@ -356,28 +309,39 @@ hook::add connected_hook [namespace current]::chat::reconnect_groupchats 99
 
 proc chat::disconnect_groupchats {connid} {
     variable chats
+    global statusdesc
 
-    if {![info exists chats(opened)]} return
+    foreach chatid [opened $connid] {
+	if {![winfo exists [chat_win $chatid]]} return
 
-    foreach chatid $chats(opened) {
-	if {[cequal $connid {}] || [cequal [get_connid $chatid] $connid]} {
-	    set chatw [chat_win $chatid]
-	    if {[winfo exists $chatw] && \
-		    $chats(status,$chatid) != "disconnected"} {
-		set chats(status,$chatid) disconnected
-		add_message $chatid [get_jid $chatid] error \
-		    [::msgcat::mc "Disconnected"] {}
-	    }
+	set jid [get_jid $chatid]
+
+	if {$chats(status,$chatid) != "disconnected"} {
+	    set chats(status,$chatid) disconnected
+	    add_message $chatid $jid error [::msgcat::mc "Disconnected"] {}
+	}
+
+	set cw [winid $chatid]
+
+	if {[winfo exists $cw.status.icon]} {
+	    $cw.status.icon configure \
+		-image [ifacetk::roster::get_jid_icon $connid $jid unavailable] \
+		-helptext ""
+	}
+
+	if {[winfo exists $cw.status.desc]} {
+	    $cw.status.desc configure \
+		-text "($statusdesc(unavailable))" \
+		-helptext ""
 	}
     }
 }
 
-hook::add disconnected_hook [namespace current]::chat::disconnect_groupchats 99
+hook::add predisconnected_hook [namespace current]::chat::disconnect_groupchats 99
 
 proc chat::open_window {chatid type args} {
     global font font_bold font_italic font_bold_italic
     global chat_width chat_height
-    variable opened
     variable chats
     variable chat_id
     variable options
@@ -425,14 +389,9 @@ proc chat::open_window {chatid type args} {
 	set chats(status,$chatid) connected
     }
     set chats(exit_status,$chatid) ""
-    debugmsg chat "OURJID: [our_jid $chatid]"
-    lappend chats(opened) $chatid
+    add_to_opened $chatid
     
-    set opened($chatid) $cw
     set chat_id($cw) $chatid
-    #debugmsg chat $cw
-    #debugmsg chat [array names opened_chats]
-    #debugmsg chat $opened_chats($chatid)
 
     lassign [chat::window_titles $chatid] chats(tabtitlename,$chatid) \
 	chats(titlename,$chatid)
@@ -441,7 +400,6 @@ proc chat::open_window {chatid type args} {
 	-tabtitle $chats(tabtitlename,$chatid) \
 	-class Chat -type $type \
 	-raisecmd "focus [list $cw.input]
-		   tab_set_updated [list $cw]
                    hook::run raise_chat_tab_hook [list $cw] [list $chatid]"
 
     frame $cw.status
@@ -449,7 +407,7 @@ proc chat::open_window {chatid type args} {
     if {[cequal $type chat]} {
 	set status [get_user_status $connid $jid]
 	Label $cw.status.icon \
-	    -image [ifacetk::roster::get_jid_icon $connid $jid] \
+	    -image [ifacetk::roster::get_jid_icon $connid $jid $status] \
 	    -helptext [get_user_status_desc $connid $jid]
 	pack $cw.status.icon -side left
 	if {$options(display_status_description)} {
@@ -480,16 +438,8 @@ proc chat::open_window {chatid type args} {
 	bind $cw.status.subject <Escape> \
 	    [list chat::restore_subject [double% $chatid]]
 
-	bind $cw.status.subject <Any-Enter> \
-	    +[list eval balloon::set_text \
-		   \[[namespace current]::set_subject_balloon [list [double% $chatid]]\] \
-		   -width \[winfo width %W\]]
-
-	bind $cw.status.subject <Any-Motion> \
-	    [list eval balloon::on_mouse_move \
-		  \[[namespace current]::set_subject_balloon [list [double% $chatid]]\] %X %Y]
-
-	bind $cw.status.subject <Any-Leave> {+ balloon::destroy}
+	balloon::setup $cw.status.subject \
+		       -command [list [namespace current]::set_subject_balloon $chatid]
     }
     foreach tag [bind Menubutton] {
         if {[string first 1 $tag] >= 0} {
@@ -502,8 +452,8 @@ proc chat::open_window {chatid type args} {
     pack $cw.pw0 -fill both -expand yes
 
 
-    set upw [$cw.pw0 add -weight 1 -minsize 0]
-    set dow [$cw.pw0 add -weight 0 -minsize 0]
+    set upw [PanedWinAdd $cw.pw0 -weight 1 -minsize 0]
+    set dow [PanedWinAdd $cw.pw0 -weight 0 -minsize 0]
 
     set isw [ScrolledWindow $cw.isw -scrollbar vertical]
     pack $cw.isw -fill both -expand yes -side bottom -in $dow
@@ -517,9 +467,9 @@ proc chat::open_window {chatid type args} {
 	PanedWin $cw.pw -side bottom -pad 2 -width 8
 	pack $cw.pw -fill both -expand yes -in $upw
 
-	set cf [$cw.pw add -weight 1 -minsize 0]
+	set cf [PanedWinAdd $cw.pw -weight 1 -minsize 0]
 
-	set uw [$cw.pw add -weight 0 -minsize 0]
+	set uw [PanedWinAdd $cw.pw -weight 0 -minsize 0]
 	set chats(userswin,$chatid) $uw.users
 
 	set rosterwidth [option get . chatRosterWidth [winfo class .]]
@@ -530,7 +480,10 @@ proc chat::open_window {chatid type args} {
 	ifacetk::roster::create $uw.users -width $rosterwidth \
 	    -popup ifacetk::roster::groupchat_popup_menu \
 	    -singleclick [list [namespace current]::user_singleclick $chatid] \
-	    -doubleclick ::ifacetk::roster::jid_doubleclick
+	    -doubleclick ::ifacetk::roster::jid_doubleclick \
+	    -draginitcmd [namespace current]::draginitcmd \
+	    -dropovercmd [list [namespace current]::dropovercmd $chatid] \
+	    -dropcmd [list [namespace current]::dropcmd $chatid]
 	pack $uw.users -fill both -side right -expand yes
 	[winfo parent $uw] configure -width $rosterwidth
 
@@ -546,38 +499,22 @@ proc chat::open_window {chatid type args} {
 
     set csw [ScrolledWindow $cf.csw -scrollbar vertical -auto none]
     pack $csw -expand yes -fill both -side top -in $pack_in
-    text $cf.chat -width $chat_width -height $chat_height -font $font -wrap word
+
+    ::richtext::richtext $cf.chat \
+        -width $chat_width -height $chat_height \
+        -font $font -wrap word
+
+    ::plugins::chatlog::config $cf.chat \
+	-theyforeground [query_optiondb $cw theyforeground] \
+	-meforeground [query_optiondb $cw meforeground] \
+	-serverlabelforeground [query_optiondb $cw serverlabelforeground] \
+	-serverforeground [query_optiondb $cw serverforeground] \
+	-infoforeground [query_optiondb $cw infoforeground] \
+	-errforeground [query_optiondb $cw errforeground] \
+	-highlightforeground [query_optiondb $cw highlightforeground]
+
     $csw setwidget $cf.chat
 
-    $cf.chat tag configure they \
-	-foreground [option get $cw theyforeground Chat]
-    $cf.chat tag configure me \
-	-foreground [option get $cw meforeground Chat]
-    $cf.chat tag configure highlight \
-	-foreground [option get $cw meforeground Chat]
-    $cf.chat tag configure server_lab \
-	-foreground [option get $cw serverlabelforeground Chat]
-    $cf.chat tag configure server \
-	-foreground [option get $cw serverforeground Chat]
-    $cf.chat tag configure info \
-	-foreground [option get $cw infoforeground Chat]
-    $cf.chat tag configure err \
-	-foreground [option get $cw errforeground Chat]
-
-    $cf.chat tag configure bold -font $font_bold
-    $cf.chat tag configure italic -font $font_italic
-    $cf.chat tag configure bold_italic -font $font_bold_italic
-    $cf.chat tag configure underlined -underline 1
-    if {$options(emphasize)} {
-	$cf.chat tag configure emphasized -elide 0
-	$cf.chat tag configure nonemphasized -elide 1
-    } else {
-	$cf.chat tag configure emphasized -elide 1
-	$cf.chat tag configure nonemphasized -elide 0
-    }
-
-
-    $cf.chat configure -state disabled
     focus $cw.input
 
     bind $cw.input <Shift-Key-Return> { }
@@ -586,15 +523,20 @@ proc chat::open_window {chatid type args} {
 	chat::send_message [list $cw] [list $chatid] [list $type]
 	break"]
 
-    bind $cw.input <Control-z> { %W undo }
-    bind $cw.input <Control-Z> { %W redo }
-
     regsub -all %W [bind Text <Prior>] $cf.chat prior_binding
     regsub -all %W [bind Text <Next>] $cf.chat next_binding
     bind $cw.input <Meta-Prior> $prior_binding
     bind $cw.input <Meta-Next> $next_binding
     bind $cw.input <Alt-Prior> $prior_binding
     bind $cw.input <Alt-Next> $next_binding
+    bind $cw.input <Meta-Prior> +break
+    bind $cw.input <Meta-Next> +break
+    bind $cw.input <Alt-Prior> +break
+    bind $cw.input <Alt-Next> +break
+    bind $cw.input <Control-Meta-Prior> continue
+    bind $cw.input <Control-Meta-Next> continue
+    bind $cw.input <Control-Alt-Prior> continue
+    bind $cw.input <Control-Alt-Next> continue
 
     bind $cw <Destroy> [list chat::close_window [double% $chatid]]
 
@@ -603,6 +545,55 @@ proc chat::open_window {chatid type args} {
 
 ###############################################################################
 
+# This proc is used by the "richtext widget" to query the option DB for
+# it's attributes which are really maintained by the main chat window
+proc chat::query_optiondb {w option} {
+    option get $w $option Chat
+}
+
+###############################################################################
+
+proc chat::draginitcmd {target x y top} {
+    return {}
+}
+
+###############################################################################
+
+proc chat::dropovercmd {chatid target source event x y op type data} {
+    variable chats
+
+    set chat_connid [get_connid $chatid]
+    lassign $data jid_connid jid
+
+    if {$source != ".roster.canvas" || \
+	    ![info exists chats(status,$chatid)] || \
+	    $chats(status,$chatid) == "disconnected" || \
+	    $chat_connid != $jid_connid || \
+	    ![::roster::itemconfig $jid_connid $jid -isuser]} {
+	DropSite::setcursor dot
+	return 0
+    } else {
+	DropSite::setcursor based_arrow_down
+	return 1
+    }
+}
+
+###############################################################################
+
+proc chat::dropcmd {chatid target source x y op type data} {
+    set group [get_jid $chatid]
+    lassign $data connid jid
+    set reason [::msgcat::mc "Please join %s" $group]
+
+    if {[muc::is_compatible $group]} {
+	muc::invite_muc $connid $group $jid $reason
+    } else {
+	muc::invite_xconference $connid $group $jid $reason
+    }
+}
+
+###############################################################################
+
 proc chat::user_singleclick {chatid cjid} {
     lassign $cjid connid jid
     set nick [get_nick $connid $jid groupchat]
@@ -622,13 +613,12 @@ hook::add open_chat_post_hook [namespace current]::chat::bind_window_click
 ###############################################################################
 
 proc chat::close_window {chatid} {
-    variable opened
     variable chats
     variable chat_id
 
-    if {![info exists opened($chatid)]} return
+    if {![is_opened $chatid]} return
 
-    unset opened($chatid)
+    remove_from_opened $chatid
 
     set connid [get_connid $chatid]
     set jid [get_jid $chatid]
@@ -636,10 +626,6 @@ proc chat::close_window {chatid} {
     set cw [winid $chatid]
     unset chat_id($cw)
 
-    set idx [lsearch -exact $chats(opened) $chatid]
-    set chats(opened) [lreplace $chats(opened) $idx $idx]
-    debugmsg chat $chats(opened)
-
     if {[is_groupchat $chatid]} {
 	muc::leave_group $connid $jid [get_our_groupchat_nick $chatid] \
 			 $chats(exit_status,$chatid)
@@ -654,15 +640,91 @@ proc chat::close_window {chatid} {
     hook::run close_chat_post_hook $chatid
 }
 
-proc chat::opened {} {
-    variable chats
-    if {[info exists chats(opened)]} {
-	return $chats(opened)
+##############################################################################
+
+proc chat::trace_room_nickname_change {connid group nick new_nick} {
+    set from $group/$nick
+    set to $group/$new_nick
+    set chatid [::chat::chatid $connid $from]
+    set ix [lsearch -exact [::chat::opened] $chatid]
+    if {$ix < 0} return
+
+    set new_cid [::chat::chatid $connid $to]
+
+    set msg [format [::msgcat::mc \
+	"%s has changed nick to %s."] \
+	$nick $new_nick]
+    ::chat::add_message $chatid "" chat $msg {}
+
+    set cw [chat_win $chatid]
+    $cw config -state normal
+    $cw delete {end - 1 char} ;# zap trailing newline
+    $cw insert end " "
+    set tooltip [::msgcat::mc "Opens a new chat window\
+	for the new nick of the room occupant"]
+    ::plugins::urls::render_url $cw url $tooltip {} \
+	-title [::msgcat::mc "Open new conversation"] \
+	-command [list [namespace current]::open_window $new_cid chat]
+    $cw insert end \n
+    $cw config -state disabled
+}
+
+hook::add room_nickname_changed_hook chat::trace_room_nickname_change
+
+##############################################################################
+
+proc chat::check_connid {connid chatid} {
+    if {[get_connid $chatid] == $connid} {
+	return 1
     } else {
-	return {}
+	return 0
     }
 }
 
+##############################################################################
+
+proc chat::opened {{connid {}}} {
+    variable chats
+
+    if {![info exists chats(opened)]} {
+	return {}
+    } elseif {$connid != {}} {
+	return [lfilter [list [namespace current]::check_connid $connid] \
+			$chats(opened)]
+    } else {
+	return $chats(opened)
+    }
+}
+
+proc chat::is_opened {chatid} {
+    variable chats
+
+    if {[info exists chats(opened)] && \
+	    ([lsearch -exact $chats(opened) $chatid] >= 0)} {
+	return 1
+    } else {
+	return 0
+    }
+}
+
+proc chat::add_to_opened {chatid} {
+    variable chats
+
+    lappend chats(opened) $chatid
+    set chats(opened) [lsort -unique $chats(opened)]
+}
+
+proc chat::remove_from_opened {chatid} {
+    variable chats
+
+    set idx [lsearch -exact $chats(opened) $chatid]
+    if {$idx >= 0} {
+	set chats(opened) [lreplace $chats(opened) $idx $idx]
+    }
+}
+
+##############################################################################
+
 proc chat::users_win {chatid} {
     variable chats
     return $chats(userswin,$chatid)
@@ -678,6 +740,17 @@ proc chat::input_win {chatid} {
     return $chats(inputwin,$chatid)
 }
 
+##############################################################################
+
+proc chat::is_chat {chatid} {
+    variable chats
+    if {[info exists chats(type,$chatid)]} {
+	return [cequal $chats(type,$chatid) chat]
+    } else {
+	return 1
+    }
+}
+
 proc chat::is_groupchat {chatid} {
     variable chats
     if {[info exists chats(type,$chatid)]} {
@@ -687,6 +760,22 @@ proc chat::is_groupchat {chatid} {
     }
 }
 
+##############################################################################
+
+proc chat::is_disconnected {chatid} {
+    variable chats
+
+    if {![info exists chats(status,$chatid)] || \
+	    $chats(status,$chatid) == "disconnected"} {
+	return 1
+    } else {
+	return 0
+    }
+
+}
+
+##############################################################################
+
 proc chat::create_user_menu {m chatid} {
     menu $m -tearoff 0
 
@@ -722,26 +811,6 @@ hook::add chat_create_conference_menu_hook chat::add_separator 40
 hook::add chat_create_conference_menu_hook chat::add_separator 42
 hook::add chat_create_conference_menu_hook chat::add_separator 50
 
-proc chat::add_chat_win_popup_menu {m chatwin X Y x y} {
-    set tags [$chatwin tag names "@$x,$y"]
-    set idx [lsearch -glob $tags url*]
-    if {$idx >= 0} {
-	$m add command -label [::msgcat::mc "Copy URL to clipboard"] \
-	    -command [list [namespace current]::copy_url $chatwin \
-			   [lindex $tags $idx]]
-    }
-}
-
-hook::add chat_win_popup_menu_hook \
-    [namespace current]::chat::add_chat_win_popup_menu 10
-
-proc chat::copy_url {chatwin tag} {
-    regsub -all %% [$chatwin tag bind $tag <ButtonPress-1><ButtonRelease-1>] % res
-    # assuming "browseurl url"
-    clipboard clear -displayof $chatwin
-    clipboard append -displayof $chatwin [lrange $res 1 end]
-}
-
 proc chat::send_message {cw chatid type} {
     set iw $cw.input
 
@@ -763,14 +832,14 @@ proc chat::send_message {cw chatid type} {
     hook::run chat_send_message_hook $chatid $user $body $type
 
     $iw delete 1.0 end
+    catch {$iw edit reset}
 }
 
 proc chat::add_message {chatid from type body x} {
     variable chats
-    variable opened
     variable options
 
-    if {[info exists opened($chatid)]} {
+    if {[is_opened $chatid]} {
 	set chatw [chat_win $chatid]
 
 	if {[lindex [$chatw yview] 1] == 1} {
@@ -784,7 +853,7 @@ proc chat::add_message {chatid from type body x} {
 
     hook::run draw_message_hook $chatid $from $type $body $x
 
-    if {[info exists opened($chatid)]} {
+    if {[is_opened $chatid]} {
 	set chatw [chat_win $chatid]
 
 	if {![$chatw compare "end -1 chars linestart" == "end -1 chars"]} {
@@ -802,163 +871,6 @@ proc chat::add_message {chatid from type body x} {
     }
 }
 
-proc chat::highlighttext {chatw tag color cursor} {
-	$chatw configure -cursor $cursor
-	$chatw tag configure $tag -foreground $color
-}
-
-proc chat::add_emoteiconed_text {chatw body defaulttag {highlightlist {}}} {
-    if {[lempty $highlightlist]} {
-	add_emoteiconed_text1 $chatw $body $defaulttag
-    } else {
-	set ind 0
-	foreach {i1 i2} $highlightlist {
-	    add_emoteiconed_text1 $chatw [crange $body $ind [expr {$i1 - 1}]] $defaulttag
-	    $chatw insert end [crange $body $i1 [expr {$i2 - 1}]] highlight
-	    set ind $i2
-	}
-	chat::add_emoteiconed_text1 $chatw [crange $body $ind end] $defaulttag
-    }
-}
-
-proc chat::encode_url {url} {
-    set utf8_url [encoding convertto utf-8 $url]
-    set len [string length $utf8_url]
-    set encoded_url ""
-    for {set i 0} {$i < $len} {incr i} {
-	binary scan $utf8_url @${i}c sym
-	set sym [expr {$sym & 0xFF}]
-	if {$sym >= 128 || $sym <= 32} {
-	    append encoded_url [format "%%%02X" $sym]
-	} else {
-	    append encoded_url [binary format c $sym]
-	}
-    }
-    return $encoded_url
-}
-
-proc chat::add_url {tw msg url args} {
-    variable urlid
-
-    set command ""
-    foreach {key val} $args {
-	switch -- $key {
-	    -command { set command $val }
-	}
-    }
-
-    set tag url$urlid
-    set urlfg    [option get $tw urlforeground       Text]
-    set urlactfg [option get $tw urlactiveforeground Text]
-    $tw tag configure $tag -foreground $urlfg -underline 1
-    $tw tag bind $tag <ButtonPress-1><ButtonRelease-1> \
-	[list browseurl [double% [encode_url $url]]]
-    if {$command != ""} {
-	$tw tag bind $tag <ButtonPress-1><ButtonRelease-1> \
-	    +[list eval $command]
-    }
-    $tw tag bind $tag <Any-Enter> \
-	[list chat::highlighttext $tw $tag $urlactfg hand2]
-    $tw tag bind $tag <Any-Leave> \
-	[list chat::highlighttext $tw $tag $urlfg xterm]
-	    
-    $tw insert end $msg $tag
-    incr urlid
-}
-
-proc chat::on_browse_url {chatid w x y} {
-    set tags [$w tag names "@$x,$y"]
-    set idx [lsearch -glob $tags url*]
-    if {$idx >= 0} {
-	return stop
-    }
-    return
-}
-
-hook::add chat_window_click_hook chat::on_browse_url 10
-	
-proc chat::add_emoteiconed_text1 {chatw body defaulttag} {
-    variable url_regexp
-    variable options
-
-    set num_of_emoticons 0
-    set words [textutil::splitx $body {([\t \r\n]+)}]
-
-    foreach word $words {
-	if {$num_of_emoticons < 200} {
-	    if {[emoteicons::get $word] != ""} {
-		emoteicons::put $chatw $word
-		incr num_of_emoticons
-		continue
-	    }
-	}
-	if {[regexp -expanded -nocase -- $url_regexp $word ignore head url ignore2 tail]} {
-	    $chatw insert end $head $defaulttag
-	    add_url $chatw $url $url
-	    $chatw insert end $tail $defaulttag
-	} else {
-	    if {0 && !$options(emphasize)} {
-		$chatw insert end $word $defaulttag
-	    } else {
-		set enrichedText(bold) 0
-		set enrichedText(italic) 0
-		set enrichedText(underline) 0
-
-		set tmp $word
-		while {[regexp -- "^(_|/|\\*)(.+)(_|/|\\*)$" $tmp wholeMatch e1 subWord e2]} {
-		    if { $e1 != $e2 } {
-			break
-		    }
-		    if { "*" == $e1 } {
-			if {$enrichedText(bold)} break
-			set enrichedText(bold) 1
-		    } elseif { "/" == $e1 } {
-			if {$enrichedText(italic)} break
-			set enrichedText(italic) 1
-		    } elseif { "_" == $e1 } {
-			if {$enrichedText(underline)} break
-			set enrichedText(underline) 1
-		    }
-		    set tmp $subWord
-		}
-
-		set tag $defaulttag
-		switch -- $enrichedText(bold),$enrichedText(italic) {
-		    1,1 {lappend tag bold_italic}
-		    1,0 {lappend tag bold}
-		    0,1 {lappend tag italic}
-		}
-		if {$enrichedText(underline) == 1} {
-		    lappend tag underlined
-		}
-		if {$tmp != $word} {
-		    lappend tag emphasized
-		    $chatw insert end $tmp $tag
-		    $chatw insert end $word [concat nonemphasized $defaulttag]
-		} else {
-		    $chatw insert end $word $defaulttag
-		}
-	    }
-	}
-    }
-}
-
-proc chat::switch_emphasize {args} {
-    variable options
-    variable opened
-
-    foreach chatid [array names opened] {
-	set cw [chat_win $chatid]
-	if {$options(emphasize)} {
-	    $cw tag configure emphasized -elide 0
-	    $cw tag configure nonemphasized -elide 1
-	} else {
-	    $cw tag configure emphasized -elide 1
-	    $cw tag configure nonemphasized -elide 0
-	}
-    }
-}
-
 proc chat::add_open_to_user_menu_item {m connid jid} {
     $m add command -label [::msgcat::mc "Start chat"] \
 	-command [list chat::open_to_user $connid $jid]
@@ -1020,7 +932,7 @@ proc chat::open_to_user {connid user args} {
 proc chat::change_presence {connid jid type x args} {
     global grouproster
     variable chats
-    variable opened
+    variable options
     global statusdesc
 
     switch -- $type {
@@ -1039,14 +951,14 @@ proc chat::change_presence {connid jid type x args} {
 
     set group [node_and_server_from_jid $jid]
     set chatid [chatid $connid $group]
-    set nick [get_nick $connid $jid groupchat]
 
-    if {[info exists opened($chatid)]} {
+    if {[is_opened $chatid]} {
 	if {[is_groupchat $chatid]} {
 	    debugmsg chat "ST: $connid $jid $status"
 	    if {[resource_from_jid $jid] == ""} {
 		return
 	    }
+	    set nick [get_nick $connid $jid groupchat]
 	    if {[cequal $status unavailable] || [cequal $status error]} {
 		if {$status == "error" && [is_our_jid $chatid $jid]} {
 		    add_message $chatid $group error \
@@ -1086,6 +998,9 @@ proc chat::change_presence {connid jid type x args} {
 
 		    muc::process_available $chatid $nick
 		    hook::run chat_user_enter $chatid $nick
+		    muc::report_available $chatid $nick 1
+		} else {
+		    muc::report_available $chatid $nick 0
 		}
 		set grouproster(status,$chatid,$jid) $status
 		ifacetk::roster::changeicon $userswin $jid roster/user/$status
@@ -1095,10 +1010,25 @@ proc chat::change_presence {connid jid type x args} {
 	}
     }
 
+    # This case traces both chat and private groupchat conversations:
+    if {$options(gen_status_change_msgs)} {
+	set chatid [chatid $connid $jid]
+	if {[is_opened $chatid]} {
+	    set msg [get_nick $connid $jid chat]
+	    append msg " " [::get_long_status_desc [::get_user_status $connid $jid]]
+	    set desc [::get_user_status_desc $connid $jid]
+	    if {$desc != {}} {
+		append msg " ($desc)"
+	    }
+	    ::chat::add_message $chatid "" chat $msg {}
+	}
+    }
+
     set cw [winid [chatid $connid $jid]]
 
     if {[winfo exists $cw.status.icon]} {
-	$cw.status.icon configure -image roster/user/$status \
+	$cw.status.icon configure \
+	    -image [ifacetk::roster::get_jid_icon $connid $jid $status] \
 	    -helptext [get_user_status_desc $connid $jid]
     }
 
@@ -1112,7 +1042,9 @@ proc chat::change_presence {connid jid type x args} {
 
     if {[winfo exists $cw.status.icon]} {
 	$cw.status.icon configure \
-	    -image roster/user/[get_user_status $connid $user]
+	    -image [ifacetk::roster::get_jid_icon \
+			$connid $user [get_user_status $connid $user]] \
+	    -helptext [get_user_status_desc $connid $user]
     }
 }
 
@@ -1132,9 +1064,8 @@ proc chat::redraw_roster {group} {
     set levels {}
     foreach jid $grouproster(users,$group) {
 	if {[info exists ::muc::users(role,$connid,$jid)]} {
-	    if {$::muc::users(role,$connid,$jid) == "" && \
-		    $::muc::users(affiliation,$connid,$jid) == ""} {
-		set level "aGreen mans"
+	    if {$::muc::users(role,$connid,$jid) == ""} {
+		set level a[::msgcat::mc "Users"]
 		lappend levels $level
 		lappend levelusers($level) $jid
 	    } else {
@@ -1164,7 +1095,7 @@ proc chat::redraw_roster {group} {
 		#}
 	    }
 	} else {
-	    set level 2[::msgcat::mc "Users"]
+	    set level 8[::msgcat::mc "Users"]
 	    lappend levels $level
 	    lappend levelusers($level) $jid
 	}	    
@@ -1216,24 +1147,22 @@ proc chat::redraw_roster_after_idle {group} {
 }
 
 proc chat::restore_subject {chatid} {
-    variable opened
     variable chats
 
-    set cw [winid $chatid]
+    set sw [winid $chatid].status.subject
 
-    if {[info exists opened($chatid)]} {
-	$cw.status.subject delete 0 end
-	$cw.status.subject insert 0 $chats(subject,$chatid)
+    if {[is_opened $chatid] && [winfo exists $sw]} {
+	$sw delete 0 end
+	$sw insert 0 $chats(subject,$chatid)
     }
 }
 
 proc chat::set_subject {chatid subject} {
-    variable opened
     variable chats
 
     set cw [winid $chatid]
 
-    if {[info exists opened($chatid)]} {
+    if {[is_opened $chatid]} {
 	$cw.status.subject delete 0 end
 	$cw.status.subject insert 0 $subject
 	set chats(subject,$chatid) $subject
@@ -1253,10 +1182,14 @@ proc chat::change_subject {chatid} {
 
 proc chat::set_subject_balloon {chatid} {
     variable chats
+
+    set sw [winid $chatid].status.subject
+
     if {[info exists chats(subject_tooltip,$chatid)]} {
-	return $chats(subject_tooltip,$chatid)
+	return [list $chatid $chats(subject_tooltip,$chatid) \
+		     -width [winfo width $sw]]
     } else {
-	return ""
+	return [list $chatid ""]
     }
 }
 
@@ -1330,7 +1263,6 @@ hook::add chat_create_conference_menu_hook \
 
 proc chat::invite_dialog {user {ignore_muc 0} args} {
     variable chats
-    variable opened
     global invite_gc
 
     foreach {opt val} $args {
@@ -1363,11 +1295,9 @@ proc chat::invite_dialog {user {ignore_muc 0} args} {
 
     set titles {}
     set jids {}
-    foreach chatid [lsort [array names opened]] {
-        if {([get_connid $chatid] == $connid) && [is_groupchat $chatid]} {
-	    lappend jids $chatid [get_jid $chatid]
-            lappend titles $chatid [node_from_jid [get_jid $chatid]]
-	}
+    foreach chatid [lsort [lfilter [namespace current]::is_groupchat [opened $connid]]] {
+	lappend jids $chatid [get_jid $chatid]
+        lappend titles $chatid [node_from_jid [get_jid $chatid]]
     }
     if {[llength $titles] == 0} {
         MessageDlg ${gw}_err -aspect 50000 -icon info \
@@ -1388,7 +1318,6 @@ proc chat::invite_dialog {user {ignore_muc 0} args} {
 
 proc chat::invite_dialog2 {jid ignore_muc args} {
     variable chats
-    variable opened
     global invite_gc
 
     foreach {opt val} $args {
@@ -1444,7 +1373,7 @@ proc chat::invitation {jid usersP ignore_muc {reason ""}} {
 		set group $gc
 	    }
 	    if {[cequal $reason ""]} {
-		set reas [format [::msgcat::mc "Please join %s"] $group]
+		set reas [::msgcat::mc "Please join %s" $group]
 	    } else {
 		set reas $reason
 	    }
@@ -1457,3 +1386,61 @@ proc chat::invitation {jid usersP ignore_muc {reason ""}} {
     }
 }
 
+#############################################################################
+
+proc chat::restore_window {cjid type nick connid jid} {
+    set chatid [chatid $connid $cjid]
+
+    if {$type == "groupchat"} {
+	set_our_groupchat_nick $chatid $nick
+    }
+
+    # TODO: Password?
+    open_window $chatid $type
+}
+
+#############################################################################
+
+proc chat::save_session {vsession} {
+    upvar 2 $vsession session
+    global usetabbar
+    variable chats
+    variable chat_id
+
+    # TODO
+    if {!$usetabbar} return
+
+    set prio 0
+    foreach page [.nb pages] {
+	set path [ifacetk::nbpath $page]
+
+	if {[info exists chat_id($path)]} {
+	    set chatid $chat_id($path)
+
+	    set connid [get_connid $chatid]
+	    set jid [get_jid $chatid]
+	    set type $chats(type,$chatid)
+
+	    if {$type == "groupchat"} {
+		set nick [get_our_groupchat_nick $chatid]
+	    } else {
+		set nick ""
+	    }
+
+	    set user [jlib::connection_requested_user $connid]
+	    set server [jlib::connection_requested_server $connid]
+	    set resource [jlib::connection_requested_resource $connid]
+
+	    lappend session [list $prio $user $server $resource \
+		[list [namespace current]::restore_window $jid $type $nick] \
+	    ]
+	}
+	incr prio
+    }
+}
+
+hook::add save_session_hook [namespace current]::chat::save_session
+
+#############################################################################
+
+# vim:ts=8:sw=4:sts=4:noet
