@@ -1,11 +1,11 @@
-# $Id: custom.tcl,v 1.26 2006/05/11 22:07:25 aleksey Exp $
+# $Id: custom.tcl 921 2007-02-11 10:27:18Z sergei $
 
 option add *Customize.varforeground       blue widgetDefault
 option add *Customize.groupnameforeground blue widgetDefault
 
 namespace eval custom {
     # Filename for saving options
-    set options(customfile) [file join ~ .tkabber custom.tcl]
+    set options(customfile) [file join $::configdir custom.tcl]
 
     # -1: stored values haven't been restored yet (only config changes vars)
     # 0: stored values are being restored now
@@ -35,7 +35,7 @@ proc custom::defgroup {id doc args} {
 		lappend group(subgroups,$val) [list group $id]
 		set group(subgroups,$val) [lrmdups $group(subgroups,$val)]
 		lappend group(parents,$id) $val
-		set group(parents,$val) [lrmdups $group(parents,$val)]
+		set group(parents,$id) [lrmdups $group(parents,$id)]
 		#set group(members,$val) [lrmdups $group(members,$val)]
 	    }
 	    -type {
@@ -65,6 +65,54 @@ proc custom::defvar {vname value doc args} {
     set var(type,$fullname) string
     set var(state,$fullname) ""
 
+    eval { configvar $fullname } $args
+}
+
+proc custom::on_var_change {varname args} {
+    variable options
+    variable var
+    variable custom_loaded
+
+    switch -- $custom_loaded {
+	-1 {
+	    set var(config,$varname) [set $varname]
+	}
+	0 { }
+	1 {
+	    # Store variable if it has been changed by
+	    # any procedure which is not in ::custom namespace
+	    if {[namespace qualifiers [caller]] != [namespace current]} {
+		# Don't store loginconf here
+		# (storing all loginconf except password may be
+		# confusing)
+		if {![regexp {^(::)+loginconf\(.*\)} $varname]} {
+		    store_vars $varname
+		}
+	    }
+	}
+    }
+}
+
+proc custom::add_radio_options {vname values} {
+    variable var
+    
+    set fullname [uplevel 1 {namespace current}]::$vname
+
+    if {![info exists $fullname]} {
+	return
+    }
+
+    set var(values,$fullname) [concat $var(values,$fullname) $values]
+}
+
+proc custom::configvar {fullname args} {
+    variable var
+    variable group
+
+    if {![info exists $fullname]} {
+	error "No such variable: $fullname"
+    }
+
     foreach {attr val} $args {
 	switch -- $attr {
 	    -type {
@@ -82,6 +130,7 @@ proc custom::defvar {vname value doc args} {
 	    }
 	}
     }
+	
     switch -- $var(type,$fullname) {
 	radio {
 	    set q 0
@@ -95,6 +144,7 @@ proc custom::defvar {vname value doc args} {
 	    }
 	}
     }
+	
     foreach {attr val} $args {
 	switch -- $attr {
 	    -command {
@@ -104,32 +154,6 @@ proc custom::defvar {vname value doc args} {
     }
 }
 
-proc custom::on_var_change {varname args} {
-    variable options
-    variable var
-    variable custom_loaded
-
-    switch -- $custom_loaded {
-	-1 {
-	    set var(config,$varname) [set $varname]
-	}
-	0 { }
-	1 { }
-    }
-}
-
-proc custom::add_radio_options {vname values} {
-    variable var
-    
-    set fullname [uplevel 1 {namespace current}]::$vname
-
-    if {![info exists $fullname]} {
-	return
-    }
-
-    set var(values,$fullname) [concat $var(values,$fullname) $values]
-}
-
 custom::defgroup Tkabber \
     [::msgcat::mc "Customization of the One True Jabber Client."]
 
@@ -143,7 +167,12 @@ proc custom::open_window {gid} {
 
     set w .customize
     if {[winfo exists $w]} {
-	return
+	if {$::usetabbar} {
+	    .nb raise [::ifacetk::nbpage $w]
+	}
+	goto $gid
+	focus $w.fields
+	return    
     }
 
     add_win $w -title [::msgcat::mc "Customize"] \
@@ -196,6 +225,8 @@ proc custom::open_window {gid} {
 
     variable curgroup $gid
 
+    hook::run open_custom_post_hook $w
+
     update idletasks
     goto $gid
 
@@ -204,16 +235,15 @@ proc custom::open_window {gid} {
 
 proc custom::go {} {
     variable curgroup
-    history_add $curgroup
-    fill_group .customize.fields $curgroup
+    goto $curgroup
 }
 
 proc custom::goto {gid} {
     history_add $gid
-    fill_group .customize.fields $gid
+    fill_group .customize.fields $gid 0
 }
 
-proc custom::fill_group {t gid} {
+proc custom::fill_group {t gid offset} {
     variable group
     variable var
     variable curgroup
@@ -252,7 +282,7 @@ proc custom::fill_group {t gid} {
 
     set butwidth 0
     foreach member [concat $group(members,$gid) \
-			[lsort -dictionary $group(subgroups,$gid)]] {
+			[lsort -dictionary -index 1 $group(subgroups,$gid)]] {
 	lassign $member type data
 	switch -- $type {
 	    group {
@@ -295,6 +325,7 @@ proc custom::fill_group {t gid} {
     }
 
     $t configure -state disabled
+    $t yview moveto $offset
 }
 
 proc custom::fill_var {t varname idx} {
@@ -343,12 +374,35 @@ proc custom::fill_var {t varname idx} {
 	    trace variable [namespace current]::tmp($varname) w \
 		[list [namespace current]::on_edit $varname]
 	    set tmp($varname) [set $varname]
-	    set e [Spinbox $t.spin$idx -1000000000 1000000000 1 [namespace current]::tmp($varname)]
+	    set e [Spinbox $t.spin$idx -1000000000 1000000000 1 \
+			   [namespace current]::tmp($varname)]
 	    $t window create end -window $e
 	    bindscroll $e $t
 	    $t insert end "\n"
 	}
 
+	options {
+	    catch {unset tmp($varname)}
+	    catch {unset var(temp,$varname)}
+	    trace variable [namespace current]::tmp($varname) w \
+		[list [namespace current]::on_edit $varname]
+	    trace variable [namespace current]::var(temp,$varname) w \
+		[list [namespace current]::on_change $t $varname]
+	    set var(temp,$varname) [set $varname]
+	    set tmp($varname) [set $varname]
+	    set options {}
+	    foreach {val text} $var(values,$varname) {
+		lappend options $text
+	    }
+	    set opt [eval [list OptionMenu $t.opt$idx \
+				[namespace current]::var(temp,$varname)] \
+			  $options]
+	    $t.opt$idx configure -cursor left_ptr
+	    $t window create end -window $t.opt$idx
+	    bindscroll $t.opt$idx $t
+	    $t insert end "\n"
+	}
+
 	list {
 	    if {![info exists var(values,$varname)]} return
 
@@ -465,7 +519,7 @@ proc custom::get_filename {varname} {
 
     set args {}
     if {$tmp($varname) == ""} {
-	lappend args -initialdir ~/.tkabber/
+	lappend args -initialdir $::configdir
     } else {
 	lappend args -initialdir [file dirname $tmp($varname)] \
 	    -initialfile [file tail $tmp($varname)]
@@ -494,6 +548,16 @@ proc custom::on_change {w varname args} {
 		$w selection set $i
 	    }
 	}
+	options {
+	    foreach {val text} $var(values,$varname) {
+		if {$text == $var(temp,$varname) && \
+			(![info exists tmp($varname)] || \
+			 $tmp($varname) != $val)} {
+		    set tmp($varname) $val
+		    break
+		}
+	    }
+	}
     }
 }
 
@@ -508,6 +572,19 @@ proc custom::on_edit {varname args} {
     variable tmp
     variable saved
 
+    switch -- $var(type,$varname) {
+	options {
+	    foreach {val text} $var(values,$varname) {
+		if {$tmp($varname) == $val && \
+			(![info exists var(temp,$varname)] || \
+			 $var(temp,$varname) != $text)} {
+		    set var(temp,$varname) $text
+		    break
+		}
+	    }
+	}
+    }
+
     set is_default [cequal [set $varname] $var(default,$varname)]
     if {[info exists var(config,$varname)]} {
 	set is_config [cequal [set $varname] $var(config,$varname)]
@@ -547,7 +624,7 @@ proc custom::create_state_menu {m varname} {
 	destroy $m
     }
 
-    menu $m
+    menu $m -tearoff 0
     $m add command -label [::msgcat::mc "Set for current session only"] \
 	-command [list [namespace current]::set_for_current_sess $varname]
     $m add command -label [::msgcat::mc "Set for current and future sessions"] \
@@ -658,7 +735,17 @@ proc custom::store {} {
     }
 
     close $fd
-    catch {file attributes ~/.tkabber/custom.tcl -permissions 00600}
+    catch {file attributes [file join $::configdir custom.tcl] -permissions 00600}
+}
+
+proc custom::store_vars {args} {
+    variable saved
+
+    foreach varname $args {
+	set saved($varname) [set $varname]
+    }
+
+    store
 }
 
 proc custom::restore {} {
@@ -689,9 +776,20 @@ proc custom::restore {} {
 
     set custom_loaded 1
 }
+
 hook::add postload_hook custom::restore 60
 
 
+proc custom::update_page_offset {} {
+    variable history
+
+    lassign [.customize.fields yview] offset
+    lassign [lindex $history(list) $history(pos)] page
+
+    set history(list) [lreplace $history(list) $history(pos) $history(pos) \
+				[list $page $offset]]
+}
+
 proc custom::history_move {shift} {
     variable history
     variable curgroup
@@ -706,25 +804,63 @@ proc custom::history_move {shift} {
 	return
     }
 
-    set newgroup [lindex $history(list) $newpos]
+    update_page_offset
+
+    lassign [lindex $history(list) $newpos] newgroup offset
     set history(pos) $newpos
     
     set curgroup $newgroup
 
-    fill_group .customize.fields $newgroup
+    fill_group .customize.fields $newgroup $offset
 }
 
 
 proc custom::history_add {gid} {
     variable history
 
-    set history(list) [lreplace $history(list) 0 \
-			   [expr {$history(pos) - 1}]]
+    update_page_offset
+
+    set history(list) [lreplace $history(list) 0 [expr {$history(pos) - 1}]]
     
-    lvarpush history(list) $gid
+    lvarpush history(list) [list $gid 0]
     set history(pos) 0
 
     debugmsg custom [array get history]
 }
 
+##############################################################################
 
+proc custom::restore_window {gid args} {
+    open_window $gid
+}
+
+proc custom::save_session {vsession} {
+    upvar 2 $vsession session
+    global usetabbar
+    variable history
+
+    # We don't need JID at all, so make it empty (special case)
+    set user     ""
+    set server   ""
+    set resource ""
+
+    # TODO
+    if {!$usetabbar} return
+
+    set prio 0
+    foreach page [.nb pages] {
+	set path [ifacetk::nbpath $page]
+
+	if {[string equal $path .customize]} {
+	    lassign [lindex $history(list) $history(pos)] gid
+	    lappend session [list $prio $user $server $resource \
+		[list [namespace current]::restore_window $gid] \
+	    ]
+	}
+	incr prio
+    }
+}
+
+hook::add save_session_hook [namespace current]::custom::save_session
+
+# vim:ts=8:sw=4:sts=4:noet
