@@ -1,4 +1,6 @@
-# $Id: disco.tcl,v 1.42 2006/03/17 04:29:42 aleksey Exp $
+# $Id: disco.tcl 1038 2007-03-10 14:09:40Z sergei $
+
+##############################################################################
 
 set ::NS(disco_items) "http://jabber.org/protocol/disco#items"
 set ::NS(disco_info)  "http://jabber.org/protocol/disco#info"
@@ -11,8 +13,12 @@ option add *JDisco.optioncolor   DarkViolet     widgetDefault
 namespace eval disco {
     variable supported_nodes
     variable supported_features {}
+    variable root_nodes {}
+    variable additional_items
 }
 
+##############################################################################
+
 proc disco::request_items {jid node args} {
     variable disco
 
@@ -27,15 +33,15 @@ proc disco::request_items {jid node args} {
 	}
     }
     if {![info exists connid]} {
-	set connid [jlib::route $jid]
+	return -code error "disco::request_items: -connection is mandatory"
     }
 
     switch -- $cache {
 	first -
 	only -
 	yes {
-	    if {[info exists disco(items,$jid,$node)]} {
-		set items $disco(items,$jid,$node)
+	    if {[info exists disco(items,$connid,$jid,$node)]} {
+		set items $disco(items,$connid,$jid,$node)
 		if {$handler != ""} {
 		    eval $handler [list OK $items]
 		}
@@ -58,17 +64,18 @@ proc disco::request_items {jid node args} {
 		 -vars $vars] \
 	-to $jid \
 	-connection $connid \
-	-command [list [namespace current]::parse_items $jid $node $handler]
+	-command [list [namespace current]::parse_items \
+		       $connid $jid $node $handler]
 }
 
-proc disco::parse_items {jid node handler res child} {
+proc disco::parse_items {connid jid node handler res child} {
     variable disco
 
     if {![cequal $res OK]} {
 	if {$handler != ""} {
 	    eval $handler [list ERR $child]
 	}
-	hook::run disco_items_hook $jid $node ERR $child
+	hook::run disco_items_hook $connid $jid $node ERR $child
 	return
     }
 
@@ -84,12 +91,12 @@ proc disco::parse_items {jid node handler res child} {
 		set inode  [jlib::wrapper:getattr $vars1 node]
 		set name  [jlib::wrapper:getattr $vars1 name]
 		lappend items [list jid $ijid node $inode name $name]
-		set disco(jidname,$ijid,$inode) $name
+		set disco(jidname,$connid,$ijid,$inode) $name
 	    }
 	}
     }
 
-    set disco(items,$jid,$node) $items
+    set disco(items,$connid,$jid,$node) $items
 
     debugmsg disco "ITEMS: [list $items]"
 
@@ -97,10 +104,10 @@ proc disco::parse_items {jid node handler res child} {
 	eval $handler [list OK $items]
     }
 
-    hook::run disco_items_hook $jid $node OK $items
+    hook::run disco_items_hook $connid $jid $node OK $items
 }
 
-
+##############################################################################
 
 proc disco::request_info {jid node args} {
     variable disco
@@ -116,18 +123,22 @@ proc disco::request_info {jid node args} {
 	}
     }
     if {![info exists connid]} {
-	set connid [jlib::route $jid]
+	return -code error "disco::request_items: -connection is mandatory"
     }
 
+    # disco(info,featured_nodes,$connid,$jid,$node) isn't cached because it
+    # isn't really reported. It's for internal use only.
+    set disco(info,featured_nodes,$connid,$jid,$node) {}
+
     switch -- $cache {
 	first -
 	only -
 	yes {
-	    if {[info exists disco(info,identities,$jid,$node)] && \
-		    [info exists disco(info,identities,$jid,$node)]} {
-		set identities $disco(info,identities,$jid,$node)
-		set features   $disco(info,features,$jid,$node)
-		set extras     $disco(info,extras,$jid,$node)
+	    if {[info exists disco(info,identities,$connid,$jid,$node)] && \
+		    [info exists disco(info,identities,$connid,$jid,$node)]} {
+		set identities $disco(info,identities,$connid,$jid,$node)
+		set features   $disco(info,features,$connid,$jid,$node)
+		set extras     $disco(info,extras,$connid,$jid,$node)
 		if {$handler != ""} {
 		    eval $handler [list OK $identities $features $extras]
 		}
@@ -150,23 +161,26 @@ proc disco::request_info {jid node args} {
 	     -vars $vars] \
 	-to $jid \
 	-connection $connid \
-	-command [list [namespace current]::parse_info $jid $node $handler]
+	-command [list [namespace current]::parse_info \
+		       $connid $jid $node $handler]
 }
 
-proc disco::parse_info {jid node handler res child} {
+proc disco::parse_info {connid jid node handler res child} {
     variable disco
+    variable additional_nodes
 
     if {![cequal $res OK]} {
 	if {$handler != ""} {
 	    eval $handler [list ERR $child {} {}]
 	}
-	hook::run disco_info_hook $jid $node ERR $child {} {}
+	hook::run disco_info_hook $connid $jid $node ERR $child {} {} {}
 	return
     }
 
     set identities {}
     set features {}
     set extras {}
+    set featured_nodes {}
 
     jlib::wrapper:splitxml $child tag vars isempty chdata children
 
@@ -186,6 +200,15 @@ proc disco::parse_info {jid node handler res child} {
 		    set var [jlib::wrapper:getattr $vars1 type]
 		}
 		lappend features [list var $var]
+		if {($node == "") && [info exists additional_nodes($var)]} {
+		    lappend featured_nodes \
+			    [concat [list jid $jid] $additional_nodes($var)]
+		    set inode [jlib::wrapper:getattr $additional_nodes($var) node]
+		    set iname [jlib::wrapper:getattr $additional_nodes($var) name]
+		    if {![info exists disco(jidname,$connid,$jid,$inode)]} {
+			set disco(jidname,$connid,$jid,$inode) $iname
+		    }
+		}
 	    }
 	    default {
 		if {[jlib::wrapper:getattr $vars1 xmlns] == "jabber:x:data" && \
@@ -196,33 +219,46 @@ proc disco::parse_info {jid node handler res child} {
 	}
     }
 
-    set disco(info,identities,$jid,$node) $identities
-    set disco(info,features,$jid,$node) $features
-    set disco(info,extras,$jid,$node) $extras
+    set disco(info,identities,$connid,$jid,$node) $identities
+    set disco(info,features,$connid,$jid,$node) $features
+    set disco(info,extras,$connid,$jid,$node) $extras
+    set disco(info,featured_nodes,$connid,$jid,$node) [lrmdups $featured_nodes]
 
     debugmsg disco \
 	"INFO: IDENTITIES [list $identities] FEATURES [list $features]\
-	 EXTRAS [list $extras]"
+	 EXTRAS [list $extras] FEATURED NODES [list [lrmdups $featured_nodes]]"
 
     if {$handler != ""} {
 	eval $handler [list OK $identities $features $extras]
     }
-    hook::run disco_info_hook $jid $node OK $identities $features $extras
+    hook::run disco_info_hook $connid $jid $node OK $identities $features \
+			      $extras [lrmdups $featured_nodes]
 }
 
-proc disco::get_jid_name {jid node} {
+###############################################################################
+
+proc disco::get_jid_name {connid jid node} {
     variable disco
-    if {[info exists disco(jidname,$jid,$node)]} {
-	return $disco(jidname,$jid,$node)
+    if {[info exists disco(jidname,$connid,$jid,$node)]} {
+	return $disco(jidname,$connid,$jid,$node)
     } else {
 	return ""
     }
 }
 
-proc disco::get_jid_identities {jid node} {
+proc disco::get_jid_identities {connid jid node} {
     variable disco
-    if {[info exists disco(info,identities,$jid,$node)]} {
-	return $disco(info,identities,$jid,$node)
+    if {[info exists disco(info,identities,$connid,$jid,$node)]} {
+	return $disco(info,identities,$connid,$jid,$node)
+    } else {
+	return {}
+    }
+}
+
+proc disco::get_jid_items {connid jid node} {
+    variable disco
+    if {[info exists disco(items,$connid,$jid,$node)]} {
+	return $disco(items,$connid,$jid,$node)
     } else {
 	return {}
     }
@@ -230,7 +266,15 @@ proc disco::get_jid_identities {jid node} {
 
 ###############################################################################
 
-proc disco::info_query_get_handler {connid from child} {
+proc disco::register_featured_node {feature node name} {
+    variable additional_nodes
+
+    set additional_nodes($feature) [list node $node name $name]
+}
+
+###############################################################################
+
+proc disco::info_query_get_handler {connid from lang child} {
     variable supported_nodes
     variable node_handlers
     variable supported_features
@@ -243,7 +287,7 @@ proc disco::info_query_get_handler {connid from child} {
 	if {![info exists supported_nodes($node)]} {
 	    return {error cancel not-allowed}
 	} else {
-	    set restags [$node_handlers($node) info $connid $from $child]
+	    set restags [$node_handlers($node) info $connid $from $lang $child]
 	    if {[cequal [lindex $restags 0] error]} {
 		return $restags
 	    } else {
@@ -281,9 +325,10 @@ iq::register_handler get query $::NS(disco_info) \
 
 ###############################################################################
 
-proc disco::items_query_get_handler {connid from child} {
+proc disco::items_query_get_handler {connid from lang child} {
     variable supported_nodes
     variable node_handlers
+    variable root_nodes
 
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set node [jlib::wrapper:getattr $vars node]
@@ -292,7 +337,7 @@ proc disco::items_query_get_handler {connid from child} {
 	if {![info exists supported_nodes($node)]} {
 	    return {error cancel not-allowed}
 	} else {
-	    set restags [$node_handlers($node) items $connid $from $child]
+	    set restags [$node_handlers($node) items $connid $from $lang $child]
 	    if {[cequal [lindex $restags 0] error]} {
 		return $restags
 	    } else {
@@ -304,18 +349,12 @@ proc disco::items_query_get_handler {connid from child} {
     } else {
 	set restags {}
 
-	set bare_from [node_and_server_from_jid $from]
-	set chatid [chat::chatid $connid $bare_from]
-	if {[chat::is_groupchat $chatid]} {
-	    set myjid [chat::our_jid $chatid]
-	} else {
-	    set myjid [jlib::connection_jid $connid]
-	}
+	set myjid [my_jid $connid $from]
 
-	foreach node [array names supported_nodes] {
+	foreach node $root_nodes {
 	    set vars [list jid $myjid]
 	    if {![cequal $supported_nodes($node) ""]} {
-		lappend vars name $supported_nodes($node)
+		lappend vars name [::trans::trans $lang $supported_nodes($node)]
 	    }
 	    if {![cequal $node ""]} {
 		lappend vars node $node
@@ -347,6 +386,15 @@ proc disco::register_feature {feature {handler ""}} {
 ###############################################################################
 
 proc disco::register_node {node handler {name ""}} {
+    variable root_nodes
+
+    lappend root_nodes $node
+    register_subnode $node $handler $name
+}
+
+###############################################################################
+
+proc disco::register_subnode {node handler {name ""}} {
     variable supported_nodes
     variable node_handlers
 
@@ -371,7 +419,7 @@ proc disco::publish_items {jid node action items args} {
 
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $::NS(disco#publish) node $node]
+	     -vars [list xmlns $::NS(disco#publish) node $node] \
 	     -subtags $items] \
 	-to $jid \
 	-connection $connid \
@@ -393,12 +441,21 @@ namespace eval disco::browser {
 
     image create photo ""
 
+    variable options
+
+    # Do not show items number in node title if this number
+    # is not greater than 20
+    # (It is questionnable whether to add this option to Customize).
+    set options(upper_items_bound) 20
+
     custom::defvar disco_list {} [::msgcat::mc "List of discovered JIDs."] \
 	    -group Hidden
     custom::defvar node_list {} [::msgcat::mc "List of discovered JID nodes."] \
 	    -group Hidden
 }
 
+###############################################################################
+
 proc disco::browser::open_win {jid args} {
     variable winid
     variable disco
@@ -440,6 +497,8 @@ proc disco::browser::open_win {jid args} {
     set config(identitycolor) [option get $w identitycolor JDisco]
     set config(optioncolor)   [option get $w optioncolor   JDisco]
 
+    bind $w <Destroy> [list [namespace current]::destroy_state $w]
+
     frame $w.navigate
     button $w.navigate.back -text <- \
 	-command [list [namespace current]::history_move $w 1]
@@ -470,7 +529,7 @@ proc disco::browser::open_win {jid args} {
     set sw [ScrolledWindow $w.sw]
 
     set tw [Tree $w.tree -deltax 16 -deltay 18 -dragenabled 1 \
-		-draginitcmd [namespace current]::draginitcmd]
+		-draginitcmd [list [namespace current]::draginitcmd $w]]
     $sw setwidget $tw
 
     pack $sw -side top -expand yes -fill both
@@ -479,12 +538,7 @@ proc disco::browser::open_win {jid args} {
 	[list [namespace current]::textaction $w]
     $tw bindText <ButtonPress-3> \
 	[list [namespace current]::textpopup $w]
-    $tw bindText <Any-Enter>  \
-	[list [namespace current]::textballoon $w enter  %X %Y]
-    $tw bindText <Any-Motion> \
-	[list [namespace current]::textballoon $w motion %X %Y]
-    $tw bindText <Any-Leave>  \
-	[list [namespace current]::textballoon $w leave  %X %Y]
+    balloon::setup $tw -command [list [namespace current]::textballoon $w]
     bindscroll $tw.c
 
     # HACK
@@ -502,6 +556,8 @@ proc disco::browser::open_win {jid args} {
     set browser(hist,$w) {}
     set browser(histpos,$w) 0
 
+    hook::run open_disco_post_hook $w $sw $tw
+
     go $w
 }
 
@@ -517,29 +573,28 @@ proc disco::browser::go {bw} {
 	history_add $bw [list $jid $node]
 
         set disco_list [update_combo_list $disco_list $jid 20]
-	$bw.navigate.entry configure -values $disco_list
-	set custom::saved([namespace current]::disco_list) $disco_list
         set node_list [update_combo_list $node_list $node 20]
+	$bw.navigate.entry configure -values $disco_list
 	$bw.navigate.node configure -values $node_list
-	set custom::saved([namespace current]::node_list) $node_list
-	custom::store
 
 	lappend browser(required,$bw) $jid
 	set browser(required,$bw) [lrmdups $browser(required,$bw)]
 
-	disco::request_items $jid $node -connection $browser(connid,$bw)
 	disco::request_info $jid $node -connection $browser(connid,$bw)
+	disco::request_items $jid $node -connection $browser(connid,$bw)
     }
 }
 
-proc disco::browser::info_receive {jid node res identities features extras} {
+proc disco::browser::info_receive \
+     {connid jid node res identities features extras featured_nodes} {
     variable browser
 
     if {![info exists browser(opened)]} return
 
     foreach w $browser(opened) {
 	if {[winfo exists $w] && [lcontain $browser(required,$w) $jid]} {
-	    draw_info $w $jid $node $res $identities $features $extras
+	    draw_info $w $connid $jid $node $res $identities \
+		      $features $extras $featured_nodes
 	}
     }
 }
@@ -547,18 +602,24 @@ proc disco::browser::info_receive {jid node res identities features extras} {
 hook::add disco_info_hook \
     [namespace current]::disco::browser::info_receive
 
-proc disco::browser::draw_info {w jid node res identities features extras} {
+proc disco::browser::draw_info \
+     {w connid jid node res identities features extras featured_nodes} {
     variable browser
     variable config
     global font
 
     set tw $browser(tree,$w)
 
-    set name [disco::get_jid_name $jid $node]
+    set name [disco::get_jid_name $connid $jid $node]
     set tnode [jid_to_tag [list $jid $node]]
     set parent_tag [jid_to_tag [list $jid $node]]
-    set data [list item $jid $node]
-    set desc [item_desc $jid $node $name]
+    set data [list item $connid $jid $node]
+    if {![$tw exists $tnode] || [llength [$tw nodes $tnode]] == 0} {
+	set nitems 0
+    } else {
+	set nitems [llength [disco::get_jid_items $connid $jid $node]]
+    }
+    set desc [item_desc $jid $node $name $nitems]
     set icon ""
 
     add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
@@ -566,7 +627,7 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
 
     if {$res != "OK"} {
 	set tnode [jid_to_tag "error info $jid $node"]
-	set data [list error_info $jid]
+	set data [list error_info $connid $jid]
 	#set name     [jlib::wrapper:getattr $identity name]
 	set desc [format [::msgcat::mc "Error getting info: %s"] \
 		      [error_to_string $identities]]
@@ -581,7 +642,6 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
 	return
     }
 
-
     set identitynodes {}
 
     set category ""
@@ -592,7 +652,7 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
 	set name     [jlib::wrapper:getattr $identity name]
 	set category [jlib::wrapper:getattr $identity category]
 	set type     [jlib::wrapper:getattr $identity type]
-	set data [list identity $jid $node $category $type]
+	set data [list identity $connid $jid $node $category $type]
 	set desc "$name ($category/$type)"
 	set icon [item_icon $category $type]
 	
@@ -603,10 +663,11 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
     set extranodes {}
     
     foreach extra $extras {
-	lassign $extra var label value
+	lassign $extra var label values
 	set tnode [jid_to_tag "extra $var $jid $node"]
 	lappend extranodes $tnode
-	set data [list extra $var $jid $node]
+	set data [list extra $var $connid $jid $node]
+	set value [join $values ", "]
 	set desc "$label ($var): $value"
 	set icon ""
 	
@@ -620,7 +681,7 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
 	set var [jlib::wrapper:getattr $feature var]
 	set tnode [jid_to_tag "feature $feature $jid $node"]
 	lappend featurenodes $tnode
-	set data [list feature $jid $node $feature $category $type]
+	set data [list feature $connid $jid $node $feature $category $type]
 	set desc $var
 	if {[info exists browser(feature_handler_desc,$var)]} {
 	    catch { array unset tmp }
@@ -631,13 +692,35 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
 		set desc "$tmp(*) ($var)"
 	    }
 	}
-	# TODO
-	set icon ""; #[item_icon $category $type]
+	set icon ""
 
 	add_line $tw $parent_tag $tnode $icon $desc $data \
 	    -fill $config(featurecolor)
     }
 
+    # Draw all implicit item nodes, which are not received explicitly
+    # (don't overwrite node because it can have different name)
+    foreach item $featured_nodes {
+	set ijid [jlib::wrapper:getattr $item jid]
+	set node [jlib::wrapper:getattr $item node]
+
+	set name [jlib::wrapper:getattr $item name]
+	set tnode [jid_to_tag [list $ijid $node]]
+	set data [list item $connid $ijid $node]
+	if {![$tw exists $tnode] || [llength [$tw nodes $tnode]] == 0} {
+	    set nitems 0
+	} else {
+	    set nitems [llength [disco::get_jid_items $connid $ijid $node]]
+	}
+	set desc [item_desc $ijid $node $name $nitems]
+	set icon ""
+
+	if {![$tw exists $tnode]} {
+	    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+		     -fill $config(fill)
+	}
+    }
+
     remove_old $tw $parent_tag identity $identitynodes
     remove_old $tw $parent_tag extra    $extranodes
     remove_old $tw $parent_tag feature  $featurenodes
@@ -645,14 +728,14 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
     reorder_node $tw $parent_tag
 }
 
-proc disco::browser::items_receive {jid node res items} {
+proc disco::browser::items_receive {connid jid node res items} {
     variable browser
 
     if {![info exists browser(opened)]} return
 
     foreach w $browser(opened) {
 	if {[winfo exists $w] && [lcontain $browser(required,$w) $jid]} {
-	    draw_items $w $jid $node $res $items
+	    draw_items $w $connid $jid $node $res $items
 	}
     }
 }
@@ -660,59 +743,90 @@ proc disco::browser::items_receive {jid node res items} {
 hook::add disco_items_hook \
     [namespace current]::disco::browser::items_receive
 
-proc disco::browser::draw_items {w jid node res items} {
+proc disco::browser::draw_items {w connid jid node res items} {
     variable browser
     variable config
     global font
 
     set tw $browser(tree,$w)
 
-    set tnode [jid_to_tag [list $jid $node]]
     set parent_tag [jid_to_tag [list $jid $node]]
-    if {![info exists browser(sort,$w,$parent_tag)]} {
-	set browser(sort,$w,$parent_tag) sort
+
+    set name [disco::get_jid_name $connid $jid $node]
+    set tnode [jid_to_tag [list $jid $node]]
+    set data [list item $connid $jid $node]
+    if {![$tw exists $tnode] || [llength [$tw nodes $tnode]] == 0} {
+	set nitems 0
+    } else {
+	set nitems [llength [disco::get_jid_items $connid $jid $node]]
     }
-    set data [list item $jid $node]
-    set desc $jid
+    set desc [item_desc $jid $node $name $nitems]
     set icon ""
 
     add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
 	-fill $config(fill)
 
     if {$res != "OK"} {
-	set tnode [jid_to_tag "error items $jid $node"]
-	set data [list error_items $jid]
-	#set name     [jlib::wrapper:getattr $identity name]
-	set desc [format [::msgcat::mc "Error getting items: %s"] \
-		      [error_to_string $items]]
-	set icon ""
+	# HACK
+	if {[info exists ::disco::disco(info,featured_nodes,$connid,$jid,$node)] && \
+	    ![lempty $::disco::disco(info,featured_nodes,$connid,$jid,$node)]} {
+	    set items {}
+	} else {
+	    set tnode [jid_to_tag "error items $jid $node"]
+	    set data [list error_items $connid $jid]
+	    #set name     [jlib::wrapper:getattr $identity name]
+	    set desc [::msgcat::mc "Error getting items: %s" \
+				   [error_to_string $items]]
+	    set icon ""
 	
-	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
-	    -fill $config(fill)
+	    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+		     -fill $config(fill)
 
-	remove_old $tw $parent_tag item [list $tnode]
-	reorder_node $tw $parent_tag
-	return
+	    remove_old $tw $parent_tag item [list $tnode]
+	    reorder_node $tw $parent_tag
+	    return
+	}
     }
 
+    # HACK
+    # Don't remove nodes, which are drawn after disco#info query
+    # (if the service's features change then this node list may be
+    # incorrect)
     set itemnodes {}
+    if {[info exists ::disco::disco(info,featured_nodes,$connid,$jid,$node)]} {
+	foreach item $::disco::disco(info,featured_nodes,$connid,$jid,$node) {
+	    set ijid [jlib::wrapper:getattr $item jid]
+	    set inode [jlib::wrapper:getattr $item node]
+	    lappend itemnodes [jid_to_tag [list $ijid $inode]]
+	}
+    }
 
     foreach item $items {
 	set ijid [jlib::wrapper:getattr $item jid]
 	set node [jlib::wrapper:getattr $item node]
+
 	set name [jlib::wrapper:getattr $item name]
 	set tnode [jid_to_tag [list $ijid $node]]
-	lappend itemnodes $tnode
-	set data [list item $ijid $node]
-	set desc [item_desc $ijid $node $name]
+	set data [list item $connid $ijid $node]
+	if {![$tw exists $tnode] || [llength [$tw nodes $tnode]] == 0} {
+	    set nitems 0
+	} else {
+	    set nitems [llength [disco::get_jid_items $connid $ijid $node]]
+	}
+	set desc [item_desc $ijid $node $name $nitems]
 	set icon ""
 
+	lappend itemnodes $tnode
+
 	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
 	    -fill $config(fill)
     }
     remove_old $tw $parent_tag item $itemnodes
     remove_old $tw $parent_tag error_items {}
-    #reorder_node $tw $parent_tag
+
+    if {![info exists browser(sort,$w,$parent_tag)]} {
+	set browser(sort,$w,$parent_tag) sort
+    }
     browser_action $browser(sort,$w,$parent_tag) $w $parent_tag
 }
 
@@ -726,7 +840,7 @@ proc disco::browser::negotiate_feature {tw connid jid parent type} {
 
     if {$res != "OK"} {
 	set node [jid_to_tag "error negotiate $parent"]
-	set data [list error_negotiate $parent $jid]
+	set data [list error_negotiate $parent $connid $jid]
 	#set name     [jlib::wrapper:getattr $identity name]
 	set desc [format [::msgcat::mc "Error negotiate: %s"] \
 		      [error_to_string $opts]]
@@ -794,11 +908,11 @@ proc disco::browser::reorder_node {tw node {order {}}} {
 	lassign [$tw itemcget $sn -data] kind
 	switch -- $kind {
 	    error_items -
-	    item     {lappend items      $sn}
-	    error_info -
-	    identity {lappend identities $sn}
-	    feature  {lappend features   $sn}
-	    extra    {lappend extras     $sn}
+	    item        {lappend items      $sn}
+	    error_info  -
+	    identity    {lappend identities $sn}
+	    feature     {lappend features   $sn}
+	    extra       {lappend extras     $sn}
 	}
     }
     if {$order == {}} {
@@ -820,64 +934,43 @@ proc disco::browser::remove_old {tw node kind newnodes} {
     }
 }
 
-proc disco::browser::item_desc {jid node name} {
+proc disco::browser::item_desc {jid node name nitems} {
+    variable options
+
     if {$node != ""} {
 	set snode " \[$node\]"
     } else {
 	set snode ""
     }
-    if {![cequal $name ""]} {
-	return "$name$snode ($jid)"
+    if {$nitems > $options(upper_items_bound)} {
+	set sitems " - $nitems"
     } else {
-	return $jid$snode
+	set sitems ""
+    }
+    if {![cequal $name ""]} {
+	return "$name$snode ($jid)$sitems"
+    } else {
+	return "$jid$snode$sitems"
     }
 }
 
 proc disco::browser::item_icon {category type} {
     switch -- $category {
-	gateway {
-	    switch -- $type {
-		aim        { return browser/aim }
-		icq        { return browser/icq }
-		msn        { return browser/msn }
-		yahoo      { return browser/yahoo }
-		x-gadugadu { return browser/gg }
-		default    { return ""}
-	    }
-	}
-	service {
-	    switch -- $type {
-		jud        { return browser/jud }
-		aim        { return browser/aim }
-		icq        { return browser/icq }
-		msn        { return browser/msn }
-		x-gadugadu { return browser/gg }
-		x-weather  { return browser/weather }
-		yahoo      { return browser/yahoo }
-		default    { return "" }
-	    }
-	}
+	service -
+	gateway -
 	application {
-	    switch -- $type {
-		x-weather { return browser/weather }
-		default   { return "" }
+	    if {[lsearch -exact [image names] browser/$type] >= 0} {
+		return browser/$type
+	    } else {
+		return ""
 	    }
 	}
-	conference {
-	    return browser/conference
-	}
-	user -
-	client {
-	    return browser/user
-	}
-	directory {
-	    return browser/jud
-	}
-	headline {
-	    return browser/message
-	}
 	default {
-	    return ""
+	    if {[lsearch -exact [image names] browser/$category] >= 0} {
+		return browser/$category
+	    } else {
+		return ""
+	    }
 	}
     }
 }
@@ -891,11 +984,11 @@ proc disco::browser::textaction {bw tnode} {
     set data2 [lassign $data type]
     switch -- $type {
 	item {
-	    lassign $data2 jid node
+	    lassign $data2 connid jid node
 	    goto $bw $jid $node
 	}
 	feature {
-	    lassign $data2 jid node feature category subtype
+	    lassign $data2 connid jid node feature category subtype
 	    set var [jlib::wrapper:getattr $feature var]
 	    debugmsg disco $jid
 	    if {$var != ""} {
@@ -903,14 +996,14 @@ proc disco::browser::textaction {bw tnode} {
 		    if {$browser(feature_handler_node,$var)} {
 			eval $browser(feature_handler,$var) [list $jid $node \
 			    -category $category -type $subtype \
-			    -connection $browser(connid,$bw)]
+			    -connection $connid]
 		    } else {
 			eval $browser(feature_handler,$var) [list $jid \
 			    -category $category -type $subtype \
-			    -connection $browser(connid,$bw)]
+			    -connection $connid]
 		    }
 		} else {
-		    negotiate_feature $tw $browser(connid,$bw) $jid $tnode $var
+		    negotiate_feature $tw $connid $jid $tnode $var
 		}
 	    }
 	}
@@ -920,7 +1013,8 @@ proc disco::browser::textaction {bw tnode} {
 proc disco::browser::textpopup {bw tnode} {
     variable browser
 
-    if {[winfo exists [set m .discopopupmenu]]} {
+    set m .discopopupmenu
+    if {[winfo exists $m]} {
 	destroy $m
     }
     menu $m -tearoff 0
@@ -929,65 +1023,62 @@ proc disco::browser::textpopup {bw tnode} {
     set data [$tw itemcget $tnode -data]
     set data2 [lassign $data type]
 
-    switch -- $type {
-	identity {
-	}
+    # Parent node category shouldn't impact node action in theory,
+    # but sometimes (e.g. when joining MUC group) it's useful.
+    set tparentnode [$tw parent $tnode]
+    set parentdata {}
+    catch {set parentdata [$tw itemcget $tparentnode -data]}
 
+    hook::run disco_node_menu_hook $m $bw $tnode $data $parentdata
+
+    tk_popup $m [winfo pointerx .] [winfo pointery .]
+}
+
+proc disco::browser::textpopup_menu_setup {m bw tnode data parentdata} {
+    variable browser
+    set tw $browser(tree,$bw)
+
+    if {[$m index end] != "none"} {
+	$m add separator
+    }
+
+    set tparentnode [$tw parent $tnode]
+
+    set data2 [lassign $data type]
+    switch -- $type {
 	feature {
 	    $m add command -label [::msgcat::mc "Browse"] \
 		-command [list [namespace current]::browser_action browse $bw $tnode]
+	    $m add separator
 	}
-
 	item {
-	    lassign $data2 jid node
-	    set identities [[namespace parent]::get_jid_identities $jid $node]
-	    # TODO: use all identities
-	    set category [jlib::wrapper:getattr [lindex $identities 0] category]
-	    switch -- $category {
-		client {
-		    message::subject_menu $m $browser(connid,$bw) \
-			   [lindex $data 1] message
-		    tk_popup $m [winfo pointerx .] [winfo pointery .]
-		    return
-		}
-
-		conference {
-		    if {[string first @ [set headjid [lindex $data 1]]] > 0} {
-			$m add command -label [::msgcat::mc "Join group..."] \
-			    -command [list join_group_dialog \
-					   -server [server_from_jid $headjid] \
-					   -group [node_from_jid $headjid] \
-					   -connection $browser(connid,$bw)]
-			$m add command -label [::msgcat::mc "Add conference..."] \
-			    -command [list plugins::conferences::add_conference_dialog \
-					   -group [node_from_jid $headjid] \
-					   -server [server_from_jid $headjid] \
-					   -connection $browser(connid,$bw)]
-			$m add separator
-		    }
-		}
-
-		default {
-		}
-	    }
 	    $m add command -label [::msgcat::mc "Browse"] \
 		-command [list [namespace current]::browser_action browse $bw $tnode]
 	    $m add command -label [::msgcat::mc "Sort items by name"] \
 		-command [list [namespace current]::browser_action sort $bw $tnode]
 	    $m add command -label [::msgcat::mc "Sort items by JID/node"] \
 		-command [list [namespace current]::browser_action sortjid $bw $tnode]
+
+	    $m add separator
+	    if {$tparentnode == "root"} {
+		set label [::msgcat::mc "Delete current node and subnodes"]
+	    } else {
+		set label [::msgcat::mc "Delete subnodes"]
+	    }
+	    $m add command -label $label \
+		-command [list [namespace current]::clear $bw $tnode]
+	}
+	default {
 	}
     }
 
-    $m add separator
-    $m add command -label [::msgcat::mc "Clear"] \
-	-command [list [namespace current]::clear $bw $tnode]
-    $m add command -label [::msgcat::mc "Clear All"] \
+    $m add command -label [::msgcat::mc "Clear window"] \
 	-command [list [namespace current]::clearall $bw]
-
-    tk_popup $m [winfo pointerx .] [winfo pointery .]
 }
 
+hook::add disco_node_menu_hook \
+	  [namespace current]::disco::browser::textpopup_menu_setup 100
+
 proc disco::browser::clearall {bw} {
     variable browser
     set tw $browser(tree,$bw)
@@ -1004,18 +1095,21 @@ proc disco::browser::clear {bw tnode} {
 
     set tparentnode [$tw parent $tnode]
     
-    set data [$tw itemcget $tnode -data]
-    set data2 [lassign $data type]
+    set type [lindex [$tw itemcget $tnode -data] 0]
 
     if {$tparentnode != "root"} {
-	if {$type == "item"} {
-	    set subnodes [$tw nodes $tnode]
-	} else {
-	    set subnodes [$tw nodes $tparentnode]
+	if {$type != "item"} {
+	    set tnode $tparentnode
 	}
-	foreach sn $subnodes {
+	foreach sn [$tw nodes $tnode] {
 	    $tw delete $sn
 	}
+	lassign [$tw itemcget $tnode -data] type connid jid node
+	if {$type == "item"} {
+	    set name [disco::get_jid_name $connid $jid $node]
+	    set desc [item_desc $jid $node $name 0]
+	    $tw itemconfigure $tnode -text $desc
+	}
     } else {
 	$tw delete $tnode
     }
@@ -1041,10 +1135,10 @@ proc disco::browser::browser_action {action bw tnode} {
 		set data [lassign [$tw itemcget $child -data] type]
 		switch -- $type {
 		    item {
-			lassign $data jid node
+			lassign $data connid jid node
 			lappend items \
 			    [list $child \
-				  [[namespace parent]::get_jid_name $jid $node]]
+				  [disco::get_jid_name $connid $jid $node]]
 		    }
 		}
             }
@@ -1067,10 +1161,10 @@ proc disco::browser::browser_action {action bw tnode} {
 		set data [lassign [$tw itemcget $child -data] type]
 		switch -- $type {
 		    item {
-			lassign $data jid node
+			lassign $data connid jid node
 			if {$node != {}} {
 			    lappend items_with_nodes \
-				[list $child [join $data "\u0000"]]
+				[list $child "$jid\u0000$node"]
 			} else {
 			    lappend items [list $child $jid]
 			}
@@ -1095,37 +1189,22 @@ proc disco::browser::browser_action {action bw tnode} {
 }
 
 # TODO
-proc disco::browser::textballoon {bw action X Y node} {
+proc disco::browser::textballoon {bw node} {
     variable disco
     variable browser
 
     set tw $browser(tree,$bw)
 
-    if {[catch { set data [$tw itemcget $node -data] }]} {
-	balloon::destroy
-	return
+    if {[catch {set data [$tw itemcget $node -data]}]} {
+	return [list $bw:$node ""]
     }
 
-    set data2 [lassign $data type]
+    lassign $data type connid jid category subtype name version
     if {$type == "jid"} {
-	    lassign $data2 jid category type name version
+	return [list $bw:$node \
+		     [item_balloon_text $jid $category $subtype $name $version]]
     } else {
-	return
-    }
-
-    switch -- $action {
-	enter {
-	    balloon::set_text \
-		[item_balloon_text $jid $category $type $name $version]
-	}
-	motion {
-	    balloon::on_mouse_move \
-		[item_balloon_text $jid $category $type $name $version] \
-		$X $Y
-	}
-	leave {
-	    balloon::destroy
-	}
+	return [list $bw:$node ""]
     }
 }
 
@@ -1140,7 +1219,7 @@ proc disco::browser::goto {bw jid node} {
 proc disco::browser::get_category_type {t tnode} {
     foreach child [$t nodes $tnode] {
 	set data [$t itemcget $child -data]
-        set data2 [lassign $data type jid node]
+        set data2 [lassign $data type connid jid node]
 	if {$type == "identity"} {
 	    return $data2
 	}
@@ -1148,9 +1227,9 @@ proc disco::browser::get_category_type {t tnode} {
     return {}
 }
 
-proc disco::browser::draginitcmd {t tnode top} {
+proc disco::browser::draginitcmd {bw t tnode top} {
     set data [$t itemcget $tnode -data]
-    set data2 [lassign $data type jid node]
+    set data2 [lassign $data type connid jid node]
 
     if {$type == "item"} {
 	if {[set img [$t itemcget $tnode -image]] != ""} {
@@ -1160,8 +1239,8 @@ proc disco::browser::draginitcmd {t tnode top} {
 	lassign [get_category_type $t $tnode] category type
 
 	if {![info exists category]} {
-	    # Using parent tag to get conference cathegory
-	    # ??? Which else cathegory could be got from parent?
+	    # Using parent tag to get conference category
+	    # ??? Which else category could be got from parent?
 	    lassign [get_category_type $t [$t parent $tnode]] category type
 
 	    if {![info exists category] || ($category != "conference")} {
@@ -1170,14 +1249,14 @@ proc disco::browser::draginitcmd {t tnode top} {
 	    }
 	}
 
-	return [list JID {copy} [list $jid $category $type "" ""]]
+	return [list JID {copy} [list $connid $jid $category $type "" ""]]
     } else {
 	return {}
     }
 }
 
 proc disco::browser::entrydropcmd {bw target source pos op type data} {
-    set jid [lindex $data 0]
+    set jid [lindex $data 1]
     goto $bw $jid ""
 }
 
@@ -1203,8 +1282,8 @@ proc disco::browser::history_move {bw shift} {
     $bw.navigate.node.e delete 0 end
     $bw.navigate.node.e insert 0 $newnode
 
-    disco::request_items $newjid $newnode -connection $browser(connid,$bw)
     disco::request_info $newjid $newnode -connection $browser(connid,$bw)
+    disco::request_items $newjid $newnode -connection $browser(connid,$bw)
 }
 
 proc disco::browser::history_add {bw jid} {
@@ -1245,6 +1324,20 @@ proc disco::browser::register_feature_handler {feature handler args} {
     }
 }
 
+# Destroy all (global) state assotiated with the given browser window.
+# Intended to be bound to a <Destroy> event handler for browser windows.
+proc disco::browser::destroy_state {bw} {
+    variable browser
+
+    array unset browser *,$bw
+    array unset browser *,$bw,*
+
+    set idx [lsearch -exact $browser(opened) $bw]
+    if {$idx >= 0} {
+	set browser(opened) [lreplace $browser(opened) $idx $idx]
+    }
+}
+
 hook::add postload_hook \
     [list browser::register_ns_handler $::NS(disco_info) disco::browser::open_win \
 	 -desc [list * [::msgcat::mc "Discover service"]]]
