@@ -2,7 +2,7 @@
 # the next line restarts using the correct interpreter \
 exec tclsh "$0" "$0" "$@"
 
-package require -exact jabberlib 0.9.0
+package require -exact jabberlib 0.10.0
 package require mime
 package require sha1
 package require tls
@@ -146,13 +146,12 @@ proc jlib::sendit {stayP to args} {
         }
         jlib::connect $connid \
 		      -transport $transport \
-		      -host $aprops(domain) \
-		      -port $port
-        jlib::wait_for_stream $connid
+		      -host [idna::domain_toascii $aprops(domain)] \
+		      -port $port \
+		      -password $options(-password)
     
-        jlib::send_auth $connid \
-	    $aprops(local) $options(-password) $aprops(resource) \
-            [namespace current]::sendit_aux
+        jlib::login $connid [namespace current]::sendit_aux
+
         vwait [namespace current]::sendit_result
     
         if {[string compare [lindex $sendit_result 0] OK]} {
@@ -196,19 +195,10 @@ proc jlib::sendit {stayP to args} {
         }
     }
     if {!$stayP} {
-# the non-blocking nature hoses us here, so force the server to close things...
-#        jlib::disconnect
-
-        set connid [lindex $lib(connections) end]
-
         set jsend::stayP 0
-        fconfigure $lib($connid,sck) -blocking 1
-        outmsg "</stream:stream>"
-        flush $lib($connid,sck)
-        vwait [namespace current]::lib($connid,sck)
+	jlib::disconnect $connid
     }
 
-
     return 1
 }
 
@@ -232,10 +222,10 @@ proc client:presence {args} {
 #    ::LOG "client:presence $args"
 }
 
-proc client:iqreply {connid from userid id type child} {
+proc client:iqreply {connid from userid id type lang child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set xmlns [jlib::wrapper:getattr $vars xmlns]
-    ::LOG "client:iqreply $from $userid $id $type $xmlns"
+    ::LOG "client:iqreply $from $userid $id $type $lang $xmlns"
 
     set result result
     set now [clock seconds]
@@ -362,7 +352,7 @@ proc jsend::follow {file argv} {
                 if {[string length $buffer] > 0} {
                     if {[catch { eval [list jlib::sendit 1] $argv \
                                       [parse $buffer] \
-                                      [list -body "$buffer\n"] } result]} {
+                                      [list -body $buffer] } result]} {
                         ::LOG $result
                         break
                     } elseif {$result} {
@@ -381,7 +371,7 @@ proc jsend::follow {file argv} {
             break
         } elseif {[set x [string first "\n" $buffer]] < 0} {
         } else {
-            set body [string range $buffer 0 $x]
+            set body [string range $buffer 0 [expr {$x-1}]]
             while {[catch { eval [list jlib::sendit 1] $argv [parse $body] \
                                  [list -body $body] } result]} {
                 ::LOG $result
@@ -544,3 +534,5 @@ for default values."
 }
 
 exit $status
+
+# vim:ft=tcl:ts=8:sw=4:sts=4:noet
