@@ -5,7 +5,7 @@ LD_LIBRARY_PATH=/usr/pkg/lib:/usr/lib:/usr/local/lib \
 export PATH LD_LIBRARY_PATH; exec tclsh "$0" "$@"
 
 
-package require jabberlib -exact 0.9.0
+package require jabberlib -exact 0.10.0
 package require http 2
 package require mime
 package require tls
@@ -117,17 +117,15 @@ proc jlib::sendit {stayP to args} {
         }
         jlib::connect $connid \
 		      -transport $transport \
-		      -host $aprops(domain) \
-		      -port $port
-        jlib::wait_for_stream $connid
+		      -host [idna::domain_toascii $aprops(domain)] \
+		      -port $port \
+		      -password $options(-password)
     
-        jlib::send_auth $connid \
-	    $aprops(local) $options(-password) $aprops(resource) \
-            [namespace current]::sendit_aux
+        jlib::login $connid [namespace current]::sendit_aux
         vwait [namespace current]::sendit_result
     
         if {[string compare [lindex $sendit_result 0] OK]} {
-            error [lindex $sendit_result 1]
+            error [lindex [stanzaerror::error_to_list [lindex $sendit_result 1]]
         }
     
         roster_get -command [namespace current]::roster_get_aux    
@@ -156,7 +154,7 @@ proc jlib::sendit {stayP to args} {
         }
     }
     if {!$stayP} {
-        jlib::disconnect
+        jlib::disconnect $connid
     }
 
     return 1
@@ -235,10 +233,10 @@ proc client:presence {connid from type x args} {
     rssbot::presence $jid $type
 }
 
-proc client:iqreply {connid from userid id type child} {
+proc client:iqreply {connid from userid id type lang child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set xmlns [jlib::wrapper:getattr $vars xmlns]
-    ::LOG "client:iqreply $from $userid $id $type $xmlns"
+    ::LOG "client:iqreply $from $userid $id $type $lang $xmlns"
 
     set result result
     set now [clock seconds]
@@ -298,7 +296,7 @@ proc client:iqreply {connid from userid id type child} {
         }
     }
 
-    jlib::send_iq $result $xmldata -to $from -id $id
+    jlib::send_iq $result $xmldata -to $from -id $id -connection [jlib::route ""]
 }
 
 proc client:roster_push {args} {}
@@ -364,6 +362,7 @@ proc rssbot::begin {argv} {
                      -vars [list xmlns jabber:iq:private] \
                      -subtags [list [jlib::wrapper:createtag $array \
                                          -vars [list xmlns rssbot.$array]]]] \
+		-connection [jlib::route ""] \
                 -command [list [namespace current]::iq_private 0]
         }
         while {$iqP > 0} {
@@ -521,6 +520,7 @@ proc rssbot::loop_aux {argv} {
                      -subtags [list [jlib::wrapper:createtag $array \
                                          -vars [list xmlns rssbot.$array] \
                                          -chdata [array get $array]]]] \
+		-connection [jlib::route ""] \
                 -command [list [namespace current]::iq_private 1]
         }
     }
@@ -906,6 +906,7 @@ unsubscribe [URL ...]
                      -subtags [list [jlib::wrapper:createtag $array \
                                          -vars [list xmlns rssbot.$array] \
                                          -chdata [array get $array]]]] \
+		-connection [jlib::route ""] \
                 -command [list [namespace current]::iq_private 1]
         }
 
@@ -945,6 +946,7 @@ proc rssbot::presence {jid status} {
                  -subtags [list [jlib::wrapper:createtag subscribers \
                                      -vars [list xmlns rssbot.subscribers] \
                                      -chdata [array get subscribers]]]] \
+	    -connection [jlib::route ""] \
             -command [list [namespace current]::iq_private 1]
 
         if {(![string compare $status available]) \
