@@ -1,6 +1,73 @@
-# $Id: gpgme.tcl,v 1.33 2006/03/28 19:12:17 aleksey Exp $
+# $Id: gpgme.tcl 1024 2007-03-07 15:58:27Z sergei $
 
-if {[catch { package require gpgme }]} {
+namespace eval ::ssj {}
+
+#############################################################################
+# Draw icons aside encrypted messages even if no GPG support
+
+proc ::ssj::draw_encrypted {chatid from type body x} {
+    # we already deciphered it in rewrite_message_hook
+
+    set chatw [chat::chat_win $chatid]
+
+    foreach xe $x {
+	jlib::wrapper:splitxml $xe tag vars isempty cdata children
+
+	if {![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(encrypted)]} {
+	    continue
+	}
+
+	if {[cequal $cdata ""] || \
+	    [cequal [info commands ::ssj::encrypted:input] ""]} {
+	    $chatw image create end -image gpg/badencrypted
+	} else {
+	    $chatw image create end -image gpg/encrypted
+	}
+	break
+    }
+}
+
+hook::add draw_message_hook ::ssj::draw_encrypted 6
+
+#############################################################################
+
+proc ::ssj::process_x_encrypted {rowvar bodyvar f x connid from type replyP} {
+    upvar 2 $rowvar row
+    upvar 2 $bodyvar body
+
+    if {!$replyP || [cequal $type error]} {
+	return
+    }
+
+    foreach xa $x {
+	jlib::wrapper:splitxml $xa tag vars isempty cdata children
+	set xmlns [jlib::wrapper:getattr $vars xmlns]
+
+	if {$xmlns != $::NS(encrypted)} continue
+
+	# we already deciphered it in rewrite_message_hook
+	set lb [join [lrange [split $f .] 0 end-1] .].title.encrypted
+	if {[winfo exists $lb]} {
+	    destroy $lb
+	}
+
+	if {[cequal $cdata ""] || \
+	    [cequal [info commands ::ssj::encrypted:input] ""]} {
+	    Label $lb -image gpg/badencrypted
+	} else {
+	    Label $lb -image gpg/encrypted
+	}
+	grid $lb -row 1 -column 3 -sticky e
+    }
+
+    return
+}
+
+hook::add message_process_x_hook ::ssj::process_x_encrypted 21
+
+#############################################################################
+
+if {[catch {package require gpgme}]} {
     debugmsg ssj "unable to load the GPGME package, so no crypto!"
     set have_gpgme 0
     return
@@ -29,6 +96,10 @@ namespace eval ::ssj {
     custom::defvar options(key) "" \
 	[::msgcat::mc "Use specified key ID for signing and decrypting messages."] \
 	-group GPGME -type string
+
+    custom::defvar options(display_sig_warnings) 1 \
+	[::msgcat::mc "Display warning dialogs when signature verification fails."] \
+	-group GPGME -type boolean
 }
 
 
@@ -64,7 +135,7 @@ namespace eval ::ssj {
                 expired    [::msgcat::mc "The signature is good but has expired"] \
                 expiredkey [::msgcat::mc "The signature is good but the key has expired"]]
 
-    catch { unset warnings }
+    catch {unset warnings}
     array set warnings {}
 
     variable signedid 0
@@ -143,7 +214,7 @@ proc ::ssj::once_only {connid {armorP 0}} {
     }
 
     set dw .selectkey$connid
-    catch { destroy $dw }
+    catch {destroy $dw}
 
     set titles {}
     set balloons {}
@@ -153,25 +224,25 @@ proc ::ssj::once_only {connid {armorP 0}} {
                 continue
             }
             foreach subkey $v {
-                catch { unset params }
+                catch {unset params}
                 array set params $subkey
                 if {![info exists params(email)]} {
                     continue
                 }
 		lappend titles $key $params(email)
-                if {![catch { format "%d%s/%s %s %s" $params(length)         \
-                                     [string range $params(algorithm) 0 0]   \
-                                     [string range $params(keyid) end-7 end] \
-                                     [clock format $params(created)          \
-                                            -format "%Y-%m-%d"]              \
-                                     $params(userid) } text]} {
+                if {![catch {format "%d%s/%s %s %s" $params(length)         \
+                                    [string range $params(algorithm) 0 0]   \
+                                    [string range $params(keyid) end-7 end] \
+                                    [clock format $params(created)          \
+                                           -format "%Y-%m-%d"]              \
+                                    $params(userid)} text]} {
 		    lappend balloons $key $text
 		}
 	    }
 	}
     }
 
-    CbDialog $dw [format [::msgcat::mc "Select Key for Signing %s Traffic"] $pattern] \
+    CbDialog $dw [::msgcat::mc "Select Key for Signing %s Traffic" $pattern] \
         [list [::msgcat::mc "Select"] "::ssj::once_only_aux $dw $connid" \
 	      [::msgcat::mc "Cancel"] "destroy $dw"] \
 	::ssj::selectkey$connid $titles $balloons \
@@ -312,20 +383,21 @@ proc ::ssj::signed:input {connid from signature data what} {
     variable s2e
     variable warnings
     variable gpg_error_id
+    variable options
 
     once_only $connid
 
-    if {[catch { $ctx($connid) -operation verify \
-			       -input     [binary format a* [encoding convertto utf-8 $data]]  \
-			       -signature [armor:decode $signature] } result]} {
+    if {[catch {$ctx($connid) -operation verify \
+			      -input     [binary format a* [encoding convertto utf-8 $data]]  \
+			      -signature [armor:decode $signature]} result]} {
         debugmsg ssj "verify processing error ($connid): $result ($from)"
 
         if {![info exists warnings(verify-traffic,$connid)]} {
 
             set warnings(verify-traffic,$connid) 1
             after idle [list NonmodalMessageDlg .verify_error$connid -aspect 50000 -icon error \
-                -message [format [::msgcat::mc "Error in signature verification software: %s."] \
-				 $result]]
+                -message [::msgcat::mc "Error in signature verification software: %s." \
+				       $result]]
         }
 
         set params(reason) $result
@@ -340,7 +412,7 @@ proc ::ssj::signed:input {connid from signature data what} {
 
     set signatures {}
     foreach signature $params(signatures) {
-        catch { unset sparams }
+        catch {unset sparams}
         array set sparams $signature
 
         if {[info exists sparams(key)]} {
@@ -350,7 +422,7 @@ proc ::ssj::signed:input {connid from signature data what} {
                     continue
                 }
                 foreach subkey $v {
-                    catch { unset kparams }
+                    catch {unset kparams}
                     array set kparams $subkey
                     if {[info exists kparams(keyid)]} {
                         set j2k($from) $kparams(keyid)
@@ -362,7 +434,7 @@ proc ::ssj::signed:input {connid from signature data what} {
 
         lappend signatures [array get sparams]
     }
-    catch { unset params }
+    catch {unset params}
     array set params [list signatures $signatures]
 
     if {![cequal $result good]} {
@@ -371,13 +443,12 @@ proc ::ssj::signed:input {connid from signature data what} {
         }
         set params(reason) $result
 
-        if {![info exists warnings(verify,$from)]} {
+        if {![info exists warnings(verify,$from)] && $options(display_sig_warnings)} {
             set warnings(verify,$from) 1
             incr gpg_error_id
             after idle [list NonmodalMessageDlg .verify_error$gpg_error_id -aspect 50000 -icon error \
-                -message [format \
-			      [::msgcat::mc "%s purportedly signed by %s can't be verified.\n\n%s."] \
-			      $what $from $result]]
+                -message [::msgcat::mc "%s purportedly signed by %s can't be verified.\n\n%s." \
+				       $what $from $result]]
         }
     }
 
@@ -397,9 +468,9 @@ proc ::ssj::signed:output {connid data args} {
 
     once_only $connid 1
 
-    if {[catch { $ctx($connid) -operation sign  \
-			       -input     [binary format a* [encoding convertto utf-8 $data]] \
-			       -mode      detach } result]} {
+    if {[catch {$ctx($connid) -operation sign  \
+			      -input     [binary format a* [encoding convertto utf-8 $data]] \
+			      -mode      detach} result]} {
         set options(sign-traffic) 0
 
         debugmsg ssj "signature processing error ($connid): $result ($data)"
@@ -407,11 +478,16 @@ proc ::ssj::signed:output {connid data args} {
         if {[llength $args] == 0} {
             set buttons ok
             set cancel 0
-            set message [format [::msgcat::mc "Unable to sign presence information: %s.\n\nPresence will be sent, but signing traffic is now disabled."] $result]
+            set message [::msgcat::mc "Unable to sign presence information:\
+				       %s.\n\nPresence will be sent, but\
+				       signing traffic is now disabled." $result]
         } else {
             set buttons {ok cancel}
             set cancel 1
-            set message [format [::msgcat::mc "Unable to sign message body: %s.\n\nSigning traffic is now disabled.\n\nSend it WITHOUT a signature?"] $result]
+            set message [::msgcat::mc "Unable to sign message body:\
+				       %s.\n\nSigning traffic is now\
+				       disabled.\n\nSend it WITHOUT a signature?"\
+				      $result]
         }
 
         incr gpg_error_id
@@ -456,7 +532,7 @@ proc ::ssj::signed:info {pinfo} {
 			    continue
 			}
 			foreach subkey $v {
-			    catch { unset sparams }
+			    catch {unset sparams}
 			    array set sparams $subkey
 			    if {[info exists sparams(email)]} {
 				append addrs $s $sparams(email)
@@ -494,12 +570,65 @@ proc ::ssj::signed:info {pinfo} {
     return [string trimleft $text]
 }
 
-proc ::ssj::signed:Label {lb pinfo} {
+proc ::ssj::signed:Label {lb connid jid pinfo} {
+    if {[set rjid [muc::get_real_jid $connid $jid]] == ""} {
+	set rjid [node_and_server_from_jid $jid]
+    } else {
+	set rjid [node_and_server_from_jid $rjid]
+    }
+
     array set params $pinfo
+
+    set checks {}
+    set trust 0
+    foreach signature $params(signatures) {
+	set emails {}
+	set valid 0
+	foreach {k v} $signature {
+	    switch -- $k {
+		key {
+		    foreach {k v} $v {
+			if {![cequal $k subkeys]} {
+			    continue
+			}
+			foreach subkey $v {
+			    catch {unset sparams}
+			    array set sparams $subkey
+			    if {[info exists sparams(email)]} {
+				lappend emails $sparams(email)
+			    }
+			}
+		    }
+		}
+		validity {
+		    switch -- $v {
+			ultimate -
+			full -
+			marginal {
+			    set valid 1
+			}
+			never -
+			undefined -
+			unknown -
+			default {
+			    set valid 0
+			}
+		    }
+		}
+	    }
+	}
+	if {$valid && ([lsearch -exact $emails $rjid] >= 0)} {
+	    set trust 1
+	    break
+	}
+    }
+
     if {[info exists params(reason)]} {
 	set args [list -image gpg/badsigned]
-    } else {
+    } elseif {$trust} {
 	set args [list -image gpg/signed]
+    } else {
+	set args [list -image gpg/vsigned]
     }
 
     if {![cequal [set info [signed:info $pinfo]] ""]} {
@@ -556,7 +685,7 @@ proc ::ssj::fetchkeys {pinfo} {
 
     set keyids {}        
     foreach signature $params(signatures) {
-	catch { unset sparams }
+	catch {unset sparams}
 	array set sparams $signature
 
 	if {[info exists sparams(fingerprint)]} {
@@ -574,6 +703,58 @@ proc ::ssj::fetchkeys {pinfo} {
     }
 }
 
+###############################################################################
+
+proc ::ssj::rewrite_message_body \
+     {vconnid vfrom vid vtype vis_subject vsubject vbody verr vthread vpriority vx} {
+    upvar 2 $vconnid connid
+    upvar 2 $vfrom from
+    upvar 2 $vbody body
+    upvar 2 $vx x
+
+    set badenc 0
+    set xs {}
+    foreach xe $x {
+	jlib::wrapper:splitxml $xe tag vars isempty cdata children
+
+	if {![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(encrypted)]} {
+	    lappend xs $xe
+	} elseif {[cequal $cdata ""]} {
+	    # in case the sender didn't check the exit code from gpg we ignore
+	    # jabber:x:encrypted
+	} elseif {[catch {ssj::encrypted:input $connid $from $cdata} msg]} {
+	    set body [::msgcat::mc ">>> Unable to decipher data: %s <<<" $msg]
+	    # Add empty x tag to show problems with gpg
+	    lappend xs [jlib::wrapper:createtag x \
+			    -vars [list xmlns $::NS(encrypted)]]
+	    set badenc 1
+	} else {
+	    set body $msg
+	    lappend xs $xe
+	}
+    }
+
+    set x $xs
+
+    if {!$badenc} return
+
+    # if decryption failed, then remove signature. It can't be correct.
+    set xs {}
+    foreach xe $x {
+	jlib::wrapper:splitxml $xe tag vars isempty cdata children
+
+	if {![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(signed)]} {
+	    lappend xs $xe
+	}
+    }
+
+    set x $xs
+}
+
+hook::add rewrite_message_hook ::ssj::rewrite_message_body 10
+
+###############################################################################
+
 proc ::ssj::encrypted:input {connid from data} {
     variable ctx
     variable warnings
@@ -581,17 +762,16 @@ proc ::ssj::encrypted:input {connid from data} {
 
     once_only $connid
 
-    if {[catch { $ctx($connid) -operation decrypt \
-			       -input     [armor:decode $data] } result]} {
+    if {[catch {$ctx($connid) -operation decrypt \
+			      -input     [armor:decode $data]} result]} {
         debugmsg ssj "decryption processing error ($connid): $result ($from)"
 
         if {![info exists warnings(decrypt,$from)]} {
             set warnings(decrypt,$from) 1
             incr gpg_error_id
             after idle [list NonmodalMessageDlg .decrypt_error$gpg_error_id -aspect 50000 -icon error \
-                -message [format \
-			      [::msgcat::mc "Data purported sent by %s can't be deciphered.\n\n%s."] \
-			      $from $result]]
+                -message [::msgcat::mc "Data purported sent by %s can't be deciphered.\n\n%s." \
+				       $from $result]]
         }
 
         error $result
@@ -616,20 +796,18 @@ proc ::ssj::encrypted:output {connid data to} {
         return
     }
 
-    #if {[cequal [set jid [roster::find_jid $to]] ""]} {
-    #    set jid $to
-    #}
-    set jid [node_and_server_from_jid $to]
-    if {![encryptP $connid $jid]} {
+    if {![encryptP $connid $to]} {
         return
     }
 
+    set bto [node_and_server_from_jid $to]
+
     if {[info exists j2k($to)]} {
         set name $j2k($to)
     } elseif {[llength [set k [array names j2k $to/*]]] > 0} {
         set name $j2k([lindex $k 0])
     } else {
-        set name $jid
+        set name $bto
     }
 
     set recipient [gpgme::recipient]
@@ -657,7 +835,7 @@ proc ::ssj::encrypted:output {connid data to} {
     if {$code} {
         debugmsg ssj "encryption processing error ($connid): $result ($data)"
 
-        set options(encrypt,$connid,$jid) 0
+        set options(encrypt,$connid,$to) 0
         incr gpg_error_id
         if {[MessageDlg .encrypt_error$gpg_error_id \
 		-aspect 50000 \
@@ -666,11 +844,10 @@ proc ::ssj::encrypted:output {connid data to} {
                 -buttons {ok cancel} \
 		-default 0 \
 		-cancel 1 \
-                -message [format \
-			      [::msgcat::mc \
-				   "Unable to encipher data for %s:\
-				    %s.\n\nEncrypting traffic to this user is\
-				    now disabled.\n\nSend it as PLAINTEXT?"] \
+                -message [::msgcat::mc \
+			      "Unable to encipher data for %s:\
+			       %s.\n\nEncrypting traffic to this user is\
+			       now disabled.\n\nSend it as PLAINTEXT?" \
 			      $to $result]]} {
             error ""
         }
@@ -726,7 +903,7 @@ proc ::ssj::prefs {connid jid} {
     }
 
     Dialog $w \
-	   -title [format [::msgcat::mc "Change security preferences for %s"] $jid] \
+	   -title [::msgcat::mc "Change security preferences for %s" $jid] \
 	   -separator 1 -anchor e -default 0 -cancel 1
 
     $w add -text [::msgcat::mc "OK"] \
@@ -760,9 +937,8 @@ proc ::ssj::prefs_ok {w connid jid} {
 }
 
 proc ::ssj::prefs_user_menu {m connid jid} {
-    set rjid [roster::find_jid $connid $jid]
     $m add command -label [::msgcat::mc "Edit security..."] \
-	-command [list ::ssj::prefs $connid $rjid]
+	-command [list ::ssj::prefs $connid $jid]
 }
 
 hook::add chat_create_user_menu_hook ::ssj::prefs_user_menu 78
@@ -790,8 +966,9 @@ proc ::ssj::encryptP {connid jid} {
     lassign [roster::get_category_and_subtype $connid $jid] \
             category subtype
     switch -- $category {
-	conference
-	    -
+	conference -
+	server     -
+	gateway    -
 	service {
 	    set resP 0
 	}
@@ -801,10 +978,16 @@ proc ::ssj::encryptP {connid jid} {
 	}
     }
 
+    set bjid [node_and_server_from_jid $jid]
+
     if {[info exists options(encrypt,$connid,$jid)]} {
         return $options(encrypt,$connid,$jid)
+    } elseif {[info exists options(encrypt,$connid,$bjid)]} {
+        return $options(encrypt,$connid,$bjid)
     } elseif {[info exists options(encrypt,$jid)]} {
 	return $options(encrypt,$jid)
+    } elseif {[info exists options(encrypt,$bjid)]} {
+	return $options(encrypt,$jid)
     }
 
     if {!$options(encrypt-traffic)} {
@@ -822,7 +1005,7 @@ proc ::ssj::encryptP {connid jid} {
     } elseif {($resP) && ([llength [set k [array names j2k $jid/*]]] > 0)} {
         set name $j2k([lindex $k 0])
     } else {
-        set name $jid
+        set name $bjid
     }
 
     [set recipient [gpgme::recipient]] \
@@ -830,9 +1013,9 @@ proc ::ssj::encryptP {connid jid} {
             -name      $name \
             -validity  full
 
-    if {[catch { $ctx($connid) -operation  encrypt        \
-			       -input      "Hello world." \
-			       -recipients $recipient }]} {
+    if {[catch {$ctx($connid) -operation  encrypt        \
+			      -input      "Hello world." \
+			      -recipients $recipient}]} {
         set options(encrypt-tried,$connid,$jid) 0
     } else {
         set options(encrypt-tried,$connid,$jid) 1
@@ -861,9 +1044,9 @@ proc ::ssj::e4meP {connid keys} {
                 -name      $signer \
                 -validity  full
 
-        if {![catch { $ctx($connid) -operation  encrypt        \
-				    -input      "Hello world." \
-				    -recipients $recipient } result]} {
+        if {![catch {$ctx($connid) -operation  encrypt        \
+				   -input      "Hello world." \
+				   -recipients $recipient} result]} {
             lappend e4me($connid) $signer
         }
 
@@ -942,7 +1125,7 @@ proc ::ssj::trace {name1 name2 op} {
 
     set new {}
     foreach script $trace($name2) {
-        if {[catch { eval $script } result]} {
+        if {[catch {eval $script} result]} {
             debugmsg ssj "$result -- $script"
         } else {
             lappend new $script
@@ -957,9 +1140,9 @@ proc ::ssj::clear_signatures {connid} {
     variable signed
 
     if {$connid == {}} {
-	catch { array unset signed }
+	catch {array unset signed}
     } else {
-	catch { array unset signed $connid,* }
+	catch {array unset signed $connid,*}
     }
 }
 
@@ -973,7 +1156,7 @@ proc ::ssj::check_signature {connid from type x args} {
     switch -- $type {
 	unavailable -
 	available {
-	    catch { unset signed($connid,$from) }
+	    catch {unset signed($connid,$from)}
 
 	    set signature ""
 	    foreach xs $x {
@@ -1005,15 +1188,16 @@ hook::add client_presence_hook ::ssj::check_signature
 
 #############################################################################
 
-proc ::ssj::make_signature {acc connid status} {
+proc ::ssj::make_signature {varname connid status} {
+    upvar 2 $varname var
     if {![cequal $status ""] && \
-	    ![catch { signed:output $connid $status } cdata] && \
+	    ![catch {signed:output $connid $status} cdata] && \
 	    ![cequal $cdata ""]} {
-	lappend acc [jlib::wrapper:createtag x \
+	lappend var [jlib::wrapper:createtag x \
 			 -vars [list xmlns $::NS(signed)] \
 			 -chdata $cdata]
     }
-    return $acc
+    return
 }
 
 hook::add presence_xlist_hook ::ssj::make_signature
@@ -1043,7 +1227,7 @@ proc ::ssj::userinfo {tab connid jid editable} {
 	foreach j $jids {
 	    regexp {[^,]*,(.*)} $j -> fjid
 	    set x [userinfo::pack_frame $presenceinfo.presence_$i $fjid]
-	    catch { array unset params }
+	    catch {array unset params}
 	    array set params $signed($j)
 
 	    set kv {}
@@ -1057,7 +1241,7 @@ proc ::ssj::userinfo {tab connid jid editable} {
 				if {![cequal $k subkeys]} continue
 
 				foreach subkey $v {
-				    catch { unset sparams }
+				    catch {unset sparams}
 				    array set sparams $subkey
 				    if {[info exists sparams(email)]} {
 					append addrs $s $sparams(email)
@@ -1143,12 +1327,13 @@ hook::add open_message_post_hook ::ssj::message_buttons
 
 #############################################################################
 
-proc ::ssj::process_x_signed {acc f x connid from type replyP} {
-    if {!$replyP || [cequal $type error]} {
-	return $acc
-    }
+proc ::ssj::process_x_signed {rowvar bodyvar f x connid from type replyP} {
+    upvar 2 $rowvar row
+    upvar 2 $bodyvar body
 
-    lassign $acc row body
+    if {!$replyP || [cequal $type error]} {
+	return
+    }
 
     foreach xa $x {
 	jlib::wrapper:splitxml $xa tag vars isempty chdata children
@@ -1158,7 +1343,7 @@ proc ::ssj::process_x_signed {acc f x connid from type replyP} {
 
 	# in case the sender didn't check the exit code from gpg...
 	if {[cequal $chdata ""]} {
-	    return $acc
+	    return
 	}
 
 	set lb [join [lrange [split $f .] 0 end-1] .].title.signed
@@ -1166,49 +1351,19 @@ proc ::ssj::process_x_signed {acc f x connid from type replyP} {
 	    destroy $lb
 	}
 
-	signed:Label $lb \
-	    [signed:input $connid $from $chdata $body \
-		 [::msgcat::mc "Message body"]]
+	signed:Label $lb $connid $from \
+		     [signed:input $connid $from $chdata $body \
+				   [::msgcat::mc "Message body"]]
 	grid $lb -row 1 -column 2 -sticky e
     }
 
-    return $acc
+    return
 }
 
 hook::add message_process_x_hook ::ssj::process_x_signed 20
 
 #############################################################################
 
-proc ::ssj::process_x_encrypted {acc f x connid from type replyP} {
-    if {!$replyP || [cequal $type error]} {
-	return $acc
-    }
-
-    lassign $acc row body
-
-    foreach xa $x {
-	jlib::wrapper:splitxml $xa tag vars isempty chdata children
-	set xmlns [jlib::wrapper:getattr $vars xmlns]
-
-	if {$xmlns != $::NS(encrypted)} continue
-
-	# we already deciphered it in client:message...
-	set lb [join [lrange [split $f .] 0 end-1] .].title.encrypted
-	if {[winfo exists $lb]} {
-	    destroy $lb
-	}
-
-	Label $lb -image gpg/encrypted
-	grid $lb -row 1 -column 3 -sticky e
-    }
-
-    return $acc
-}
-
-hook::add message_process_x_hook ::ssj::process_x_encrypted 21
-
-#############################################################################
-
 proc ::ssj::signed:icon {} {
     return [lindex [list toolbar/gpg-unsigned toolbar/gpg-signed] \
                    [signP]]
@@ -1221,27 +1376,6 @@ proc ::ssj::encrypted:icon {{connid ""} {jid ""}} {
 
 #############################################################################
 
-proc ::ssj::draw_encrypted {chatid from type body x} {
-    # we already deciphered it in client:message...
-
-    set chatw [chat::chat_win $chatid]
-
-    foreach xe $x {
-	jlib::wrapper:splitxml $xe tag vars isempty chdata children
-
-	if {![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(encrypted)]} {
-	    continue
-	}
-
-	$chatw image create end -image gpg/encrypted
-	break
-    }
-}
-
-hook::add draw_message_hook ::ssj::draw_encrypted 7
-
-#############################################################################
-
 proc ::ssj::draw_signed {chatid from type body x} {
     variable signedid
 
@@ -1260,14 +1394,14 @@ proc ::ssj::draw_signed {chatid from type body x} {
 	set connid [chat::get_connid $chatid]
         catch {
             $chatw window create end \
-                  -window [signed:Label $chatw.signed$signedid \
+                  -window [signed:Label $chatw.signed$signedid $connid $from \
                               [signed:input $connid $from $chdata $body \
                                   [::msgcat::mc "Message body"]]]
         }
     }
 }
 
-hook::add draw_message_hook ::ssj::draw_signed 6
+hook::add draw_message_hook ::ssj::draw_signed 7
 
 ###############################################################################
 
@@ -1276,20 +1410,16 @@ proc ::ssj::chat_window_button {chatid type} {
     set jid [chat::get_jid $chatid]
     set cw [chat::winid $chatid]
 
-    if {[cequal [set sjid [roster::find_jid $connid $jid]] ""]} {
-        set sjid $jid
-    }
-
     Button $cw.status.encrypted \
 	   -relief flat \
-           -image [encrypted:icon $connid $sjid] \
+           -image [encrypted:icon $connid $jid] \
            -helptype balloon \
            -helptext [::msgcat::mc "Toggle encryption"] \
-           -command [list ::ssj::encrypt:toggleP $connid $sjid]
+           -command [list ::ssj::encrypt:toggleP $connid $jid]
 
     encrypted:trace "$cw.status.encrypted configure \
-		-image \[::ssj::encrypted:icon $connid $sjid\]" \
-        $connid $sjid
+		-image \[::ssj::encrypted:icon $connid $jid\]" \
+        $connid $jid
     pack $cw.status.encrypted -side left -before $cw.status.mb
 }
 
@@ -1298,31 +1428,50 @@ hook::add open_chat_post_hook ::ssj::chat_window_button
 ###############################################################################
 
 proc ::ssj::toolbar {} {
-    set mf .mainframe
-    set toolbar [$mf gettoolbar 0]
-    set bbox $toolbar.bbox
-
-    ifacetk::add_toolbar_button $bbox \
-	[signed:icon] \
-	::ssj::sign:toggleP \
-	[::msgcat::mc "Toggle signing"]
+    set idx [ifacetk::add_toolbar_button \
+		 [signed:icon] \
+		 ::ssj::sign:toggleP \
+		 [::msgcat::mc "Toggle signing"]]
     signed:trace \
-	[list ifacetk::set_toolbar_icon $bbox [$bbox index end] \
-	      ::ssj::signed:icon]
+	[list ifacetk::set_toolbar_icon $idx ::ssj::signed:icon]
 
-    ifacetk::add_toolbar_button $bbox \
-	[encrypted:icon] \
-	::ssj::encrypt:toggleP \
-	[::msgcat::mc "Toggle encryption (when possible)"]
+    set idx [ifacetk::add_toolbar_button \
+		 [encrypted:icon] \
+		 ::ssj::encrypt:toggleP \
+		 [::msgcat::mc "Toggle encryption (when possible)"]]
     encrypted:trace \
-	[list ifacetk::set_toolbar_icon $bbox [$bbox index end] \
-	      ::ssj::encrypted:icon]
+	[list ifacetk::set_toolbar_icon $idx ::ssj::encrypted:icon]
 }
 
 hook::add finload_hook ::ssj::toolbar
 
 ###############################################################################
 
+proc ::ssj::setup_menu {} {
+    variable options
+
+    catch {
+	set m [.mainframe getmenu tkabber]
+	set ind [$m index [::msgcat::mc "View"]]
+	incr ind -1
+
+	set mm .ssj_menu
+	menu $mm -tearoff $::ifacetk::options(show_tearoffs)
+	$mm add checkbutton -label [::msgcat::mc "Sign traffic"] \
+	    -variable ::ssj::options(sign-traffic)
+	$mm add checkbutton -label [::msgcat::mc "Encrypt traffic (when possible)"] \
+	    -variable ::ssj::options(encrypt-traffic)
+
+	$m insert $ind cascade -label [::msgcat::mc "Encryption"] \
+	    -menu $mm
+    }
+}
+
+
+hook::add finload_hook ::ssj::setup_menu
+
+###############################################################################
+
 proc ::ssj::add_user_popup_info {infovar connid jid} {
     variable signed
 
