@@ -1,4 +1,4 @@
-# $Id: iface.tcl,v 1.50 2006/06/12 20:38:16 aleksey Exp $
+# $Id: iface.tcl 1093 2007-04-09 18:31:05Z sergei $
 
 namespace eval ifacetk {
     variable options
@@ -32,7 +32,7 @@ namespace eval ifacetk {
 	-group IFace -type boolean \
 	-command [namespace current]::switch_statusbar
 
-    if {$::tcl_platform(platform) != "unix"} {
+    if {!$::xlfd_fonts} {
 	custom::defvar options(font) $::default_font \
 	    [::msgcat::mc "Font to use in roster, chat windows etc."] \
 	    -group IFace -type font \
@@ -57,7 +57,7 @@ namespace eval ifacetk {
 
     custom::defvar options(closebuttonaction) close \
 	[::msgcat::mc "What action does the close button."] \
-	-group IFace -type radio -layout vertical \
+	-group IFace -type options \
 	-values [list \
 		     close    [::msgcat::mc "Close Tkabber"] \
 		     nothing  [::msgcat::mc "Do nothing"] \
@@ -85,6 +85,8 @@ namespace eval ifacetk {
 
     variable number_msg
     variable personal_msg
+
+    namespace export add_win tab_set_updated
 }
 
 proc ifacetk::configure_tabs {args} {
@@ -109,8 +111,6 @@ proc ifacetk::save_mainwindowstate {w} {
     set state [wm state .]
     if {$state != $mainwindowstate} {
 	set mainwindowstate $state
-	set custom::saved([namespace current]::mainwindowstate) $mainwindowstate
-	custom::store
     }
 }
 
@@ -118,6 +118,8 @@ hook::add map_window_hook [namespace current]::ifacetk::save_mainwindowstate
 
 option add *errorForeground red widgetDefault
 
+wm protocol . WM_SAVE_YOURSELF session_checkpoint
+
 wm protocol . WM_DELETE_WINDOW \
    [list [namespace current]::ifacetk::wm_delete_window]
 
@@ -251,15 +253,24 @@ proc ifacetk::load_descmenu {} {
 		       -command {change_password_dialog}] \
 		  [list command [::msgcat::mc "Edit my info..."] {} {} {} \
 		       -command {
-			    set connid [jlib::route ""]
-			    userinfo::open [jlib::connection_bare_jid $connid] \
-				-editable 1 -connection $connid}] \
+			    if {![lempty [jlib::connections]]} {
+				set connid [jlib::route ""]
+				userinfo::open [jlib::connection_bare_jid $connid] \
+				    -editable 1 -connection $connid
+			    }}] \
 		  [list cascad [::msgcat::mc "Privacy rules"] {} privacy $ifacetk::options(show_tearoffs) \
 		       [list \
+			    [list checkbutton [::msgcat::mc "Activate lists at startup"] {} {} {} \
+				 -variable privacy::options(activate_at_startup)] \
 			    [list command [string trim [::msgcat::mc "Edit invisible list "]] {} {} {} \
 				 -command {privacy::edit_special_list invisible}] \
 			    [list command [string trim [::msgcat::mc "Edit ignore list "]] {} {} {} \
 				 -command {privacy::edit_special_list ignore}] \
+			    [list command [string trim [::msgcat::mc "Edit conference list "]] {} {} {} \
+				 -command {privacy::edit_special_list conference}] \
+			    [list checkbutton [::msgcat::mc "Accept messages from roster users only"] {} {} {} \
+				 -variable privacy::accept_from_roster_only \
+				 -command privacy::on_accept_from_roster_only_change] \
 			    {separator} \
 			    [list command [::msgcat::mc "Manually edit rules"] {} {} {} \
 				 -command {privacy::request_lists}]]] \
@@ -280,10 +291,9 @@ proc ifacetk::load_descmenu {} {
 		   [list \
 		    [list checkbutton [::msgcat::mc "Show online users only"] \
 			 {} {} {} \
-			 -variable [namespace current]::roster::show_only_online \
-			 -command [namespace current]::roster::redraw_after_idle] \
-		    [list checkbutton [::msgcat::mc "Use aliases"] {} {} {} \
-			 -variable [namespace current]::roster::use_aliases] \
+			 -variable [namespace current]::roster::show_only_online] \
+		    [list checkbutton [::msgcat::mc "Show own resources"] {} {} {} \
+			 -variable [namespace current]::roster::options(show_own_resources)] \
 		    [list cascad [::msgcat::mc "Export roster..."] \
 			 export_roster export_roster 0 {}] \
 		    [list cascad [::msgcat::mc "Import roster..."] \
@@ -293,14 +303,16 @@ proc ifacetk::load_descmenu {} {
 		  ]] \
 		  [list cascad [::msgcat::mc "Chats"] {} chats $ifacetk::options(show_tearoffs) \
 		   [list \
-		    [list checkbutton [::msgcat::mc "Generate event messages"] {} {} {} \
-			 -variable muc::options(gen_events)] \
+		    [list checkbutton [::msgcat::mc "Generate enter/exit messages"] {} {} {} \
+			 -variable muc::options(gen_enter_exit_msgs)] \
 		    [list checkbutton [::msgcat::mc "Smart autoscroll"] {} {} {} \
 			 -variable chat::options(smart_scroll)] \
 		    [list checkbutton [::msgcat::mc "Stop autoscroll"] {} {} {} \
 			 -variable chat::options(stop_scroll)] \
 		    [list checkbutton [::msgcat::mc "Emphasize"] {} {} {} \
-			 -variable chat::options(emphasize)] \
+			 -variable plugins::stylecodes::options(emphasize)] \
+		    [list checkbutton [::msgcat::mc "Show emoticons"] {} {} {} \
+			 -variable plugins::emoticons::options(show_emoticons)] \
 		    {separator} \
 		  ]] \
 		  {separator} \
@@ -329,7 +341,7 @@ proc ifacetk::load_descmenu {} {
 		       -command {chat::open_chat_dialog}] \
 		  [list command [::msgcat::mc "Join group..."] {} {} {}\
 		       -command {join_group_dialog}] \
-		  [list command [::msgcat::mc "Show user info..."] {} {} {} \
+		  [list command [::msgcat::mc "Show user or service info..."] {} {} {} \
 		       -command {userinfo::show_info_dialog}] \
 		  {separator} \
 		  [list command [::msgcat::mc "Jabber Browser"] {} {} {} \
@@ -364,7 +376,7 @@ proc ifacetk::load_descmenu {} {
 		 ] \
 	    ]
     
-    if {![info exists enable_profiling] || !$enable_profiling || \
+    if {![info exists ::enable_profiling] || !$::enable_profiling || \
 	    ([clength [info commands profile]] == 0)} {
 	set tmpmenu {}
 	set eatP 0
@@ -386,6 +398,12 @@ proc ifacetk::load_descmenu {} {
 
 ###############################################################################
 
+proc ifacetk::get_conf {w option} {
+    return [lindex [$w configure $option] 4]
+}
+
+###############################################################################
+
 proc ifacetk::send_announce_message {resource} {
     if {[lempty [jlib::connections]]} return
 
@@ -401,53 +419,108 @@ proc ifacetk::send_announce_message {resource} {
 ###############################################################################
 
 proc ifacetk::quick_help_window {} {
-    help_window [::msgcat::mc "Quick Help"] \
-"\n[::msgcat::mc {Main window:}]
-    $::tk_modify-L\t\t\t[::msgcat::mc {Log in}]
-    $::tk_modify-J\t\t\t[::msgcat::mc {Log out}]
-\n[::msgcat::mc Tabs:]
-    $::tk_modify-F4\t\t\t[::msgcat::mc {Close tab}]
-    $::tk_modify-PgUp/Down\t\t[::msgcat::mc {Previous/Next tab}]
+    set w .quick_help
+
+    if {[winfo exists $w]} {
+	destroy $w
+    }
+
+    Dialog $w -anchor e \
+	      -separator yes \
+	      -title [::msgcat::mc "Quick Help"] \
+	      -side bottom \
+	      -modal none \
+	      -default 0 \
+	      -cancel 0
+    $w add -text [::msgcat::mc "Close"] -command [list destroy $w]
+
+    set frame [$w getframe]
+    set sw [ScrolledWindow $frame.sw]
+    pack $sw -fill both -expand yes
+
+    set t [text $frame.help -wrap none]
+
+    set tabstop [font measure [get_conf $t -font] \
+		      "    [::msgcat::mc {Middle mouse button}]      "]
+    $t configure -tabs $tabstop
+
+    $sw setwidget $t
+    $t insert 0.0 \
+"[::msgcat::mc {Main window:}]
+    $::tk_modify-L\t[::msgcat::mc {Log in}]
+    $::tk_modify-J\t[::msgcat::mc {Log out}]
+
+[::msgcat::mc Tabs:]
+    $::tk_modify-F4,
+    [::msgcat::mc {Middle mouse button}]\t[::msgcat::mc {Close tab}]
+    $::tk_modify-PgUp/Down\t[::msgcat::mc {Previous/Next tab}]
     $::tk_modify-Alt-PgUp/Down\t[::msgcat::mc {Move tab left/right}]
-    Alt-\[1-9,0\]\t\t[::msgcat::mc {Switch to tab number 1-9,10}]
-    $::tk_modify-R\t\t\t[::msgcat::mc {Hide/Show roster}]
-\n[::msgcat::mc Chats:]
-    TAB\t\t\t[::msgcat::mc {Complete nickname}]
-    $::tk_modify-Up/Down\t\t[::msgcat::mc {Previous/Next history message}]
-    Alt-E\t\t\t[::msgcat::mc {Show emoticons}]
-    $::tk_modify-Z\t\t\t[::msgcat::mc {Undo}]
-    $::tk_modify-Shift-Z\t\t[::msgcat::mc {Redo}]
-    Alt-PgUp/Down\t\t[::msgcat::mc {Scroll chat window up/down}]
-    $::tk_modify-S\t\t\t[::msgcat::mc {Search in chat window}]
-    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Correct word}]
-"
+    Alt-\[1-9,0\]\t[::msgcat::mc {Switch to tab number 1-9,10}]
+    $::tk_modify-R\t[::msgcat::mc {Hide/Show roster}]
+
+[::msgcat::mc Common:]
+    $::tk_modify-S\t[::msgcat::mc {Activate search panel}]
+
+[::msgcat::mc Chats:]
+    TAB\t[::msgcat::mc {Complete nickname or command}]
+    $::tk_modify-Up/Down\t[::msgcat::mc {Previous/Next history message}]
+    Alt-E\t[::msgcat::mc {Show palette of emoticons}]
+    $::tk_modify-Z\t[::msgcat::mc {Undo}]
+    $::tk_modify-Shift-Z\t[::msgcat::mc {Redo}]
+    Alt-PgUp/Down\t[::msgcat::mc {Scroll chat window up/down}]
+    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Correct word}]"
+    $t configure -state disabled
+
+    $w draw
 }
 
 proc ifacetk::about_window {} {
-    global version
+    global tkabber_version toolkit_version
 
-    help_window [::msgcat::mc "About"] "
-Tkabber $version
+    set w .about
 
-Copyright \u00a9 2002-2006 [::msgcat::mc {Alexey Shchepin}]
+    if {[winfo exists $w]} {
+	destroy $w
+    }
+
+    Dialog $w -anchor e \
+	      -separator yes \
+	      -title [::msgcat::mc "About"] \
+	      -image tkabber/logo \
+	      -side bottom \
+	      -modal none \
+	      -default 0 \
+	      -cancel 0
+    $w add -text [::msgcat::mc "Close"] -command [list destroy $w]
+
+    set frame [$w getframe]
+    set m [message $frame.msg -text "
+Tkabber $tkabber_version ($toolkit_version)
+
+Copyright \u00a9 2002-2007 [::msgcat::mc {Alexey Shchepin}]
 \n[::msgcat::mc Authors:]
     [::msgcat::mc {Alexey Shchepin}]
     [::msgcat::mc {Marshall T. Rose}]
     [::msgcat::mc {Sergei Golovan}]
     [::msgcat::mc {Michail Litvak}]
+"]
+    pack $m -side top -anchor w
 
-http://tkabber.jabberstudio.org/"
-}
+    set t [text $frame.url -cursor [get_conf $frame.msg -cursor] \
+			   -bg [get_conf $frame.msg -bg] \
+			   -height 1 \
+			   -width 25 \
+			   -bd 0 \
+			   -highlightthickness 0 \
+			   -takefocus 0]
+    ::richtext::config $t -using url
+    ::richtext::render_message $t "  http://tkabber.jabber.ru/" ""
+    $t delete {end - 1 char}
+    $t configure -state disabled
 
-proc ifacetk::help_window {title message} {
-    Dialog .m -anchor e -separator yes -title $title -image tkabber/logo \
-	    -side bottom -modal local -default 0 -cancel 0
-    .m add -text [::msgcat::mc "OK"]
-    set frame [.m getframe]
-    message $frame.msg -text $message
-    pack $frame.msg -side left
-    .m draw
-    destroy .m
+    pack $t -side top -anchor w -fill x
+
+    $w draw
 }
 
 proc ifacetk::switch_font {args} {
@@ -564,12 +637,38 @@ proc ifacetk::update_ssl_ind {args} {
 hook::add connected_hook [namespace current]::ifacetk::update_ssl_ind
 hook::add disconnected_hook [namespace current]::ifacetk::update_ssl_ind
 
-proc ifacetk::add_toolbar_button {path icon command helptext} {
-    $path add -image $icon \
-	-highlightthickness 0 -takefocus 0 -relief link -bd $::tk_borderwidth \
-	-padx 1 -pady 1 -command $command -helptext $helptext
+###############################################################################
+
+proc ifacetk::add_toolbar_button {icon command helptext} {
+    if {[catch {set bbox [.mainframe gettoolbar 0].bbox}] || \
+	    ![winfo exists $bbox]} {
+	return 0
+    }
+    $bbox add -image $icon \
+	      -highlightthickness 0 \
+	      -takefocus 0 \
+	      -relief link \
+	      -bd $::tk_borderwidth \
+	      -padx 1 \
+	      -pady 1 \
+	      -command $command \
+	      -helptext $helptext
+    return [$bbox index end]
 }
 
+###############################################################################
+
+proc ifacetk::set_toolbar_icon {index script args} {
+    if {[catch {set bbox [.mainframe gettoolbar 0].bbox}] || \
+	    ![winfo exists $bbox]} {
+	return
+    }
+    set image [eval $script]
+    $bbox itemconfigure $index -image $image
+}
+
+###############################################################################
+
 proc ifacetk::online_icon {args} {
     if {$roster::show_only_online} {
 	return toolbar/show-online
@@ -578,14 +677,6 @@ proc ifacetk::online_icon {args} {
     }
 }
 
-proc ifacetk::set_toolbar_icon {path index script args} {
-    if {![winfo exists $path]} {
-	return
-    }
-    set image [eval $script]
-    $path itemconfigure $index -image $image
-}
-
 proc ifacetk::create_main_window {} {
     global usetabbar
     global user_status_list
@@ -593,8 +684,6 @@ proc ifacetk::create_main_window {} {
     global descmenu
     variable mf
     variable rosterwidth
-    variable rw
-    variable nw
     variable ssl_ind
     variable main_window_title
     variable options
@@ -623,18 +712,18 @@ proc ifacetk::create_main_window {} {
 
     set bbox [ButtonBox [$mf addtoolbar].bbox -spacing 0 -padx 1 -pady 1]
 
-    add_toolbar_button $bbox toolbar/add-user {message::send_subscribe_dialog ""} \
+    add_toolbar_button toolbar/add-user {message::send_subscribe_dialog ""} \
 	[::msgcat::mc "Add new user..."]
-    add_toolbar_button $bbox toolbar/disco {disco::browser::open_win ""} \
+    add_toolbar_button toolbar/disco {disco::browser::open_win ""} \
 	[::msgcat::mc "Service Discovery"]
-    add_toolbar_button $bbox toolbar/join-conference join_group_dialog \
+    add_toolbar_button toolbar/join-conference join_group_dialog \
 	[::msgcat::mc "Join group..."]
 
-    add_toolbar_button $bbox [ifacetk::online_icon] \
-	[namespace current]::roster::switch_only_online \
-	[::msgcat::mc "Toggle showing offline users"]
+    set idx [add_toolbar_button [ifacetk::online_icon] \
+		 [namespace current]::roster::switch_only_online \
+		 [::msgcat::mc "Toggle showing offline users"]]
     trace variable [namespace current]::roster::show_only_online w \
-	[list [namespace current]::set_toolbar_icon $bbox [$bbox index end] \
+	[list [namespace current]::set_toolbar_icon $idx \
 	      [namespace current]::online_icon]
 
     pack $bbox -side left -anchor w
@@ -697,14 +786,16 @@ proc ifacetk::create_main_window {} {
     if {$usetabbar} {
 	set pw [PanedWin [$mf getframe].pw -side bottom -pad 2 -width 8]
 	pack $pw -fill both -expand yes
-	set rw [$pw add -minsize 0 -weight 0]
-	set nw [$pw add -minsize 32 -weight 1]
+	set rw [PanedWinAdd $pw -minsize 0 -weight 0]
+	set nw [PanedWinAdd $pw -minsize 32 -weight 1]
 
 	roster::create .roster -width $rosterwidth -height 300 \
 	    -popup [namespace current]::roster::popup_menu \
 	    -grouppopup [namespace current]::roster::group_popup_menu \
 	    -singleclick [namespace current]::roster::user_singleclick \
-	    -doubleclick [namespace current]::roster::jid_doubleclick
+	    -doubleclick [namespace current]::roster::jid_doubleclick \
+	    -draginitcmd [namespace current]::roster::draginitcmd \
+	    -dropcmd [namespace current]::roster::dropcmd
 	pack .roster -expand yes -fill both -side left -in $rw
 
 	if {$options(tabs_at_bottom)} {
@@ -715,7 +806,7 @@ proc ifacetk::create_main_window {} {
 
 	NoteBook .nb -width 400 -side $side
 	pack .nb -side right -in $nw -fill both -expand yes
-	[winfo parent $rw] configure -width $rosterwidth
+	PanedWinConf $pw 0 -width $rosterwidth
 
 	event add <<CollapseRoster>> <Control-Key-r>
 	bind . <<CollapseRoster>>      [list [namespace current]::collapse_roster]
@@ -769,8 +860,9 @@ proc ifacetk::create_main_window {} {
 	    }
 
 	.nb bindtabs <3> [list [namespace current]::tab_menu %X %Y]
-	.nb bindtabs <4> [list [namespace current]::tab_move .nb -1]
-	.nb bindtabs <5> [list [namespace current]::tab_move .nb 1]
+	.nb bindtabs <2> [list [namespace current]::destroy_tab]
+	.nb bindtabs <<ScrollUp>> [list [namespace current]::tab_move .nb -1]
+	.nb bindtabs <<ScrollDown>> [list [namespace current]::tab_move .nb 1]
 
 	DragSite::register .nb.c -draginitcmd [namespace current]::draginitcmd
 	DropSite::register .nb.c -dropovercmd [namespace current]::dropovercmd \
@@ -785,7 +877,9 @@ proc ifacetk::create_main_window {} {
 	    -popup [namespace current]::roster::popup_menu \
 	    -grouppopup [namespace current]::roster::group_popup_menu \
 	    -singleclick [namespace current]::roster::user_singleclick \
-	    -doubleclick [namespace current]::roster::jid_doubleclick
+	    -doubleclick [namespace current]::roster::jid_doubleclick \
+	    -draginitcmd [namespace current]::roster::draginitcmd \
+	    -dropcmd [namespace current]::roster::dropcmd
 	pack .roster -expand yes -fill both -side left -in [$mf getframe]
 	set geometry [option get . geometry [winfo class .]]
 	if {$geometry == ""} {
@@ -793,7 +887,7 @@ proc ifacetk::create_main_window {} {
 	}
     }
 
-    bind . <Map> [list hook::run map_window_hook .]
+    bind . <Map> [list hook::run map_window_hook %W]
 
     wm geometry . $geometry
 
@@ -806,6 +900,14 @@ proc ifacetk::create_main_window {} {
     update idletasks
 }
 
+proc ifacetk::destroy_tab {page} {
+    if {[.nb raise] != ""} {
+	eval destroy [pack slaves [.nb getframe $page]]
+	.nb delete $page 1
+	ifacetk::tab_move .nb 0
+    }
+}
+
 proc ifacetk::draginitcmd {target x y top} {
     set c .nb.c
 
@@ -911,13 +1013,17 @@ proc ifacetk::nbpage {path} {
     return [crange [win_id tab $path] 1 end]
 }
 
+proc ifacetk::nbpath {page} {
+    return [lindex [pack slaves [.nb getframe $page]] 0]
+}
+
 proc ifacetk::update_chat_title {chatid} {
     global usetabbar
     variable options
     variable number_msg
     variable personal_msg
 
-    set cw $chat::opened($chatid)
+    set cw [chat::winid $chatid]
 
     if {$usetabbar} {
 	set tabtitle $chat::chats(tabtitlename,$chatid)
@@ -996,8 +1102,25 @@ proc ifacetk::add_number_of_messages_to_title {chatid from type body extras} {
     variable number_msg
     variable personal_msg
 
+    if {![catch {::plugins::mucignore::is_ignored $connid $from $type} ignore] && \
+	    $ignore != ""} {
+	    return
+    }
+
+    foreach xelem $extras {
+	jlib::wrapper:splitxml $xelem tag vars isempty chdata children
+	# Don't add number to title if this 'empty' tag is present. It indicates
+	# messages history in chat window.
+	if {[cequal $tag ""] && \
+	    [cequal [jlib::wrapper:getattr $vars xmlns] tkabber:x:nolog]} {
+	    return
+	}
+    }
+
     if {[chat_window_is_active $chatid]} return
 
+    if {$from == ""} return
+
     if {$type == "chat"} {
 	incr number_msg($chatid)
 	set personal_msg($chatid) 1
@@ -1005,11 +1128,11 @@ proc ifacetk::add_number_of_messages_to_title {chatid from type body extras} {
 	set jid [chat::get_jid $chatid]
 	set myjid [chat::our_jid $chatid]
 	set mynick [chat::get_nick [chat::get_connid $chatid] $myjid $type]
-	if {![cequal $myjid $from]} {
+	if {![cequal $jid $from] && ![cequal $myjid $from]} {
 	    incr number_msg($chatid)
 	}
 	if {![cequal $jid $from] && ![cequal $myjid $from] && \
-		[lindex [check_message $mynick $body] 0]} {
+		[check_message $mynick $body]} {
 	    set personal_msg($chatid) 1
 	}
     }
@@ -1045,10 +1168,6 @@ hook::add connected_hook \
 hook::add disconnected_hook \
     [namespace current]::ifacetk::set_main_window_title_on_disconnect
 
-proc add_win {args} {
-    eval ifacetk::add_win $args
-}
-
 proc ifacetk::add_win {path args} {
     global usetabbar
     variable options
@@ -1072,7 +1191,10 @@ proc ifacetk::add_win {path args} {
 
     if {$usetabbar} {
 	set page [nbpage $path]
-	set f [.nb insert end $page -text $tabtitle -raisecmd $raisecmd]
+	set f [.nb insert end $page \
+		   -text $tabtitle \
+		   -raisecmd [list [namespace current]::tab_raise \
+				   $path $raisecmd]]
 	frame $path -class $class
 	pack $path -expand yes -fill both -in $f
 	#tkwait visibility $path
@@ -1094,6 +1216,13 @@ proc ifacetk::add_win {path args} {
     }
 }
 
+proc ifacetk::tab_raise {path command} {
+    tab_set_updated $path
+    if {$command != ""} {
+	eval $command
+    }
+}
+
 proc ifacetk::get_focus {path} {
     variable focused
     variable after_focused_id
@@ -1110,6 +1239,8 @@ proc ifacetk::get_focus {path} {
 	set after_focused_id \
 	    [after $options(update_title_delay) [list [namespace current]::set_title $path]]
 	set focused $path
+
+	hook::run got_focus_hook $path
     }
 }
 
@@ -1148,6 +1279,8 @@ proc ifacetk::loose_focus {path} {
     }
     set focused ""
     balloon::destroy
+
+    hook::run lost_focus_hook $path
 }
 
 proc ifacetk::tab_move {nb shift args} {
@@ -1211,10 +1344,6 @@ proc ifacetk::define_alert_colors {} {
     }
 }
 
-proc tab_set_updated {args} {
-    eval ifacetk::tab_set_updated $args
-}
-
 array set ::alert_lvls {info 1 error 1 server 1 message 2 mesg_to_user 3}
 
 proc ifacetk::tab_set_updated {path {updated 0} {level ""}} {
@@ -1280,19 +1409,20 @@ proc ifacetk::tab_menu {x y page} {
 ###############################################################################
 
 proc ifacetk::collapse_roster {} {
-    variable rw
-    variable nw
+    variable mf
     variable rosterwidth
 
-    set r [[winfo parent $rw] cget -width]
-    set n [[winfo parent $nw] cget -width]
+    set pw [$mf getframe].pw
+    set r [PanedWinConf $pw 0 -width]
+    set n [PanedWinConf $pw 1 -width]
+
     if {$r > 0} {
 	set rosterwidth $r
-	[winfo parent $rw] configure -width 0
-	[winfo parent $nw] configure -width [expr {$r + $n}]
+	PanedWinConf $pw 0 -width 0
+	PanedWinConf $pw 1 -width [expr {$r + $n}]
     } else {
-	[winfo parent $rw] configure -width $rosterwidth
-	[winfo parent $nw] configure -width [expr {$r + $n - $rosterwidth}]
+	PanedWinConf $pw 0 -width $rosterwidth
+	PanedWinConf $pw 1 -width [expr {$r + $n - $rosterwidth}]
     }
 }
 
@@ -1353,10 +1483,11 @@ proc ifacetk::deiconify {} {
 	    wm state . zoomed
 	}
     }
+    update
     wm deiconify .
 }
 
-hook::add finload_hook [namespace current]::ifacetk::deiconify 99
+hook::add finload_hook [namespace current]::ifacetk::deiconify 100
 
 proc client:errormsg {message} {
     if {[winfo exists .client_error]} {
@@ -1367,3 +1498,22 @@ proc client:errormsg {message} {
 	-message $message -type user -buttons ok -default 0 -cancel 0
 }
 
+# This proc should be called by WM_SAVE_YOURSELF protocol callback.
+# On Windows (and Tk >= 8.4.13) this means WM_QUERYENDSESSION (so we should quit),
+# on Unix, an X session manager may call this repeatedly.
+proc session_checkpoint {} {
+    global tcl_platform
+    ::hook::run save_yourself_hook
+    if {$tcl_platform(platform) == "windows"} quit
+}
+
+# Trap SIGTERM to quit gracefully on Unix when Tclx is available:
+if {$tcl_platform(platform) == "unix" 
+    && ![catch {package require Tclx}]} {
+    signal trap SIGTERM quit
+}
+
+# Import add_win tab_set_updated to the root namespace:
+namespace import ::ifacetk::add_win ::ifacetk::tab_set_updated
+
+# vim:ts=8:sw=4:sts=4:noet
