@@ -1,4 +1,4 @@
-# $Id: iroster.tcl,v 1.46 2006/04/09 19:45:37 aleksey Exp $
+# $Id: iroster.tcl 1085 2007-04-08 14:12:50Z sergei $
 
 Tree .faketree
 foreach {k v} [list background       White \
@@ -78,6 +78,10 @@ namespace eval roster {
 	[::msgcat::mc "Default nested roster group delimiter."] \
 	-type string -group Roster \
 	-command [namespace current]::redraw_after_idle
+    custom::defvar options(show_own_resources) 0 \
+	[::msgcat::mc "Show my own resources in the roster."] \
+	-type boolean -group Roster \
+	-command [namespace current]::redraw_after_idle
     custom::defvar options(chats_group) 0 \
 	[::msgcat::mc "Add chats group in roster."] \
 	-type boolean -group Roster \
@@ -106,6 +110,7 @@ but also over any item in group."] \
 
     variable undef_group_name $::roster::undef_group_name
     variable chats_group_name $::roster::chats_group_name
+    variable own_resources_group_name $roster::own_resources_group_name
 }
 
 proc roster::get_group_lists {} {
@@ -138,6 +143,9 @@ proc roster::set_group_lists {connid} {
     variable undef_group_name
     variable chats_group_name
 
+    set _collapsed_group_list $collapsed_group_list
+    set _show_offline_group_list $show_offline_group_list
+
     if {$connid == {}} {
 	set connidlist [jlib::connections]
     } else {
@@ -182,28 +190,28 @@ proc roster::set_group_lists {connid} {
 	    }
 	}
 	if {![lempty $tmpc]} {
-	    while {[set idx [lsearch -exact $collapsed_group_list $bare_jid]] >= 0} {
-		set collapsed_group_list \
-		    [lreplace $collapsed_group_list $idx [expr {$idx + 1}]]
+	    while {[set idx [lsearch -exact $_collapsed_group_list $bare_jid]] >= 0} {
+		set _collapsed_group_list \
+		    [lreplace $_collapsed_group_list $idx [expr {$idx + 1}]]
 	    }
-	    lappend collapsed_group_list $bare_jid $tmpc
+	    lappend _collapsed_group_list $bare_jid $tmpc
 	}
 	if {![lempty $tmps]} {
-	    while {[set idx [lsearch -exact $show_offline_group_list $bare_jid]] >= 0} {
-		set show_offline_group_list \
-		    [lreplace $show_offline_group_list $idx [expr {$idx + 1}]]
+	    while {[set idx [lsearch -exact $_show_offline_group_list $bare_jid]] >= 0} {
+		set _show_offline_group_list \
+		    [lreplace $_show_offline_group_list $idx [expr {$idx + 1}]]
 	    }
-	    lappend show_offline_group_list $bare_jid $tmps
+	    lappend _show_offline_group_list $bare_jid $tmps
 	}
-	set custom::saved([namespace current]::collapsed_group_list) $collapsed_group_list
-	set custom::saved([namespace current]::show_offline_group_list) $show_offline_group_list
-	custom::store
     }
+
+    set collapsed_group_list $_collapsed_group_list
+    set show_offline_group_list $_show_offline_group_list
 }
 
 hook::add predisconnected_hook [namespace current]::roster::set_group_lists
 
-proc roster::process_item {connid jid name groups subsc ask category subtype} {
+proc roster::process_item {connid jid name groups subsc ask} {
     after cancel [namespace parent]::update_chat_titles
     after idle [namespace parent]::update_chat_titles
 }
@@ -222,6 +230,7 @@ proc roster::redraw {} {
     variable show_transport_user_icons
     variable undef_group_name
     variable chats_group_name
+    variable own_resources_group_name
     variable collapsed
     variable show_offline
 
@@ -353,17 +362,13 @@ proc roster::redraw {} {
 	}
 	if {$options(chats_group)} {
 	    set cgroup [list $chats_group_name]
-	    foreach chatid [array names chat::opened] {
-		set cid [chat::get_connid $chatid]
-		if {$cid == $connid} {
-		    set jid [chat::get_jid $chatid]
-		    lappend jidsingroup($cgroup) $jid
-		    if {[cequal [roster::itemconfig $connid $jid -isuser] ""]} {
-			roster::itemconfig $connid $jid -isuser 1
-			roster::itemconfig $connid $jid \
-			    -name [chat::get_nick $connid $jid chat]
-			roster::itemconfig $connid $jid -subsc none
-		    }
+	    foreach chatid [chat::opened $connid] {
+		set jid [chat::get_jid $chatid]
+		lappend jidsingroup($cgroup) $jid
+		if {[cequal [roster::itemconfig $connid $jid -isuser] ""]} {
+		    roster::itemconfig $connid $jid \
+			-name [chat::get_nick $connid $jid chat]
+		    roster::itemconfig $connid $jid -subsc none
 		}
 	    }
 	    if {[info exists jidsingroup($cgroup)]} {
@@ -372,6 +377,15 @@ proc roster::redraw {} {
 	    set groupsundergroup($cgroup) {}
 	    set jidsundergroup($cgroup) {}
 	}
+	if {$options(show_own_resources)} {
+	    set cgroup [list $own_resources_group_name]
+	    set jid [::jlib::connection_bare_jid $connid]
+	    set jidsingroup($cgroup) [list $jid]
+	    set groups [linsert $groups 0 [list [join $cgroup "\u0000"] $cgroup]]
+	    roster::itemconfig $connid $jid -subsc both
+	    set groupsundergroup($cgroup) {}
+	    set jidsundergroup($cgroup) {}
+	}
 	foreach group $groups {
 	    set group [lindex $group 1]
 	    set gid [list $connid $group]
@@ -512,7 +526,11 @@ proc roster::redraw {} {
 			} else {
 			    if {$numjids <= 1 && $category == "user" && \
 				    !$show_transport_user_icons} {
-				set status $jstat($jid)
+				if {[info exists jstat($jid)]} {
+				    set status $jstat($jid)
+				} else {
+				    set status [get_user_aliases_status $connid $jid]
+				}
 				set subsc [::roster::itemconfig $connid $jid -subsc]
 				if {([cequal $subsc from] || [cequal $subsc none]) && \
 					$status == "unavailable"} {
@@ -609,6 +627,8 @@ proc roster::get_jid_foreground {connid jid} {
 		return unavailable
 	    }
 	}
+	server  -
+	gateway -
 	service {
 	    return [get_service_foreground $connid $jid $type]
 	}
@@ -618,58 +638,6 @@ proc roster::get_jid_foreground {connid jid} {
     }
 }
 
-proc roster::get_jid_icon {connid jid} {
-    lassign [::roster::get_category_and_subtype $connid $jid] category type
-
-    switch -- $category {
-	"" {
-	    return [get_user_icon $connid $jid]
-	}
-	user {
-	    return [get_user_icon $connid $jid]
-	}
-	conference {
-	    if {[get_jid_status $connid $jid] != "unavailable"} {
-		return roster/conference/available
-	    }
-	    return roster/conference/unavailable
-	}
-	service {
-		return [get_service_icon $connid $jid $type]
-	}
-	default {
-	    return ""
-	}
-    }
-}
-
-proc roster::get_service_icon {connid service type} {
-    variable show_transport_icons
-
-    if {$show_transport_icons} {
-	switch -- $type {
-	    jud {return services/jud}
-	    sms {return services/sms}
-	}
-	if {![cequal [::roster::itemconfig $connid $service -subsc] none]} {
-	    set status [get_user_status $connid $service]
-	    if {![catch { image type services/$type/$status }]} {
-		return services/$type/$status
-	    } else {
-		return roster/user/$status
-	    }
-	} else {
-	    return roster/user/unsubscribed
-	}
-    } else {
-	if {![cequal [::roster::itemconfig $connid $service -subsc] none]} {
-	    return roster/user/[get_user_status $connid $service]
-	} else {
-	    return roster/user/unsubscribed
-	}
-    }
-}
-
 proc roster::get_service_foreground {connid service type} {
     switch -- $type {
 	jud {return ""}
@@ -698,10 +666,71 @@ proc roster::get_user_foreground {connid user} {
     }
 }
 
-proc roster::get_user_icon {connid user} {
-    variable show_transport_user_icons
+proc roster::get_jid_icon {connid jid {status ""}} {
+    lassign [::roster::get_category_and_subtype $connid $jid] category type
 
-    set status [get_user_aliases_status $connid $user]
+    switch -- $category {
+	"" -
+	user {
+	    if {$status == ""} {
+		set status [get_user_aliases_status $connid $jid]
+	    }
+	    return [get_user_icon $connid $jid $status]
+	}
+	conference {
+	    if {$status == ""} {
+		set status [get_jid_status $connid $jid]
+	    }
+	    if {$status != "unavailable"} {
+		return roster/conference/available
+	    }
+	    return roster/conference/unavailable
+	}
+	server  -
+	gateway -
+	service {
+	    if {$status == ""} {
+		set status [get_user_status $connid $jid]
+	    }
+	    return [get_service_icon $connid $jid $type $status]
+	}
+	default {
+	    if {$status == ""} {
+		set status [get_jid_status $connid $jid]
+	    }
+	    return [get_user_icon $connid $jid $status]
+	}
+    }
+}
+
+proc roster::get_service_icon {connid service type status} {
+    variable show_transport_icons
+
+    if {$show_transport_icons} {
+	switch -- $type {
+	    jud {return services/jud}
+	    sms {return services/sms}
+	}
+	if {![cequal [::roster::itemconfig $connid $service -subsc] none]} {
+	    if {![catch { image type services/$type/$status }]} {
+		return services/$type/$status
+	    } else {
+		return roster/user/$status
+	    }
+	} else {
+	    return roster/user/unsubscribed
+	}
+    } else {
+	if {![cequal [::roster::itemconfig $connid $service -subsc] none]} {
+	    return roster/user/$status
+	} else {
+	    return roster/user/unsubscribed
+	}
+    }
+}
+
+proc roster::get_user_icon {connid user status} {
+    variable show_transport_user_icons
 
     set subsc [::roster::itemconfig $connid $user -subsc]
     if {[cequal $subsc ""]} {
@@ -714,9 +743,14 @@ proc roster::get_user_icon {connid user} {
 	if {$show_transport_user_icons} {
 	    set service [server_from_jid $user]
 	    lassign [::roster::get_category_and_subtype $connid $service] category type
-	    switch -- $type {
-		jud {return services/jud}
-		sms {return services/sms}
+	    switch -glob -- $category/$type {
+		directory/* -
+		*/jud {
+		    return services/jud
+		}
+		*/sms {
+		    return services/sms
+		}
 	    }
 	    if {![catch { image type services/$type/$status }]} {
 		return services/$type/$status
@@ -769,6 +803,9 @@ proc roster::create {w args} {
 	    -grouppopup {set grouppopupproc $val}
 	    -singleclick {set singleclickproc $val}
 	    -doubleclick {set doubleclickproc $val}
+	    -draginitcmd {set draginitcmd $val}
+	    -dropovercmd {set dropovercmd $val}
+	    -dropcmd {set dropcmd $val}
 	}
     }
 
@@ -828,10 +865,19 @@ proc roster::create {w args} {
 
     bindscroll $w.canvas
 
-    if {$w == ".roster"} {
-	DropSite::register .roster.canvas -dropcmd [namespace current]::dropcmd \
-	    -droptypes {JID}
-	DragSite::register .roster.canvas -draginitcmd [namespace current]::draginitcmd
+    if {[info exists draginitcmd]} {
+	DragSite::register $w.canvas -draginitcmd $draginitcmd
+    }
+
+    set args {}
+    if {[info exists dropovercmd]} {
+	lappend args -dropovercmd $dropovercmd
+    }
+    if {[info exists dropcmd]} {
+	lappend args -dropcmd $dropcmd
+    }
+    if {![lempty $args]} {
+	eval [list DropSite::register $w.canvas -droptypes {JID}] $args
     }
 }
 
@@ -1155,17 +1201,17 @@ proc roster::user_popup_info {connid user i} {
     set subsc [::roster::itemconfig $connid $bare_user -subsc]
     if {($options(show_subscription) && ![cequal $subsc ""]) &&
 	    !([cequal $category1 conference] && [cequal $category user])} {
+	set subsc [format "\n\t[::msgcat::mc {Subscription:}] %s" $subsc]
 	set ask [::roster::itemconfig $connid $bare_user -ask]
 	if {![cequal $ask ""]} {
-	    set subsc " \[$subsc, $ask\]"
-	} else {
-	    set subsc " \[$subsc\]"
+	    set ask [format "  [::msgcat::mc {Ask:}] %s" $ask]
 	}
     } else {
 	set subsc ""
+	set ask ""
     }
 
-    set user_popup_info "$name$subsc: $status"
+    set user_popup_info "$name: $status$subsc$ask"
 
     if {!([cequal $category1 conference] && $i > 0) || \
 	    $options(show_conference_user_info)} {
@@ -1183,7 +1229,7 @@ proc roster::switch_only_online {args} {
 }
 
 proc roster::is_online {connid jid} {
-    if {[::roster::is_user $connid $jid]} {
+    if {[::roster::itemconfig $connid $jid -isuser]} {
 	switch -- [get_user_aliases_status $connid $jid] {
 	    unavailable {return 0}
 	    default {return 1}
@@ -1198,10 +1244,13 @@ proc roster::is_online {connid jid} {
 proc roster::add_remove_item_menu_item {m connid jid} {
     set rjid [roster::find_jid $connid $jid]
     if {$rjid == ""} {
-	set rjid [node_and_server_from_jid $jid]
+	set state disabled
+    } else {
+	set state normal
     }
     $m add command -label [::msgcat::mc "Remove from roster..."] \
-	-command [list ifacetk::roster::remove_item_dialog $connid $rjid]
+	-command [list ifacetk::roster::remove_item_dialog $connid $rjid] \
+	-state $state
 }
 
 hook::add chat_create_user_menu_hook \
@@ -1278,7 +1327,7 @@ proc roster::dropcmd {target source X Y op type data} {
 
     debugmsg roster "GG: $gr; $tags"
 
-    lassign $data jid category type name version fromgid
+    lassign $data _connid jid category type name version fromgid
     set subsc ""
 
     if {[info exists fromgid]} {
@@ -1344,7 +1393,7 @@ proc roster::draginitcmd {target x y top} {
 	set cjid [tag_to_jid $tag]
 	lassign $cjid connid jid
 
-	set data [list $jid \
+	set data [list $connid $jid \
 		      [::roster::itemconfig $connid $jid -category] \
 		      [::roster::itemconfig $connid $jid -subtype] \
 		      [::roster::itemconfig $connid $jid -name] {} \
@@ -1384,6 +1433,8 @@ proc roster::popup_menu {id} {
     switch -- $category {
 	user       {set menu [create_user_menu $connid $jid]}
 	conference {set menu [conference_popup_menu $connid $jid]}
+	server  -
+	gateway -
 	service    {set menu [service_popup_menu $connid $jid]}
 	default    {set menu [jid_popup_menu $connid $jid]}
     }
@@ -1467,6 +1518,10 @@ proc roster::add_menu_submenu {m prefix suffix jids} {
 	    set m2 [menu $m.$suffix2 -tearoff 0]
 	    $m add cascad -label $label -menu $m2
 	    add_menu_submenu $m2 $prefix .$suffix2 $jids
+	} elseif {![catch { $m1 entrycget $i -variable } var]} {
+	    # Can't distinguish checkbuttons and radiobuttons
+	    # Works only for checkbuttons
+	    add_checkbutton_submenu $m $prefix $suffix $i $label $jids
 	} else {
 	    add_command_submenu $m $prefix $suffix $i $label $jids
 	}
@@ -1503,6 +1558,38 @@ proc roster::add_command_submenu {m prefix suffix i label jids} {
     }
 }
 
+###############################################################################
+
+proc roster::add_checkbutton_submenu {m prefix suffix i label jids} {
+    set command_list {}
+    foreach jid $jids {
+	set m1 $prefix[jid_to_tag $jid]$suffix
+	set idx [$m1 index $label]
+	if {$idx != "none"} {
+	    set var [$m1 entrycget $idx -variable]
+	    set command [$m1 entrycget $idx -command]
+	    if {![lcontain $command_list [list $var $command]]} {
+		lappend command_list [list $var $command]
+	    }
+	}
+    }
+    if {[llength $command_list] > 1} {
+	set m2 [menu $m.$i -tearoff 0]
+	$m add cascad -label $label -menu $m2
+	foreach jid $jids {
+	    set m1 $prefix[jid_to_tag $jid]$suffix
+	    set idx [$m1 index $label]
+	    if {$idx != "none"} {
+		set var [$m1 entrycget $idx -variable]
+		set command [$m1 entrycget $idx -command]
+		$m2 add checkbutton -label $jid -variable $var -command $command
+	    }
+	}
+    } else {
+	lassign [lindex $command_list 0] var command
+	$m add checkbutton -label $label -variable $var -command $command
+    }
+}
 
 ###############################################################################
 
@@ -1632,6 +1719,15 @@ proc roster::create_group_popup_menu {connid name} {
 	-state $state
     $m add command -label [::msgcat::mc "Remove all users in group..."] \
 	-command [list [namespace current]::remove_users_group_dialog $connid $name]
+
+    set last [$m index end]
+
+    ::hook::run roster_group_popup_menu_hook $m $connid $name
+
+    if {[$m index end] > $last} {
+	$m insert [expr $last + 1] separator
+    }
+
     return $m
 }
 
@@ -1804,3 +1900,4 @@ proc roster::add_group_custom_presence_menu {m connid name} {
 
 ###############################################################################
 
+# vim:ts=8:sw=4:sts=4:noet
