@@ -1,4 +1,4 @@
-# $Id: iq.tcl,v 1.11 2006/03/17 04:29:42 aleksey Exp $
+# $Id: iq.tcl 881 2007-01-22 12:08:18Z sergei $
 
 namespace eval iq {
     variable options
@@ -23,7 +23,7 @@ proc iq::register_handler {type tag xmlns h} {
     set supported_ns [lrmdups $supported_ns]
 }
 
-proc iq::process_iq {connid from useid id type child} {
+proc iq::process_iq {connid from useid id type lang child} {
     variable handler
     variable options
 
@@ -50,7 +50,7 @@ proc iq::process_iq {connid from useid id type child} {
 	set_status [format [::msgcat::mc "%s request from %s"] $xmlns_short $from]
     }
     if {[info exists h]} {
-	set res [$h $connid $from $child]
+	set res [$h $connid $from $lang $child]
 
 	if {$res != {}} {
 	    switch -- [lindex $res 0] {
@@ -69,6 +69,9 @@ proc iq::process_iq {connid from useid id type child} {
 			-connection $connid \
 			-id $id
 		}
+		ignore {
+		    # Do nothing, swallow the request
+		}
 	    }
 	}
     } else {
@@ -81,10 +84,10 @@ proc iq::process_iq {connid from useid id type child} {
     }
 }
 
-proc client:iqreply {connid from useid id type child} {
-    debugmsg iq "IQREPLY: $from; $useid; $id; $type; $child"
+proc client:iqreply {connid from useid id type lang child} {
+    debugmsg iq "IQREPLY: $from; $useid; $id; $type; $lang; $child"
 
-    iq::process_iq $connid $from $useid $id $type $child
+    iq::process_iq $connid $from $useid $id $type $lang $child
 }
 
 
