@@ -1,6 +1,6 @@
 ######################################################################
 #
-# $Header: /home/cvs/tkabber/tkabber/jabberlib-tclxml/jabberlib.tcl,v 1.96 2006/06/12 20:38:16 aleksey Exp $
+# $Header$
 #
 # This is JabberLib (abbreviated jlib), the Tcl library for 
 # use in making Jabber clients.
@@ -102,7 +102,7 @@ namespace eval jlib {
 
     set lib(connections) {}
     set lib(connid) 0
-    set iq(num) 0
+    set iq(num) 1
 
     # Export procedures.
     #
@@ -210,7 +210,6 @@ proc jlib::connect {connid args} {
     set cacertstore ""
     set certfile ""
     set keyfile ""
-    set send_to 0
 
     foreach {attr val} $args {
 	switch -- $attr {
@@ -227,22 +226,21 @@ proc jlib::connect {connid args} {
 	    -cacertstore    {set cacertstore $val}
 	    -certfile       {set certfile $val}
 	    -keyfile        {set keyfile $val}
-	    -sendto         {set send_to $val}
 	}
     }
 
     if {$hosts == {}} {
-	set hosts [list [list $host $port]]
+	set hosts [list [list [idna::domain_toascii $host] $port]]
     }
 
     ::LOG "(jlib::connect) Server:'$server' ConnectionID:'$connid'"
 
     foreach hp $hosts {
 	if {[catch {
-		set host [idna::domain_toascii [lindex $hp 0]]
-		set port [lindex $hp 1]
-		eval [list transport::${transport}::connect $connid \
-							    $host $port] $args
+		eval [list transport::${transport}::connect \
+			   $connid \
+			   [lindex $hp 0] \
+			   [lindex $hp 1]] $args
 	    } sock]} {
 	    set error 1
 	} else {
@@ -271,8 +269,8 @@ proc jlib::connect {connid args} {
     set lib($connid,cacertstore) $cacertstore
     set lib($connid,certfile) $certfile
     set lib($connid,keyfile) $keyfile
-    set lib($connid,send_to) $send_to
     set lib($connid,disconnecting) 0
+    set lib($connid,bytes_counter) 0
 
     catch { unset lib($connid,features) }
     set lib($connid,version) 0.0
@@ -323,6 +321,19 @@ proc jlib::connect {connid args} {
 
 ######################################################################
 
+proc jlib::socket_ip {connid} {
+    variable lib
+
+    if {[info exists lib($connid,sck)] && \
+	![catch {fconfigure $lib($connid,sck) -sockname} sock]} {
+	return [lindex $sock 0]
+    } else {
+	return ""
+    }
+}
+
+######################################################################
+
 proc jlib::reset {connid} {
     variable lib
 
@@ -418,6 +429,9 @@ proc jlib::disconnect {{connections {}}} {
     }
 
     foreach connid $connections {
+	set idx [lsearch -exact $lib(connections) $connid]
+	if {$idx < 0} continue
+
 	cancel_keepalive $connid
 	outmsg [wrapper:streamtrailer] -connection $connid
 	set lib($connid,disconnecting) 1
@@ -426,9 +440,8 @@ proc jlib::disconnect {{connections {}}} {
 	    transport::$lib($connid,transport)::close $connid
 	}
 
-	clear_vars $connid
-	set idx [lsearch -exact $lib(connections) $connid]
 	set lib(connections) [lreplace $lib(connections) $idx $idx]
+	clear_vars $connid
     }
 
     if {$lib(connections) == {}} {
@@ -461,6 +474,7 @@ proc jlib::got_stream {connid vars} {
 		       -vars [list xmlns $::NS(iq-auth)]]]
     }
     set lib($connid,sessionid) $sessionid
+    set lib($connid,bytes_counter) 0
 }
 
 ######################################################################
@@ -582,6 +596,11 @@ proc jlib::inmsg {connid msg eof} {
     variable keep_alive_id
     variable lib
 
+    if {[lsearch -exact $lib(connections) $connid] < 0} {
+	::LOG "error (jlib::inmsg) Connection $connid doesn't exist"
+	return -1
+    }
+
     # TODO
     if {$keep_alive} {
 	cancel_keepalive $connid
@@ -589,6 +608,8 @@ proc jlib::inmsg {connid msg eof} {
 			       [namespace current]::out_keepalive $connid]
     }
 
+    incr lib($connid,bytes_counter) [string bytelength $msg]
+
     ::LOG "(jlib::inmsg) ($connid) '$msg'"
     ::LOG_INPUT $connid $msg
     wrapper:parser $lib($connid,wrap) parse $msg
@@ -665,12 +686,13 @@ proc jlib::clear_iqs {} {
 
     foreach id [array names iq] {
 	if {$id != "num"} {
-	    uplevel #0 $iq($id) [list DISCONNECT [::msgcat::mc "Disconnected"]]
+	    set cmd $iq($id)
 	    unset iq($id)
+	    uplevel #0 $cmd [list DISCONNECT [::msgcat::mc "Disconnected"]]
 	}
     }
 
-    set iq(num) 0
+    set iq(num) 1
 }
 
 ######################################################################
@@ -724,13 +746,45 @@ proc jlib::connection_server {connid} {
     variable connjid
 
     if {[info exists lib($connid,sasltoken)]} {
-	set server   [$lib($connid,sasltoken) cget -server]
+	set server [$lib($connid,sasltoken) cget -server]
 	return $server
     } else {
 	return $connjid($connid,server)
     }
 }
 
+proc jlib::connection_resource {connid} {
+    variable lib
+    variable connjid
+
+    if {[info exists lib($connid,sasltoken)]} {
+	set resource [$lib($connid,sasltoken) cget -resource]
+	return $resource
+    } else {
+	return $connjid($connid,resource)
+    }
+}
+
+######################################################################
+
+proc jlib::connection_requested_user {connid} {
+    variable connjid
+
+    return $connjid($connid,user)
+}
+
+proc jlib::connection_requested_server {connid} {
+    variable connjid
+
+    return $connjid($connid,server)
+}
+
+proc jlib::connection_requested_resource {connid} {
+    variable connjid
+
+    return $connjid($connid,resource)
+}
+
 ######################################################################
 
 proc jlib::register_xmlns {connid xmlns callback} {
@@ -802,7 +856,14 @@ proc jlib::element_callback {connid element} {
 ######################################################################
 
 proc jlib::parse {connid xmldata} {
+
+    variable lib
+    set size 0
+    catch {set size $lib($connid,bytes_counter) }
+    set lib($connid,bytes_counter) 0
+    
     after idle [list [namespace current]::parse1 $connid $xmldata]
+    after idle [list ::LOG_INPUT_SIZE $connid $xmldata $size]
 }
 
 proc jlib::parse1 {connid xmldata} {
@@ -822,11 +883,12 @@ proc jlib::parse1 {connid xmldata} {
 
     wrapper:splitxml $xmldata tag vars isempty chdata children
 
-    set usefrom 0
-    set from ""
     if {[wrapper:isattr $vars from]} {
 	set usefrom 1
 	set from [wrapper:getattr $vars from]
+    } else {
+	set usefrom 0
+	set from ""
     }
 
     set xmlns [wrapper:getattr $vars xmlns]
@@ -841,6 +903,12 @@ proc jlib::parse1 {connid xmldata} {
 	return
     }
 
+    if {[wrapper:isattr $vars xml:lang]} {
+	set lang [wrapper:getattr $vars xml:lang]
+    } else {
+	set lang en
+    }
+
     switch -- $tag {
 	iq {
 	    set useid 0
@@ -956,13 +1024,7 @@ proc jlib::parse1 {connid xmldata} {
 		    return
 		}
 
-		if ($lib($connid,send_to)) {
-		    set param [list -to [wrapper:getattr $vars to]]
-		} else {
-		    set param {}
-		}
-		eval [list client iqreply $connid $from $useid \
-			   $id $type $child] $param
+		client iqreply $connid $from $useid $id $type $lang $child
 	    }
 	}
 	message {
@@ -1006,13 +1068,8 @@ proc jlib::parse1 {connid xmldata} {
 		}
 	    }
 
-	    if ($lib($connid,send_to)) {
-		set param [list -to [wrapper:getattr $vars to]]
-	    } else {
-		set param {}
-            }
-	    eval [list client message $connid $from $id $type $is_subject \
-		       $subject $body $err $thread $priority $x] $param
+	    client message $connid $from $id $type $is_subject \
+			   $subject $body $err $thread $priority $x
 	}
 	presence {
 	    set type [wrapper:getattr $vars type]
@@ -1027,9 +1084,6 @@ proc jlib::parse1 {connid xmldata} {
 	    set x        ""
 
 	    set param    ""
-	    if ($lib($connid,send_to)) {
-		lappend param -to [wrapper:getattr $vars to]
-	    }
 
 	    if {[wrapper:isattr $vars id]} {
 		set id [wrapper:getattr $vars id]
@@ -1048,7 +1102,11 @@ proc jlib::parse1 {connid xmldata} {
 		wrapper:splitxml $child ctag cvars cisempty cchdata cchildren
 
 		switch -- $ctag {
-		    status   {lappend param -status   $cchdata}
+		    status   {
+			if {$type != "error"} {
+			    lappend param -status $cchdata
+			}
+		    }
 		    priority {lappend param -priority $cchdata}
 		    meta     {lappend param -meta     $cchdata}
 		    icon     {lappend param -icon     $cchdata}
@@ -1056,15 +1114,17 @@ proc jlib::parse1 {connid xmldata} {
 		    loc      {lappend param -loc      $cchdata}
 		    x        {lappend x $child}
 		    error {
-			set errcode [wrapper:getattr $cvars code]
-			set errtype [wrapper:getattr $cvars type]
-			if {$errtype == ""} {
-			    set err [list $errcode $cchdata]
-			} else {
-			    set err [list $errtype $child]
+			if {$type == "error"} {
+			    set errcode [wrapper:getattr $cvars code]
+			    set errtype [wrapper:getattr $cvars type]
+			    if {$errtype == ""} {
+				set err [list $errcode $cchdata]
+			    } else {
+				set err [list $errtype $child]
+			    }
+			    lappend param -status [lindex [stanzaerror::error_to_list $err] 2]
+			    lappend param -error [lrange [stanzaerror::error_to_list $err] 0 1]
 			}
-			lappend param -status [lindex [stanzaerror::error_to_list $err] 2]
-			lappend param -error [lrange [stanzaerror::error_to_list $err] 0 1]
 		    }
 		}
 	    }
@@ -1141,12 +1201,6 @@ proc jlib::parse_roster_get {connid ispush cmd type data} {
 		set name  [wrapper:getattr $cvars name]
 		set subsc [wrapper:getattr $cvars subscription]
 		set ask   [wrapper:getattr $cvars ask]
-		if {$ctag != {item}} {
-		    set category $ctag
-		} else {
-		    set category [wrapper:getattr $cvars category]
-		}
-		set subtype [wrapper:getattr $cvars type]
 		
 		foreach subchild $cchildren {
 		    wrapper:splitxml $subchild subtag tmp tmp subchdata tmp
@@ -1171,17 +1225,10 @@ proc jlib::parse_roster_get {connid ispush cmd type data} {
 		add_connection_route $connid $jid
 
 		# ...and call client procedures
-		if ($lib($connid,send_to)) {
-		    set param [list -to [wrapper:getattr $vars to]]
-		} else {
-		    set param {}
-		}
 		if $ispush {
-		    eval [list client roster_push $connid $jid $name $groups \
-			       $subsc $ask $category $subtype] $param
+		    client roster_push $connid $jid $name $groups $subsc $ask
 		} else {
-		    eval [list client roster_item $connid $jid $name $groups \
-			       $subsc $ask $category $subtype] $param
+		    client roster_item $connid $jid $name $groups $subsc $ask
 		}
 	    }
 	}
@@ -1329,10 +1376,6 @@ proc jlib::send_iq {type xmldata args} {
     variable iq
 
     ::LOG "(jlib::send_iq) type:'$type'"
-    if {$lib(connections) == {}} {
-          ::LOG "error (jlib::send_iq) No connection"
-          return -1
-    }
 
     set useto  0
     set useid  0
@@ -1340,10 +1383,16 @@ proc jlib::send_iq {type xmldata args} {
     set id     {}
     set cmd    [namespace current]::noop
     set vars   {}
+    set timeout 0
 
     foreach {attr val} $args {
 	switch -- $attr {
 	    -command    {set cmd $val}
+	    -timeout {
+		if {$val > 0} {
+		    set timeout $val
+		}
+	    }
 	    -to         {set useto 1; set to $val}
 	    -id         {set useid 1; set id $val}
 	    -connection {set connid $val}
@@ -1351,7 +1400,15 @@ proc jlib::send_iq {type xmldata args} {
 	}
     }
     if {![info exists connid]} {
-	set connid [route $to]
+	return -code error "jlib::send_iq: -connection is mandatory"
+    }
+
+    if {[lsearch [connections] $connid] < 0} {
+        ::LOG "error (jlib::send_iq) Connection $connid doesn't exist"
+	if {$cmd != ""} {
+	    uplevel #0 $cmd [list DISCONNECT [::msgcat::mc "Disconnected"]]
+	}
+	return -1
     }
 
     if {$type != "set" && $type != "result" && $type != "error"} {
@@ -1368,6 +1425,9 @@ proc jlib::send_iq {type xmldata args} {
     if {$type == "get" || $type == "set"} {
 	lappend vars id $iq(num)
 	set iq($iq(num)) $cmd
+	if {$timeout > 0} {
+	    after $timeout [list [namespace current]::iq_timeout $iq(num)]
+	}
 	incr iq(num)
     } elseif {$useid} {
 	lappend vars id $id
@@ -1384,7 +1444,22 @@ proc jlib::send_iq {type xmldata args} {
 	set data [wrapper:createtag iq -vars $vars]
     }
     ::LOG_OUTPUT_XML $connid $data
-    outmsg [wrapper:createxml $data] -connection $connid
+    set text [wrapper:createxml $data] 
+    ::LOG_OUTPUT_SIZE $connid $data [string bytelength $text] 
+    outmsg $text -connection $connid 
+}
+
+######################################################################
+
+proc jlib::iq_timeout {id} {
+    variable iq
+
+    ::LOG "(jlib::iq_timeout) id: $id"
+    if {[info exists iq($id)]} {
+	set cmd $iq($id)
+	unset iq($id)
+	uplevel #0 $cmd [list TIMEOUT [::msgcat::mc "Timeout"]]
+    }
 }
 
 ######################################################################
@@ -1431,7 +1506,7 @@ proc jlib::add_connection_route {connid jid} {
 
 ######################################################################
 # TODO
-proc jlib::send_create {user pass name email cmd} {
+proc jlib::send_create {connid user pass name email cmd} {
     variable lib
 
     ::LOG "(jlib::send_create) username:'$user' password:'$pass' name:'$name' email:'$email'"
@@ -1449,6 +1524,7 @@ proc jlib::send_create {user pass name email cmd} {
 				[wrapper:createtag password -chdata $pass]]]
 
     send_iq set $data \
+	-connection $connid \
 	-command [list [namespace current]::parse_send_create $cmd]
 }
 
@@ -1502,7 +1578,9 @@ proc jlib::send_msg {to args} {
 
     set data [wrapper:createtag message -vars $vars -subtags $children]
     ::LOG_OUTPUT_XML $connid $data
-    outmsg [wrapper:createxml $data] -connection $connid
+    set text [wrapper:createxml $data] 
+    ::LOG_OUTPUT_SIZE $connid $data [string bytelength $text] 
+    outmsg $text -connection $connid 
 }
 
 ######################################################################
@@ -1580,7 +1658,9 @@ proc jlib::send_presence {args} {
 
     set data [wrapper:createtag presence -vars $vars -subtags $children]
     ::LOG_OUTPUT_XML $connid $data
-    outmsg [wrapper:createxml $data] -connection $connid
+    set text [wrapper:createxml $data] 
+    ::LOG_OUTPUT_SIZE $connid $data [string bytelength $text] 
+    outmsg $text -connection $connid 
 }
 
 ######################################################################
@@ -1617,7 +1697,7 @@ proc jlib::roster_get {args} {
 }
 
 ######################################################################
-proc jlib::roster_set {item args} {
+proc jlib::roster_set {connid item args} {
     variable lib
     variable roster
 
@@ -1665,11 +1745,12 @@ proc jlib::roster_set {item args} {
 					  -subtags $subdata]]]
 
     send_iq set $xmldata \
+	-connection $connid \
 	-command [list [namespace current]::parse_roster_set $item $cmd $groups $name]
 }
 
 ######################################################################
-proc jlib::roster_del {item args} {
+proc jlib::roster_del {connid item args} {
     variable lib
     variable roster
 
@@ -1694,9 +1775,45 @@ proc jlib::roster_del {item args} {
 						     subscription remove]]]]
 
     send_iq set $xmldata \
+	-connection $connid \
 	-command [list [namespace current]::parse_roster_del $item $cmd]
 }
 
+######################################################################
+#
+proc jlib::x_delay {xml} {
+    foreach xelem $xml {
+	jlib::wrapper:splitxml $xelem tag vars isempty chdata children
+
+	switch -- [jlib::wrapper:getattr $vars xmlns] {
+	    urn:xmpp:delay {
+		# 2006-07-17T05:29:12Z
+		# 2006-11-18T03:35:56.415699Z
+		if {[regsub {(\d+)-(\d\d)-(\d\d)T(\d+:\d+:\d+)[^Z]*Z?} \
+			    [jlib::wrapper:getattr $vars stamp] \
+			    {\1\2\3T\4} \
+			    stamp]} {
+		    if {![catch {clock scan $stamp -gmt 1} seconds]} {
+			return $seconds
+		    }
+		}
+	    }
+	    jabber:x:delay {
+		# 20060717T05:29:12
+		# 20061118T03:35:56.415699
+		if {[regexp {\d+\d\d\d\dT\d+:\d+:\d+} \
+			    [jlib::wrapper:getattr $vars stamp] \
+			    stamp]} {
+		    if {![catch {clock scan $stamp -gmt 1} seconds]} {
+			return $seconds
+		    }
+		}
+	    }
+	}
+    }
+    return [clock seconds]
+}
+
 ######################################################################
 #
 proc ::LOG {text} {
@@ -1708,8 +1825,10 @@ proc ::LOG {text} {
 
 proc ::LOG_OUTPUT     {connid t} {}
 proc ::LOG_OUTPUT_XML {connid x} {}
+proc ::LOG_OUTPUT_SIZE {connid x size} {}
 proc ::LOG_INPUT      {connid t} {}
 proc ::LOG_INPUT_XML  {connid x} {}
+proc ::LOG_INPUT_SIZE {connid x size} {}
 
 ######################################################################
 proc jlib::noop {args} {}
@@ -1717,7 +1836,11 @@ proc jlib::noop {args} {}
 ######################################################################
 proc jlib::get_lang {} {
     set prefs [::msgcat::mcpreferences]
-    set lang [lindex $prefs end]
+      if {[info tclversion] > 8.4} {
+          set lang [lindex $prefs end-1]
+      } else {
+          set lang [lindex $prefs end]
+      }
     switch -- $lang {
 	"" -
 	c -
@@ -1725,7 +1848,11 @@ proc jlib::get_lang {} {
 	    return en
 	}
     }
-    set lang2 [lindex $prefs end-1]
+      if {[info tclversion] > 8.4} {
+          set lang2 [lindex $prefs end-2]
+      } else {
+          set lang2 [lindex $prefs end-1]
+      }
     if {[regexp {^([A-Za-z]+)_([0-9A-Za-z]+)} $lang2 ignore l1 l2]} {
 	return "[string tolower $l1]-[string toupper $l2]"
     } else {
@@ -1738,5 +1865,5 @@ proc jlib::get_lang {} {
 # Now that we're done...
 #
 
-package provide jabberlib 0.9.0
+package provide jabberlib 0.10.0
 
