@@ -6,7 +6,7 @@
 #  Copyright (c) 2005 Sergei Golovan <sgolovan@nes.ru>
 #  Based on jlibsasl by Mats Bengtson
 #  
-# $Id: jlibsasl.tcl,v 1.5 2006/04/27 20:29:35 aleksey Exp $
+# $Id: jlibsasl.tcl 771 2006-10-25 11:17:05Z sergei $
 #
 # SYNOPSIS
 #   jlibsasl::new connid args
@@ -53,10 +53,11 @@ namespace eval jlibsasl {
     variable uid 0
     variable saslpack
 
-    if {![catch { package require sasl 1.0 }]} {
+    if {![catch {package require sasl 1.0}]} {
 	set saslpack tclsasl
-    } elseif {![catch { package require SASL 1.0 }]} {
-	catch { package require SASL::NTLM }
+    } elseif {![catch {package require SASL 1.0}]} {
+	catch {package require SASL::NTLM}
+	catch {package require SASL::XGoogleToken}
 	set saslpack tcllib
     } else {
 	return -code error "No SASL package found"
@@ -100,7 +101,7 @@ proc jlibsasl::new {connid args} {
 
     set state(-connid) $connid
     set state(-allow_plain) 0
-    catch { unset state(-mechanisms) }
+    catch {unset state(-mechanisms)}
 
     proc $token {cmd args} \
 	"eval {[namespace current]::\$cmd} {$token} \$args"
@@ -135,8 +136,8 @@ proc jlibsasl::free {token} {
 	}
     }
 
-    catch { unset state }
-    catch { rename $token "" }
+    catch {unset state}
+    catch {rename $token ""}
 }
 
 ##########################################################################
@@ -264,6 +265,8 @@ proc jlibsasl::auth {token args} {
 			   -server $state(-server) \
 			   -callback [list [namespace code tcllib_callback] \
 					   $token]]
+	    # Workaround a bug 1545306 in tcllib SASL module
+	    set ::SASL::digest_md5_noncecount 0
 	}
     }
 
@@ -302,18 +305,16 @@ proc jlibsasl::auth_continue {token} {
 	    } result]
 	}
 	tcllib {
-	    set code [catch { choose_mech $token } result]
+	    set code [catch {choose_mech $token} result]
 
 	    if {!$code} {
 		set mech $result
 		SASL::configure $state(-token) -mech $mech
 		switch -- $mech {
-		    PLAIN {
+		    PLAIN -
+		    X-GOOGLE-TOKEN {
 			# Initial responce
-			set code \
-			    [catch {SASL::step $state(-token) ""} \
-				 result]
-
+			set code [catch {SASL::step $state(-token) ""} result]
 			if {!$code} {
 			    set output [SASL::response $state(-token)]
 			}
@@ -392,7 +393,7 @@ proc jlibsasl::step {token serverin64} {
 	    } result]
 	}
 	tcllib {
-	    set code [catch { SASL::step $state(-token) $serverin } result]
+	    set code [catch {SASL::step $state(-token) $serverin} result]
 
 	    if {!$code} {
 		set result [SASL::response $state(-token)]
@@ -585,7 +586,7 @@ proc jlibsasl::send_session {token res xmldata} {
 		-connection $state(-connid)
 	}
 	default {
-	    ::LOG "error (jlibsasl::send_session) $data"
+	    ::LOG "error (jlibsasl::send_session) $xmldata"
 	    finish $token $res $xmldata
 	}
     }
@@ -604,7 +605,9 @@ proc jlibsasl::finish {token res child} {
     } else {
 	jlib::client status [::msgcat::mc "Authentication successful"]
     }
-    uplevel #0 $state(-command) [list $res $child]
+    if {[info exists state(-command)]} {
+	uplevel #0 $state(-command) [list $res $child]
+    }
 }
 
 ##########################################################################
