@@ -1,4 +1,4 @@
-# $Id: joingrdialog.tcl,v 1.46 2005/08/11 20:21:23 aleksey Exp $
+# $Id: joingrdialog.tcl 834 2006-12-24 16:30:40Z sergei $
 
 #set gr_nick $user
 set gr_group "talks"
@@ -127,10 +127,6 @@ proc join_group1 {gw} {
     set gr_nick_list [update_combo_list $gr_nick_list $gr_nick 10]
     set gr_group_list [update_combo_list $gr_group_list $gr_group 20]
     set gr_server_list [update_combo_list $gr_server_list $gr_server 10]
-    set custom::saved(::::gr_nick_list) $gr_nick_list
-    set custom::saved(::::gr_group_list) $gr_group_list
-    set custom::saved(::::gr_server_list) $gr_server_list
-    custom::store
 
     foreach c [jlib::connections] {
 	if {[jlib::connection_jid $c] == $gr_connid} {
@@ -165,6 +161,14 @@ proc join_group {jid args} {
 	set nick [get_group_nick $jid $gr_nick]
     }
 
+    set cw [chat::winid [chat::chatid $connid $jid]]
+    if {[winfo exists $cw]} {
+	if {$::usetabbar} {
+	    .nb raise [ifacetk::nbpage $cw]
+	}
+	focus -force $cw.input
+    }
+
     muc::join_group $connid $jid $nick $password
 }
 
@@ -173,14 +177,48 @@ proc set_our_groupchat_nick {group nick} {
 
     set group [tolower_node_and_domain $group]
     set groupchats(nick,$group) $nick
-    debugmsg conference [array get groupchats]
 }
 
 proc get_our_groupchat_nick {group} {
     global groupchats
     
-    debugmsg conference [array get groupchats]
     debugmsg conference "GET NICK: [list $group]"
     return $groupchats(nick,$group)
 }
 
+###############################################################################
+
+proc joingroup_disco_node_menu_setup {m bw tnode data parentdata} {
+    lassign $data type connid jid node
+    lassign $parentdata ptype pconnid pjid pnode
+    switch -- $type {
+	item {
+	    set identities [disco::get_jid_identities $connid $jid $node]
+	    set pidentities [disco::get_jid_identities $pconnid $pjid $pnode]
+
+	    # JID with resource is not a room JID
+	    if {[node_and_server_from_jid $jid] != $jid} return
+
+	    if {[lempty $identities]} {
+		set identities $pidentities
+	    }
+
+	    foreach id $identities {
+		if {[jlib::wrapper:getattr $id category] == "conference"} {
+		    $m add command -label [::msgcat::mc "Join group..."] \
+			-command [list join_group_dialog \
+				       -server [server_from_jid $jid] \
+				       -group [node_from_jid $jid] \
+				       -connection $connid]
+		    break
+		}
+	    }
+	}
+    }
+}
+
+hook::add disco_node_menu_hook \
+	  [namespace current]::joingroup_disco_node_menu_setup 45
+
+###############################################################################
+
