@@ -1,4 +1,4 @@
-# $Id: login.tcl,v 1.85 2006/05/11 22:07:25 aleksey Exp $
+# $Id: login.tcl 1038 2007-03-10 14:09:40Z sergei $
 
 
 if {[lcontain [jlib::capabilities transport] tls]} {
@@ -86,7 +86,7 @@ if {$use_tls} {
 if {$use_tls || $have_compress} {
     custom::defvar loginconf(stream_options) plaintext \
 	[::msgcat::mc "XMPP stream options when connecting to server."] \
-	-group Login -type radio -layout horizontal \
+	-group Login -type options \
 	-values $values
 }
 
@@ -153,7 +153,7 @@ if {$have_http_poll} {
 before creating new key sequence."] \
 	-group Login -type integer
     custom::defvar loginconf(polltimeout) 0 \
-	[::msgcat::mc "Timeout for waiting for HTTP poll responces (if set\
+	[::msgcat::mc "Timeout for waiting for HTTP poll responses (if set\
 to zero, Tkabber will wait forever)."] \
 	-group Login -type integer
     custom::defvar loginconf(pollmin) 3000 \
@@ -185,6 +185,9 @@ proc login {logindata} {
 	if {$lc(connect_forever)} {
 	    login_retry $logindata
 	} else {
+	    if {[winfo exists .connect_err]} {
+		destroy .connect_err
+	    }
 	    set res [MessageDlg .connect_err -width 600 -icon error \
 		-message [format [::msgcat::mc "Failed to connect: %s"] $connid] \
 		-type user -buttons [list abort [::msgcat::mc "Keep trying"]] \
@@ -198,7 +201,7 @@ proc login {logindata} {
 	return
     }
     # OK, connected.
-    debugmsg login "Connect successful ($user)"
+    debugmsg login "Connect successful ($user) $connid"
     set login_after_time 15000
     login_login $logindata $connid
 }
@@ -284,10 +287,6 @@ proc login_connect {logindata} {
 
     array set lc $logindata
 
-    if {$lc(replace_opened) && [jlib::connections] != {}} {
-	return -1
-    }
-
     set connid [jlib::new -user $lc(user) \
 			  -server $lc(server) \
 			  -resource $lc(resource)]
@@ -309,11 +308,13 @@ proc login_connect {logindata} {
 	lappend args -proxyuseragent $lc(httpuseragent)
     }
 
+    set ascii_server [idna::domain_toascii $lc(server)]
+
     if {$have_http_poll && $lc(usehttppoll)} {
 	if {$lc(pollurl) != ""} {
 	    set url $lc(pollurl)
 	} else {
-	    set url [jlibdns::get_http_poll_url $lc(server)]
+	    set url [jlibdns::get_http_poll_url $ascii_server]
 	}
 	return [eval [list jlib::connect $connid \
 				-transport http_poll \
@@ -332,9 +333,9 @@ proc login_connect {logindata} {
 	if {$lc(usealtserver)} {
 	    set hosts {}
 	} else {
-	    set hosts [jlibdns::get_addr_port $lc(server)]
+	    set hosts [jlibdns::get_addr_port $ascii_server]
 	    if {[lempty $hosts]} {
-		set hosts [list [list $lc(server) 5222]]
+		set hosts [list [list $ascii_server 5222]]
 	    }
 	}
 	set transport tcp
@@ -369,7 +370,7 @@ proc login_connect {logindata} {
 	}
 
 	if {$lc(usealtserver)} {
-	    set hosts [list [list $lc(altserver) $lc(altport)]]
+	    set hosts [list [list [idna::domain_toascii $lc(altserver)] $lc(altport)]]
 	}
 
 	return [eval [list jlib::connect $connid \
@@ -433,6 +434,10 @@ proc client:reconnect {connid} {
     global loginconf_hist
 
     debugmsg login "RECONNECT $connid"
+
+    hook::run predisconnected_hook $connid
+
+    jlib::disconnect $connid
     roster::clean_connection $connid
 
     if {[jlib::connections] == {}} {
@@ -440,6 +445,7 @@ proc client:reconnect {connid} {
     }
 
     disconnected $connid
+
     if {[incr reconnect_retries] <= 3} {
         after 1000 [list login $loginconf_hist($connid)]
     }
@@ -485,6 +491,7 @@ proc recv_auth_result {connid logindata res args} {
 					     -chdata $lc(user)] \
 					[jlib::wrapper:createtag password \
 					     -chdata $lc(password)]]] \
+		    -connection $connid \
 		    -command [list recv_register_result $connid $logindata]
 		return
 	    }
@@ -576,6 +583,8 @@ proc send_change_password {} {
 	return
     }
 
+    set connid [jlib::route $loginconf(server)]
+
     jlib::send_iq set \
 	    [jlib::wrapper:createtag query \
 		    -vars {xmlns jabber:iq:register} \
@@ -584,6 +593,7 @@ proc send_change_password {} {
 				   [jlib::wrapper:createtag password \
 					-chdata $password]]] \
 	    -to $loginconf(server) \
+	    -connection $connid \
 	    -command recv_change_password_result
 }
 
@@ -658,8 +668,6 @@ proc logout_reason {} {
     global logoutuserstatus logouttextstatus logoutpriority reason reasonlist
 
     set reasonlist [update_combo_list $reasonlist $reason 10]
-    set custom::saved(::::reasonlist) $reasonlist
-    custom::store
 
     set lw .logout
     $lw withdraw
