@@ -1,4 +1,4 @@
-# $Id: messages.tcl,v 1.97 2006/05/20 21:36:52 aleksey Exp $
+# $Id: messages.tcl 1099 2007-04-12 06:44:00Z sergei $
 
 namespace eval message {
     variable msgid 0
@@ -37,7 +37,7 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
             $subject $body $x
     }
 
-    set mw .msgshow$msgid
+    set mw .msgshow[incr msgid]
     toplevel $mw -class Message
     wm group $mw .
 
@@ -79,6 +79,8 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
     frame $mw.frame
     pack $mw.frame -side top -fill both -expand yes -padx 2m -pady 2m
 
+    label $mw.thread -text $thread
+
     if {$replyP} {
         set title [::msgcat::mc "Message from:"]
     } else {
@@ -140,8 +142,8 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 
     incr row
     set last $row
-    lassign [hook::foldl message_process_x_hook [list $row $body] \
-	    $mw.rf $x $connid $from $type $replyP] row body
+    hook::run message_process_x_hook row body \
+	      $mw.rf $x $connid $from $type $replyP
 
     if {(!$replyP) && ($row == $last)} {
         destroy $mw
@@ -155,9 +157,8 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 
     ScrolledWindow $mw.rsw
     text $mw.rbody -width 60 -height 8 -wrap word
-    $mw.rbody tag configure emphasized -elide 1
-    $mw.rbody tag configure nonemphasized -elide 0
-    ::chat::add_emoteiconed_text $mw.rbody $body ""
+    ::richtext::config $mw.rbody -using {url emoticon stylecode}
+    ::richtext::render_message $mw.rbody $body ""
     $mw.rbody configure -state disabled
     pack $mw.rsw -side top -fill both -expand yes -in $mw.frame
     pack $mw.rbody -side top -fill both -expand yes -in $mw.rsw
@@ -195,8 +196,6 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 
         textUndoable $mw.body -width 60 -height 8 -wrap word -font $font
         pack $mw.body -side top -fill both -expand yes -in $mw.sw
-        bind $mw.body <Control-z> { %W undo }
-        bind $mw.body <Control-Z> { %W redo }
         bind $mw.body <Control-Return> "ButtonBox::invoke $bbox 0
                 break"
         $mw.sw setwidget $mw.body
@@ -206,8 +205,7 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 	hook::run open_message_post_hook $mw $connid $from
     }
 
-    incr msgid
-
+    BWidget::place $mw 0 0 center
     wm deiconify $mw
 }
 
@@ -217,8 +215,9 @@ proc message::quote {mw body} {
     $mw.body insert insert "\n"
 }
 
-proc message::process_x {acc f x connid from type replyP} {
-    lassign $acc row body
+proc message::process_x {rowvar bodyvar f x connid from type replyP} {
+    upvar 2 $rowvar row
+    upvar 2 $bodyvar body
 
     foreach xa $x {
 	jlib::wrapper:splitxml $xa tag vars isempty chdata children
@@ -229,11 +228,13 @@ proc message::process_x {acc f x connid from type replyP} {
 		set row [process_x_oob $f $children $row $from]
 	    } \
 	    $::NS(data) {
-		process_x_data $f $from $x
+		if {![lempty $children]} {
+		    process_x_data $f $from $xa
+		}
 	    } \
 	}
 
-    return [list $row $body]
+    return
 }
 
 hook::add message_process_x_hook message::process_x 99
@@ -245,24 +246,24 @@ proc message::process_x_oob {f x row from} {
         jlib::wrapper:splitxml $item tag vars isempty chdata children   
 
         switch -- $tag {
-            desc - url {
+            desc -
+	    url {
                 set $tag $chdata
             }
         }
     }
-    if {([cequal $desc ""]) || ([cequal $url ""])} {
+    if {[cequal $url ""]} {
         return $row
     }
+    if {[cequal $desc ""]} {
+	set desc $url
+    }
 
-    label $f.luser$row -text [::msgcat::mc "Attached file:"]
-    set cb [button $f.user$row -text $url \
-                -command [list ft::http::recv_file_dialog \
-			      $from [list $url] $desc]]
-    bind $cb <Any-Enter>  [list balloon::default_balloon $cb enter  %X %Y \
-                                $desc]
-    bind $cb <Any-Motion> [list balloon::default_balloon $cb motion %X %Y \
-                                $desc]
-    bind $cb <Any-Leave>  [list balloon::default_balloon $cb leave  %X %Y]
+    label $f.luser$row -text [::msgcat::mc "Attached URL:"]
+    set cb [button $f.user$row \
+		   -text $desc \
+		   -command [list browseurl $url]]
+    balloon::setup $cb -text $url
     grid $f.luser$row -row $row -column 0 -sticky e
     grid $f.user$row  -row $row -column 1 -sticky ew
 
@@ -282,18 +283,22 @@ proc message::send_x_data {to w restags} {
     destroy $w
 }
 
-proc message::send_dialog_item {state m connid jid} {
+proc message::send_dialog_item {m connid jid} {
     $m add command -label [::msgcat::mc "Send message..."] \
-	-state $state \
 	-command [list message::send_dialog \
 		       -to $jid -connection $connid]
 }
 
-hook::add roster_jid_popup_menu_hook {message::send_dialog_item normal} 15
-hook::add message_dialog_menu_hook {message::send_dialog_item normal} 15
-hook::add search_popup_menu_hook {message::send_dialog_item normal} 15
-hook::add roster_create_groupchat_user_menu_hook \
-    {message::send_dialog_item disabled} 15
+hook::add roster_jid_popup_menu_hook message::send_dialog_item 15
+hook::add message_dialog_menu_hook message::send_dialog_item 15
+hook::add search_popup_menu_hook message::send_dialog_item 15
+hook::add roster_create_groupchat_user_menu_hook message::send_dialog_item 15
+
+package require sha1
+
+proc message::generate_thread {from to} {
+    return [sha1::sha1 $from$to[random 1000000000]]
+}
 
 proc message::send_dialog {args} {
     global font
@@ -305,7 +310,6 @@ proc message::send_dialog {args} {
 
     set to ""
     set subject ""
-    set thread ""
     set group 0
     set cstate normal
     foreach {opt val} $args {
@@ -320,6 +324,7 @@ proc message::send_dialog {args} {
 	    }
 	}
     }
+
     if {![info exists connid]} {
 	if {$group} {
 	    set connid [jlib::route ""]
@@ -328,18 +333,16 @@ proc message::send_dialog {args} {
 	}
     }
 
-    set send_dialog_connid [jlib::connection_jid $connid]
-
-    set sendargs ""
-
-    if {$thread != ""} {
-        lappend sendargs -thread $thread
-    }
-
-    set mw .msgsend$msgid
+    set mw .msgsend[incr msgid]
     toplevel $mw -class Message
     wm group $mw .
 
+    set send_dialog_connid($mw) [jlib::connection_jid $connid]
+
+    if {![info exists thread]} {
+	set thread [generate_thread $send_dialog_connid($mw) $to]
+    }
+
     if {$group} {
 	if {$to != ""} {
 	    set title [format [::msgcat::mc "Send message to group %s"] $to]
@@ -379,6 +382,8 @@ proc message::send_dialog {args} {
     set sep [Separator::create $mw.sep -orient horizontal]
     pack $sep -pady 1m -fill x -side bottom
 
+    label $mw.thread -text $thread
+
     frame $mw.frame
     pack $mw.frame -side top -fill both -expand yes -padx 2m -pady 2m
 
@@ -392,7 +397,7 @@ proc message::send_dialog {args} {
 	}
 	label $mw.f.lconnection -text [::msgcat::mc "From: "]
 	ComboBox $mw.f.connection \
-	    -textvariable [namespace current]::send_dialog_connid \
+	    -textvariable [namespace current]::send_dialog_connid($mw) \
 	    -values $connections \
 	    -font $font \
 	    -state $cstate
@@ -436,8 +441,6 @@ proc message::send_dialog {args} {
 
     textUndoable $mw.body -width 60 -height 8 -wrap word -font $font
     pack $mw.body -side top -fill both -expand yes -in $mw.sw
-    bind $mw.body <Control-z> { %W undo }
-    bind $mw.body <Control-Z> { %W redo }
     bind $mw.body <Control-Return> "ButtonBox::invoke $bbox 0
             break"
     $mw.sw setwidget $mw.body
@@ -448,10 +451,9 @@ proc message::send_dialog {args} {
         focus $mw.f.to
     }
 
-    incr msgid
-
     hook::run open_message_post_hook $mw $connid $to
 
+    BWidget::place $mw 0 0 center
     wm deiconify $mw
 }
 
@@ -459,10 +461,12 @@ proc message::send0 {mw args} {
     variable send_dialog_connid
     
     foreach c [jlib::connections] {
-	if {[jlib::connection_jid $c] == $send_dialog_connid} {
+	if {[jlib::connection_jid $c] == $send_dialog_connid($mw)} {
 	    set connid $c
 	}
     }
+    unset send_dialog_connid($mw)
+
     if {![info exists connid]} {
 	eval [list send $mw] $args
     } else {
@@ -489,22 +493,20 @@ proc message::send {mw args} {
 		      -delimiter $::ifacetk::roster::options(nested_delimiter)]
     } else {
 	set message_dest_list [update_combo_list $message_dest_list $jid 20]
-	set custom::saved([namespace current]::message_dest_list) \
-	    $message_dest_list
-	custom::store
 	set jids [list $jid]
 	if {![info exists connid]} {
 	    set connid [jlib::route $jid]
 	}
     }
 
+    set thread [$mw.thread cget -text]
     set subj [$mw.f.subj get]
     set body [$mw.body get 1.0 {end -1 chars}]
 
     foreach jid $jids {
-	if {!$group || [roster::is_user $connid $jid]} {
+	if {!$group || [::roster::itemconfig $connid $jid -isuser]} {
 	    send_msg $jid -type normal \
-		-subject $subj -body $body \
+		-subject $subj -body $body -thread $thread \
 		-connection $connid
 	}
     }
@@ -572,7 +574,7 @@ proc message::send_msg {to args} {
 }
 
 proc message::jiddropcmd {target source pos op type data} {
-    set jid [lindex $data 0]
+    set jid [lindex $data 1]
     $target delete 0 end
     $target insert 0 $jid
 }
@@ -597,34 +599,20 @@ proc message::show_subscribe_dialog {connid from type x args} {
         }
     }
 
-    set mw .subscshow$msgid
+    set mw .subscshow[incr msgid]
     toplevel $mw -class Message
     wm group $mw .
+    wm withdraw $mw
 
-    set title [format [::msgcat::mc "Subscribe request from %s"] $from]
+    set title [format [::msgcat::mc "Subscription request from %s"] $from]
     wm title $mw $title
     wm iconname $mw $title
 
     set bbox [ButtonBox $mw.buttons -spacing 0 -padx 10 -default 0]
-    $bbox add -text [::msgcat::mc "Subscribe"] \
-        -command "jlib::send_presence -to [list $from] -type subscribed \
-		      -connection $connid
-                  switch -- \
-                      \[roster::itemconfig $connid \
-                          [list [tolower_node_and_domain $from]] -subsc\] {
-                      {}   -
-                      none -
-                      from {
-                          message::send_subscribe_dialog [list $from] \
-			      -connection $connid
-                      }
-                  }
-                  destroy [list $mw]"
-
-    $bbox add -text [::msgcat::mc "Unsubscribe"] \
-        -command [join [list [list jlib::send_presence -to $from \
-                                  -type unsubscribed -connection $connid] \
-                            [list destroy $mw]] \n]
+    $bbox add -text [::msgcat::mc "Approve subscription"] \
+	      -command [list [namespace current]::subscribe $mw $connid $from]
+    $bbox add -text [::msgcat::mc "Decline subscription"] \
+	      -command [list [namespace current]::unsubscribe $mw $connid $from]
     bind $mw <Return> "ButtonBox::invoke $bbox default"
     bind $mw <Escape> "ButtonBox::invoke $bbox 1"
     pack $bbox -side bottom -anchor e -padx 2m -pady 2m
@@ -648,7 +636,7 @@ proc message::show_subscribe_dialog {connid from type x args} {
     }
 
 
-    label $mw.subj.lab -text [::msgcat::mc "Subscribe request from:"]
+    label $mw.subj.lab -text [::msgcat::mc "Subscription request from:"]
     menubutton $mw.subj.mb -text $from -font $font -menu $mw.subj.mb.menu
     subject_menu $mw.subj.mb.menu $connid $from subscribe
     grid $mw.subj.lab -row 1 -column 0 -sticky e
@@ -673,14 +661,54 @@ proc message::show_subscribe_dialog {connid from type x args} {
     pack $mw.body -side bottom -fill both -expand yes -in $mw.sw
     $mw.sw setwidget $mw.body
 
-    incr msgid
+    BWidget::place $mw 0 0 center
+    wm deiconify $mw
 }
 
 hook::add client_presence_hook message::show_subscribe_dialog
 
 ###############################################################################
 
-proc message::destroy_subscription_dialogs {connid jid name groups subsc ask category subtype} {
+proc message::show_unsubscribed_dialog {connid from type x args} {
+    variable msgid
+
+    if {$type != "unsubscribed"} return
+
+    MessageDlg .unsubscribed[incr msgid] -aspect 50000 -icon info \
+	-title [::msgcat::mc "Unsubscribed from %s" $from] \
+	-message [::msgcat::mc "You are unsubscribed from %s" $from] \
+	-type user -buttons {ok} -default 0 -cancel 0
+}
+
+hook::add client_presence_hook message::show_unsubscribed_dialog
+
+###############################################################################
+
+proc message::subscribe {mw connid jid} {
+    jlib::send_presence -to $jid -type subscribed -connection $connid
+
+    switch -- [roster::itemconfig $connid \
+				  [tolower_node_and_domain $jid] -subsc] {
+        {}   -
+        none -
+        from {
+            message::send_subscribe_dialog $jid -connection $connid
+        }
+    }
+
+    destroy $mw
+}
+
+###############################################################################
+
+proc message::unsubscribe {mw connid jid} {
+    ::roster::remove_item $connid [tolower_node_and_domain $jid]
+    destroy $mw
+}
+
+###############################################################################
+
+proc message::destroy_subscription_dialogs {connid jid name groups subsc ask} {
     switch -- $subsc {
 	both -
 	from {}
@@ -701,6 +729,27 @@ hook::add roster_push_hook message::destroy_subscription_dialogs
 
 ###############################################################################
 
+proc message::disco_popup_menu {m bw tnode data parentdata} {
+    lassign $data type connid jid node
+    switch -- $type {
+	item {
+	    set identities [disco::get_jid_identities $connid $jid $node]
+
+	    foreach id $identities {
+		if {[jlib::wrapper:getattr $id category] == "client"} {
+		    subject_menu $m $connid $jid message
+		    break
+		}
+	    }
+	}
+    }
+
+}
+
+hook::add disco_node_menu_hook message::disco_popup_menu
+
+###############################################################################
+
 proc message::subject_menu {m connid jid type} {
     if {[winfo exists $m]} {
         destroy $m          
@@ -763,20 +812,21 @@ proc message::send_subscribe_dialog {to args} {
     }
     set send_subscribe_connid [jlib::connection_jid $connid]
 
-    set mw .subscsend$msgid
+    set mw .subscsend[incr msgid]
     toplevel $mw -class Message
     wm group $mw .
+    wm withdraw $mw
 
     if {[cequal $to ""]} {
-        set title [::msgcat::mc "Send subscription"]
+        set title [::msgcat::mc "Send subscription request"]
     } else {
-        set title [format [::msgcat::mc "Send subscription to %s"] $to]
+        set title [format [::msgcat::mc "Send subscription request to %s"] $to]
     }
     wm title $mw $title
     wm iconname $mw $title
 
     set bbox [ButtonBox $mw.buttons -spacing 0 -padx 10 -default 0]
-    $bbox add -text [::msgcat::mc "Subscribe"] \
+    $bbox add -text [::msgcat::mc "Request subscription"] \
         -command [list message::send_subscribe0 $mw]
     $bbox add -text [::msgcat::mc "Cancel"] -command [list destroy $mw]
     bind $mw <Return> "ButtonBox::invoke $bbox default"
@@ -809,7 +859,7 @@ proc message::send_subscribe_dialog {to args} {
 	grid $mw.subj.connection  -row 0 -column 1 -sticky ew
     }
 
-    label $mw.subj.lab -text [::msgcat::mc "Send subscription to: "]
+    label $mw.subj.lab -text [::msgcat::mc "Send request to: "]
     entry $mw.subj.entry -font $font
     $mw.subj.entry insert 0 $to
     grid $mw.subj.lab   -row 1 -column 0 -sticky e
@@ -821,14 +871,15 @@ proc message::send_subscribe_dialog {to args} {
     ScrolledWindow $mw.sw
     pack $mw.sw -side top -fill both -expand yes -in $mw.frame
 
-    text $mw.body -width 60 -height 8 -wrap word
-    $mw.body insert 0.0 "I would like to add you to my roster."
+    textUndoable $mw.body -width 60 -height 8 -wrap word
+    $mw.body insert 0.0 [::msgcat::mc "I would like to add you to my roster."]
     pack $mw.body -side top -fill both -expand yes -in $mw.sw
     $mw.sw setwidget $mw.body
 
     focus $mw.subj.entry
 
-    incr msgid
+    BWidget::place $mw 0 0 center
+    wm deiconify $mw
 }
 
 ###############################################################################
@@ -883,11 +934,26 @@ proc message::send_subscribe {mw args} {
 
 proc message::resubscribe_menu_item {m connid jid} {
     set rjid [roster::find_jid $connid $jid]
-    $m add command -label [::msgcat::mc "Resubscribe"] \
+    if {$rjid == ""} {
+	set state disabled
+    } else {
+	set state normal
+    }
+    set mm [menu $m.subscription -tearoff 0]
+    $mm add command -label [::msgcat::mc "Request subscription"] \
 	-command [list jlib::send_presence \
 		      -to $rjid \
 		      -type subscribe \
 		      -connection $connid]
+    $mm add command -label [::msgcat::mc "Grant subscription"] \
+	-command [list jlib::send_presence \
+		      -to $rjid \
+		      -type subscribed \
+		      -connection $connid]
+
+    $m add cascad -label [::msgcat::mc "Subscription"] \
+		  -menu $mm \
+		  -state $state
 }
 
 hook::add roster_jid_popup_menu_hook message::resubscribe_menu_item 30
