@@ -1,14 +1,18 @@
-# $Id: muc.tcl,v 1.77 2006/04/09 19:45:37 aleksey Exp $
+# $Id: muc.tcl 1099 2007-04-12 06:44:00Z sergei $
 
-# Multi-User Chat support (JEP-0045)
+# Multi-User Chat support (XEP-0045)
 
 ###############################################################################
 
 namespace eval muc {
     set winid 0
-    custom::defvar options(gen_events) 1 \
-	[::msgcat::mc "Generate event messages in MUC compatible\
-		       conference rooms."] \
+    custom::defvar options(gen_enter_exit_msgs) 1 \
+	[::msgcat::mc "Generate status messages when occupants\
+	    enter/exit MUC compatible conference rooms."] \
+	-type boolean -group Chat
+    custom::defvar options(gen_muc_status_change_msgs) 0 \
+	[::msgcat::mc "Generate groupchat messages when occupant\
+	    changes his/her status and/or status message"] \
 	-type boolean -group Chat
     custom::defvar options(propose_configure) 0 \
 	[::msgcat::mc "Propose to configure newly created MUC room.\
@@ -105,6 +109,10 @@ proc muc::create_conference_menu_items {m connid jid} {
 
     trace variable ::muc::muc_compatible($jid) w \
 	[list muc::add_conference_menu_items $m $chatid $idx]
+
+    bind $m <Destroy> [list trace vdelete ::muc::muc_compatible($jid) w \
+			    [list muc::add_conference_menu_items $m $chatid $idx]]
+
 }
     
 hook::add chat_create_conference_menu_hook muc::create_conference_menu_items 37
@@ -117,23 +125,27 @@ proc muc::add_conference_menu_items {m chatid idx args} {
     trace vdelete ::muc::muc_compatible($group) w \
 	[list muc::add_conference_menu_items $m $chatid $idx]
 
-    $m insert $idx command -label [::msgcat::mc "Destroy room"] \
-	-command [list muc::request_destruction_dialog $chatid "" ""]
-    $m insert $idx command -label [::msgcat::mc "Edit owner list"] \
-	-command [list muc::request_list affiliation owner $chatid]
-    $m insert $idx command -label [::msgcat::mc "Edit admin list"] \
-	-command [list muc::request_list affiliation admin $chatid]
-    $m insert $idx command -label [::msgcat::mc "Edit moderator list"] \
-	-command [list muc::request_list role moderator $chatid]
-    $m insert $idx command -label [::msgcat::mc "Edit member list"] \
-	-command [list muc::request_list affiliation member $chatid]
-    $m insert $idx command -label [::msgcat::mc "Edit ban list"] \
-	-command [list muc::request_list affiliation outcast $chatid]
-    $m insert $idx command -label [::msgcat::mc "Edit voice list"] \
-	-command [list muc::request_list role participant $chatid]
-    $m insert $idx command -label [::msgcat::mc "Configure room"] \
+    set mm [menu $m.muc -tearoff 0]
+
+    $mm add command -label [::msgcat::mc "Configure room"] \
 	-command [list muc::request_config $chatid]
-    $m insert $idx separator
+    $mm add command -label [::msgcat::mc "Edit voice list"] \
+	-command [list muc::request_list role participant $chatid]
+    $mm add command -label [::msgcat::mc "Edit ban list"] \
+	-command [list muc::request_list affiliation outcast $chatid]
+    $mm add command -label [::msgcat::mc "Edit member list"] \
+	-command [list muc::request_list affiliation member $chatid]
+    $mm add command -label [::msgcat::mc "Edit moderator list"] \
+	-command [list muc::request_list role moderator $chatid]
+    $mm add command -label [::msgcat::mc "Edit admin list"] \
+	-command [list muc::request_list affiliation admin $chatid]
+    $mm add command -label [::msgcat::mc "Edit owner list"] \
+	-command [list muc::request_list affiliation owner $chatid]
+    $mm add separator
+    $mm add command -label [::msgcat::mc "Destroy room"] \
+	-command [list muc::request_destruction_dialog $chatid "" ""]
+
+    $m insert $idx cascade -label [::msgcat::mc "MUC"] -menu $mm
 }
 
 ###############################################################################
@@ -571,7 +583,7 @@ proc muc::fill_list {sf f items attr val} {
 
     grid columnconfigure $f 0 -weight 1
     grid columnconfigure $f 1 -weight 1
-    grid columnconfigure $f 2 -weight 1
+    grid columnconfigure $f 2 -weight 0
     grid columnconfigure $f 3 -weight 2
 
     label $f.lnick -text [::msgcat::mc "Nick"]
@@ -607,22 +619,42 @@ proc muc::fill_list {sf f items attr val} {
 		set jid [jlib::wrapper:getattr $vars jid]
 		set role [jlib::wrapper:getattr $vars role]
 		set affiliation [jlib::wrapper:getattr $vars affiliation]
-		lappend items2 [list $nick $jid $role $affiliation]
+		set reason ""
+		foreach subitem $children {
+		    jlib::wrapper:splitxml $subitem tag1 vars1 isempty1 chdata1 children1
+		    if {$tag1 == "reason"} {
+			set reason $chdata1
+		    }
+		}
+		lappend items2 [list $nick $jid $role $affiliation $reason]
 	    }
 	}
     }
 
     foreach item [lsort -dictionary -index 1 $items2] {
-	lassign $item nick jid role affiliation
+	lassign $item listdata($f,nick,$row) listdata($f,jid,$row) \
+		      role affiliation listdata($f,reason,$row)
 
-	label $f.nick$row -text $nick -font $font \
-	    -textvariable muc::listdata($f,nick,$row)
-	grid $f.nick$row -row $row -column 0 -sticky w -padx 1m
+	entry $f.nick$row -font $font \
+	    -textvariable muc::listdata($f,nick,$row) \
+	    -takefocus 0 \
+	    -highlightthickness 0 \
+	    -width 20
+	if {[catch {$f.nick$row configure -state readonly}]} {
+	    $f.nick$row configure -state disabled
+	}
+	grid $f.nick$row -row $row -column 0 -sticky we -padx 1m
 	bindscroll $f.nick$row $sf
 
-	label $f.jid$row -text $jid -font $font \
-	    -textvariable muc::listdata($f,jid,$row)
-	grid $f.jid$row -row $row -column 1 -sticky w -padx 1m
+	entry $f.jid$row -font $font \
+	    -textvariable muc::listdata($f,jid,$row) \
+	    -takefocus 0 \
+	    -highlightthickness 0 \
+	    -width 30
+	if {[catch {$f.jid$row configure -state readonly}]} {
+	    $f.jid$row configure -state disabled
+	}
+	grid $f.jid$row -row $row -column 1 -sticky we -padx 1m
 	bindscroll $f.jid$row $sf
 
 	switch -- $attr {
@@ -647,7 +679,8 @@ proc muc::fill_list {sf f items attr val} {
 	}
 
 	entry $f.reason$row -font $font \
-	    -textvariable muc::listdata($f,reason,$row)
+	    -textvariable muc::listdata($f,reason,$row) \
+	    -width 40
 	grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
 	bindscroll $f.reason$row $sf
 	
@@ -666,11 +699,17 @@ proc muc::list_add_item {sf f attr val} {
 
     set row [incr listdata($f,rows)]
 
-    entry $f.nick$row -font $font -textvariable muc::listdata($f,nick,$row)
+    entry $f.nick$row \
+	-font $font \
+	-textvariable muc::listdata($f,nick,$row) \
+	-width 20
     grid $f.nick$row -row $row -column 0 -sticky we -padx 1m
     bindscroll $f.nick$row $sf
     
-    entry $f.jid$row -font $font -textvariable muc::listdata($f,jid,$row)
+    entry $f.jid$row \
+	-font $font \
+	-textvariable muc::listdata($f,jid,$row) \
+	-width 30
     grid $f.jid$row -row $row -column 1 -sticky we -padx 1m
     bindscroll $f.jid$row $sf
 
@@ -695,7 +734,10 @@ proc muc::list_add_item {sf f attr val} {
 	}
     }
 
-    entry $f.reason$row -font $font -textvariable muc::listdata($f,reason,$row)
+    entry $f.reason$row \
+	-font $font \
+	-textvariable muc::listdata($f,reason,$row) \
+	-width 40
     grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
     bindscroll $f.reason$row $sf
 
@@ -712,7 +754,8 @@ proc muc::send_list {chatid attr val w f} {
 
     for {set i 1} {$i <= $origlistdata($f,rows)} {incr i} {
 	set vars {}
-	if {$listdata($f,$attr,$i) != $origlistdata($f,$attr,$i)} {
+	if {$listdata($f,$attr,$i) != $origlistdata($f,$attr,$i) || \
+		$listdata($f,reason,$i) != $origlistdata($f,reason,$i)} {
 	    lappend vars $attr $listdata($f,$attr,$i)
 	}
 
@@ -850,7 +893,8 @@ proc muc::receive_config {chatid res child} {
 
     jlib::wrapper:splitxml $child tag vars isempty chdata children
 
-    data::draw_window $children [list muc::send_config $chatid]
+    data::draw_window $children [list muc::send_config $chatid] \
+				[list muc::cancel_config $chatid]
     return
 }
 
@@ -871,6 +915,23 @@ proc muc::send_config {chatid w restags} {
 
 ###############################################################################
 
+proc muc::cancel_config {chatid w} {
+    set connid [chat::get_connid $chatid]
+    set group [chat::get_jid $chatid]
+    jlib::send_iq set [jlib::wrapper:createtag query \
+			   -vars [list xmlns $::NS(muc#owner)] \
+			   -subtags [list [jlib::wrapper:createtag x \
+					       -vars [list xmlns jabber:x:data \
+							   type cancel]]]] \
+	-to $group \
+	-connection $connid \
+	-command [list muc::test_error_res \
+		       [::msgcat::mc "Cancelling configure form"] $connid $group]
+    destroy $w
+}
+
+###############################################################################
+
 proc muc::test_error_res {op connid group res child} {
     if {![cequal $res OK]} {
 	set chatid [chat::chatid $connid $group]
@@ -944,7 +1005,7 @@ proc muc::process_muc_user {connid user type childrens} {
 		if {$altjid != ""} {
 		    append msg [::msgcat::mc "\nAlternative venue: %s" $altjid]
 		}
-		if {$options(gen_events)} {
+		if {$options(gen_enter_exit_msgs)} {
 		    chat::add_message \
 			$chatid $group groupchat $msg {}
 		}
@@ -959,7 +1020,7 @@ proc muc::process_muc_user {connid user type childrens} {
 			if {$options(propose_configure)} {
 			    request_config_dialog $chatid
 			} else {
-			    # requesting "instant" room as specified in JEP-0045
+			    # requesting "instant" room as specified in XEP-0045
 			    # if the user wants to configure room (s)he can do it later
 			    request_instant_room $chatid
 			}
@@ -1002,7 +1063,7 @@ proc muc::process_muc_user {connid user type childrens} {
 			    append action ": $reason"
 			}
 
-			if {$options(gen_events)} {
+			if {$options(gen_enter_exit_msgs)} {
 			    variable ignore_unavailable $nick
 			    chat::add_message \
 				$chatid $group groupchat $action {}
@@ -1015,7 +1076,9 @@ proc muc::process_muc_user {connid user type childrens} {
 			    if {$nick == [get_our_groupchat_nick $chatid]} {
 				set_our_groupchat_nick $chatid $new_nick
 			    }
-			    if {$options(gen_events)} {
+			    # TODO may be this reporting should not be done
+			    # if the $nick is being ignored (MUC ignore)
+			    if {$options(gen_enter_exit_msgs)} {
 				variable ignore_available   $new_nick
 				variable ignore_unavailable $nick
 
@@ -1028,6 +1091,7 @@ proc muc::process_muc_user {connid user type childrens} {
 				    [::msgcat::mc "%s is now known as %s" \
 					 $nick$real_jid $new_nick] {}
 			    }
+			    ::hook::run room_nickname_changed_hook $connid $group $nick $new_nick
 			}
 		    }
 		}
@@ -1038,8 +1102,6 @@ proc muc::process_muc_user {connid user type childrens} {
 
 
 proc muc::process_available {chatid nick} {
-    variable options
-    variable ignore_available
     variable pending_nicks
 
     set connid [chat::get_connid $chatid]
@@ -1050,18 +1112,6 @@ proc muc::process_available {chatid nick} {
 	set_our_groupchat_nick $chatid $nick
 	unset pending_nicks($chatid)
     }
-    
-    if {[is_compatible $group] && $options(gen_events) && \
-	    (![info exists ignore_available] || \
-		 $ignore_available != $nick)} {
-	set real_jid [get_real_jid $connid $group/$nick]
-	if {$real_jid != ""} {
-	    set real_jid " ($real_jid)"
-	}
-	chat::add_message $chatid $group groupchat \
-	    [::msgcat::mc "%s has become available" $nick$real_jid] {}
-    }
-    catch { unset ignore_available }
 }
 
 
@@ -1071,7 +1121,7 @@ proc muc::process_unavailable {chatid nick} {
 
     set group [chat::get_jid $chatid]
 
-    if {[is_compatible $group] && $options(gen_events) && \
+    if {[is_compatible $group] && $options(gen_enter_exit_msgs) && \
 	    (![info exists ignore_unavailable] || \
 		 $ignore_unavailable != $nick)} {
 	set connid [chat::get_connid $chatid]
@@ -1087,6 +1137,50 @@ proc muc::process_unavailable {chatid nick} {
     catch { unset ignore_unavailable }
 }
 
+proc muc::report_available {chatid nick entered} {
+    variable options
+    variable ignore_available
+
+    set connid [::chat::get_connid $chatid]
+    set group [::chat::get_jid $chatid]
+
+    if {![is_compatible $group]} return
+
+    set jid $group/$nick
+
+    set msg ""
+
+    set report_entered [expr {$entered && $options(gen_enter_exit_msgs) \
+	&& (![info exists ignore_available] || $ignore_available != $nick)}]
+
+    if {$report_entered} {
+	set real_jid [get_real_jid $connid $jid]
+	if {$real_jid != ""} {
+	    set occupant "$nick ($real_jid)"
+	} else {
+	    set occupant $nick
+	}
+	set msg [format [::msgcat::mc "%s has entered"] $occupant]
+    }
+    catch { unset ignore_available }
+
+    if {$options(gen_muc_status_change_msgs)} {
+	set status [::get_user_status $connid $jid]
+	if {$report_entered} {
+	    append msg " " [::msgcat::mc "and"] " "
+	} else {
+	    append msg $nick " "
+	}
+	append msg [::get_long_status_desc $status]
+	set desc [::get_user_status_desc $connid $jid]
+	if {$desc != {}} {
+	    append msg " ($desc)"
+	}
+    }
+
+    ::chat::add_message $chatid $group groupchat $msg {}
+}
+
 ###############################################################################
 
 proc muc::change_nick {chatid nick} {
@@ -1124,9 +1218,9 @@ proc muc::process_change_nick {chatid connid from type x args} {
 	return
     }
 
-    if {![cequal [lindex [jlib::wrapper:getattr $args -error] 1] conflict]} {
-	return
-    }
+#    if {![cequal [lindex [jlib::wrapper:getattr $args -error] 1] conflict]} {
+#	return
+#    }
 
     if {![is_compatible [chat::get_jid $chatid]] && \
 	    [is_changing_nick $chatid]} {
@@ -1211,8 +1305,10 @@ hook::add roster_user_popup_info_hook muc::add_user_popup_info
 
 proc muc::set_message_timestamp {chatid from type body x} {
     variable timestamps
-    
-    set timestamps($chatid) [clock seconds]
+
+    if {![chat::is_disconnected $chatid]} {
+	set timestamps($chatid) [clock seconds]
+    }
 }
 
 hook::add draw_message_hook muc::set_message_timestamp 15
@@ -1234,6 +1330,10 @@ proc muc::join_group {connid group nick {password ""}} {
     variable timestamps
     variable muc_password
 
+    set group [tolower_node_and_domain $group]
+
+    privacy::add_to_special_list $connid conference [server_from_jid $group]
+
     set chatid [chat::chatid $connid $group]
     set_our_groupchat_nick $chatid $nick
 
@@ -1243,9 +1343,8 @@ proc muc::join_group {connid group nick {password ""}} {
     request_negotiation $connid $group
 
     set x_subtags {}
-    if {$password != ""} {
-	lappend x_subtags [jlib::wrapper:createtag password -chdata $password]
-    }
+
+    lappend x_subtags [jlib::wrapper:createtag password -chdata $password]
     set muc_password($chatid) $password
 
     set history_vars {}
@@ -1330,8 +1429,9 @@ proc muc::invite_xconference {connid group jid reason} {
 
 ###############################################################################
 
-proc muc::process_invitation {acc f x connid from type replyP} {
-    lassign $acc row body
+proc muc::process_invitation {rowvar bodyvar f x connid from type replyP} {
+    upvar 2 $rowvar row
+    upvar 2 $bodyvar body
 
     foreach xa $x {
 	jlib::wrapper:splitxml $xa tag vars isempty chdata children
@@ -1401,15 +1501,17 @@ proc muc::process_invitation {acc f x connid from type replyP} {
     if {[info exists muc_group] && $muc_group != ""} {
 	process_x_conference $f $connid $muc_group $muc_password $row
 	incr row
-	return [list $row $muc_body]
+	set body $muc_body
+	return
     } elseif {[info exists xconference_group] && $xconference_group != ""} {
 	process_x_conference $f $connid $xconference_group \
 	    $xconference_password $row
 	incr row
-	return [list $row $xconference_body]
+	set body $xconference_body
+	return
     }
     
-    return $acc
+    return
 }
 
 hook::add message_process_x_hook muc::process_invitation
@@ -1476,17 +1578,12 @@ hook::add postload_hook \
 
 ###############################################################################
 
-proc muc::iq_reply {connid from child} {
-    set res [jlib::wrapper:createtag query \
-		     -vars [list xmlns $::NS(muc)]]
-    return [list result $res]
-}
 
 iq::register_handler get query $::NS(muc) muc::iq_reply
 
 ###############################################################################
 
-proc muc::disco_reply {type connid from child} {
+proc muc::disco_reply {type connid from lang child} {
     variable options
 
     if {!$options(report_muc_rooms)} {
@@ -1499,12 +1596,9 @@ proc muc::disco_reply {type connid from child} {
 	}
 	items {
 	    set res {}
-	    foreach chatid [array names chat::opened] {
-		set connid1 [chat::get_connid $chatid]
+	    foreach chatid [lfilter chat::is_groupchat [chat::opened $connid]] {
 		set group [chat::get_jid $chatid]
-		if {[cequal $connid $connid1] &&
-			[cequal $chat::chats(type,$chatid) groupchat] && \
-			[is_compatible $group]} {
+		if {[is_compatible $group]} {
 		    lappend res [jlib::wrapper:createtag item \
 				     -vars [list jid $group]]
 		}
@@ -1515,7 +1609,9 @@ proc muc::disco_reply {type connid from child} {
 }
 
 hook::add postload_hook \
-    [list disco::register_node $::NS(muc#rooms) muc::disco_reply ""]
+    [list disco::register_node $::NS(muc#rooms) muc::disco_reply \
+	  [::trans::trans "Current rooms"]]
 
 ###############################################################################
 
+# vim:ts=8:sw=4:sts=4:noet
