@@ -1,4 +1,4 @@
-# $Id: plugins.tcl,v 1.10 2006/02/25 19:57:36 aleksey Exp $
+# $Id: plugins.tcl 828 2006-12-15 20:58:11Z sergei $
 
 namespace eval plugins {}
 
@@ -22,7 +22,7 @@ proc plugins::load {dir args} {
 }
 
 proc plugins::load_dir {plugins_dir} {
-    foreach dir [lsort [glob -nocomplain -type d [file join $plugins_dir *]]] {
+    foreach dir [lsort [glob -nocomplain -type {d l} [file join $plugins_dir *]]] {
 	set file [file join $dir [file tail $dir].tcl]
 	if {[file exists $file]} {
 	    debugmsg plugins "Loading plugin from $file"
