@@ -1,15 +1,17 @@
-# $Id: draw_server_message.tcl,v 1.3 2003/06/21 19:16:25 aleksey Exp $
+# $Id: draw_server_message.tcl 903 2007-02-07 19:31:31Z sergei $
 
 proc handle_server_message {chatid from type body x} {
     set jid [chat::get_jid $chatid]
-    if {[cequal $type groupchat] && [string equal $jid $from]} {
+    if {([cequal $type groupchat] && [string equal $jid $from]) \
+	|| ([cequal $type chat] && $from == "")} {
 	set chatw [chat::chat_win $chatid]
 
 	$chatw insert end --- server_lab " "
-	chat::add_emoteiconed_text [::chat::chat_win $chatid] $body server
+	::richtext::render_message [::chat::chat_win $chatid] $body server
 	set cw [chat::winid $chatid]
-	tab_set_updated $cw 1 server
 	return stop
     }
 }
 hook::add draw_message_hook [namespace current]::handle_server_message 20
+
+# vim:ts=8:sw=4:sts=4:noet
