@@ -1,4 +1,4 @@
-# $Id: draw_xhtml_message.tcl,v 1.5 2005/03/27 20:31:12 aleksey Exp $
+# $Id: draw_xhtml_message.tcl 882 2007-01-26 17:55:57Z sergei $
 
 namespace eval xhtml {
     set statevars {
@@ -64,18 +64,15 @@ proc xhtml::draw_xhtml_message {chatid from type plainbody x} {
 
 	if {[crange $body 0 [expr [clength $mynick] + 1]] == "${mynick}: "} {
 	    $chatw insert end $mynick me
-	    chat::add_emoteiconed_text [::chat::chat_win $chatid] \
+	    ::richtext::render_message [::chat::chat_win $chatid] \
 		[crange $body [clength $mynick] end] ""
-	    tab_set_updated $cw 1 mesg_to_user
 	} else {
-	    chat::add_emoteiconed_text [::chat::chat_win $chatid] $body ""
-	    tab_set_updated $cw 1 message
+	    ::richtext::render_message [::chat::chat_win $chatid] $body ""
 	}
     } else {
 	$chatw insert end "<$nick>" $tag " "
 	init [::chat::chat_win $chatid]
 	add_xhtml [::chat::chat_win $chatid] $body
-	tab_set_updated $cw 1 mesg_to_user
     }
 
     return stop
@@ -200,7 +197,7 @@ proc xhtml::add_xhtml {cw xhtml} {
 	}
 
 	li {
-	    chat::add_emoteiconed_text $cw $prefix ""
+	    ::richtext::render_message $cw $prefix ""
 	    set prefix ""
 	    switch -- $state(list_style) {
 		ul {
@@ -214,7 +211,7 @@ proc xhtml::add_xhtml {cw xhtml} {
 			     $state(list_counter)]
 		}
 	    }
-	    chat::add_emoteiconed_text $cw $item_prefix \
+	    ::richtext::render_message $cw $item_prefix \
 		[concat xhtml_symb [get_tags $cw]]
 	}
 	ul {
@@ -248,8 +245,8 @@ proc xhtml::add_xhtml {cw xhtml} {
 	set state(afterspace) [expr {[string index $formated end] == " "}]
     }
 
-    chat::add_emoteiconed_text $cw $prefix $tag
-    chat::add_emoteiconed_text $cw $formated $tag
+    ::richtext::render_message $cw $prefix $tag
+    ::richtext::render_message $cw $formated $tag
 
     if {$formated != ""} {
 	set state(lastnl) 0
@@ -268,7 +265,7 @@ proc xhtml::add_xhtml {cw xhtml} {
 	    set state(afterspace) [expr {[string index $formated end] == " "}]
 	}
 
-	chat::add_emoteiconed_text $cw $formated $tag
+	::richtext::render_message $cw $formated $tag
 	if {$formated != ""} {
 	    set state(lastnl) 0
 	}
@@ -284,13 +281,13 @@ proc xhtml::add_xhtml {cw xhtml} {
 	}
     }
     if {$suffix == "\n\n"} {
-	chat::add_emoteiconed_text $cw \
+	::richtext::render_message $cw \
 	    [string repeat "\n" [expr {2 - $state(lastnl)}]] ""
     } elseif {$suffix == "\n"} {
-	chat::add_emoteiconed_text $cw \
+	::richtext::render_message $cw \
 	    [string repeat "\n" [expr {1 - $state(lastnl)}]] ""
     } else {
-	chat::add_emoteiconed_text $cw $suffix $tag
+	::richtext::render_message $cw $suffix $tag
     }
     set state(lastnl) 0
     if {[$cw get "end - 2c"] == "\n"} {
@@ -392,9 +389,9 @@ proc xhtml::get_url_tag {chatw url} {
     $chatw tag configure $tag -foreground $urlfg -underline 1
     $chatw tag bind $tag <1> [list browseurl [double% $url]]
     $chatw tag bind $tag <Any-Enter> \
-	[list chat::highlighttext $chatw $tag $urlactfg hand2]
+	[list ::richtext::highlighttext $chatw $tag $urlactfg hand2]
     $chatw tag bind $tag <Any-Leave> \
-	[list chat::highlighttext $chatw $tag $urlfg xterm]
+	[list ::richtext::highlighttext $chatw $tag $urlfg xterm]
     $chatw tag raise $tag
     return $tag
 }
