@@ -3,7 +3,7 @@
 # Do full text coloring based upon nicks. Includes a color editor and
 # persistence for modified color selection.
 #
-# $Id: nick_colors.tcl,v 1.6 2005/05/15 18:31:01 aleksey Exp $
+# $Id: nick_colors.tcl 877 2007-01-16 22:11:52Z sergei $
 
 namespace eval nickcolors {
 
@@ -72,7 +72,7 @@ proc nickcolors::add_nick_colors_menu {} {
 #
 proc nickcolors::load_nick_colors {} {
     variable NickColors
-    set filename [file join ~ .tkabber nickcolors.tcl]
+    set filename [file join $::configdir nickcolors.tcl]
     if {[file exists $filename]} {
         set f [open $filename r]
 	fconfigure $f -encoding utf-8
@@ -92,7 +92,7 @@ proc nickcolors::load_nick_colors {} {
 # Called at shutdown to save the current set of nick-colors to file.
 proc nickcolors::save_nick_colors {} {
     variable NickColors
-    set filename [file join ~ .tkabber nickcolors.tcl]
+    set filename [file join $::configdir nickcolors.tcl]
     set f [open $filename w]
     fconfigure $f -encoding utf-8
     puts $f "# This is an automatically generated file. Do not edit."
@@ -117,6 +117,7 @@ proc nickcolors::chat_add_nick_colors {chatid type} {
     foreach {nick clr} [array get NickColors] {
 	if {$options(use_colored_nicks)} {
 	    $W tag configure NICK-$nick -foreground $clr
+	    $W tag configure NICKMSG-$nick -foreground $clr
 	}
 	if {$options(use_colored_messages)} {
 	    $W tag configure MSG-$nick -foreground $clr
@@ -137,6 +138,7 @@ proc nickcolors::get_nick_color {nick} {
 	    if {[winfo exists $wn]} {
 		if {$options(use_colored_nicks)} {
 		    $wn tag configure NICK-$nick -foreground $NickColors($nick)
+		    $wn tag configure NICKMSG-$nick -foreground $NickColors($nick)
 		}
 		if {$options(use_colored_messages)} {
 		    $wn tag configure MSG-$nick -foreground $NickColors($nick)
@@ -268,6 +270,8 @@ proc nickcolors::change_options {args} {
             foreach {nick clr} [array get NickColors] {
                 $wn tag configure NICK-$nick \
 		    -foreground [expr {$options(use_colored_nicks) ? $clr : ""}]
+                $wn tag configure NICKMSG-$nick \
+		    -foreground [expr {$options(use_colored_nicks) ? $clr : ""}]
                 $wn tag configure MSG-$nick \
 		    -foreground [expr {$options(use_colored_messages) ? $clr : ""}]
             }
@@ -289,6 +293,9 @@ proc nickcolors::add_chat_win_popup_menu {m chatwin X Y x y} {
 	if {[set idx [lsearch -glob $tags NICK-*]] >= 0} {
 	    set nick [string range [lindex $tags $idx] 5 end]
 	}
+	if {[set idx [lsearch -glob $tags NICKMSG-*]] >= 0} {
+	    set nick [string range [lindex $tags $idx] 5 end]
+	}
     }
 
     if {$nick == ""} return
