@@ -1,4 +1,4 @@
-# $Id: autoaway.tcl,v 1.13 2005/03/13 21:14:21 aleksey Exp $
+# $Id: autoaway.tcl 875 2007-01-15 21:06:31Z sergei $
 
 namespace eval autoaway {
     variable options
@@ -12,50 +12,54 @@ proc autoaway::load {} {
     global tcl_platform
     global idle_command
 
-    switch -- $tcl_platform(platform) {
-        macintosh {
-	    set idle_command [namespace current]::AquaIdleTime
-	}
+    if {[catch {tk inactive}] || ([tk inactive] < 0)} {
+	switch -- $tcl_platform(platform) {
+	    macintosh {
+		set idle_command [namespace current]::AquaIdleTime
+	    }
 
-	unix {
-	    if {$::aquaP} {
-	        set idle_command [namespace current]::AquaIdleTime
-	    } elseif {[catch { package require tkXwin }]} {
-		return
-	    } else {
-	        set idle_command tkXwin::idletime
+	    unix {
+		if {$::aquaP} {
+		    set idle_command [namespace current]::AquaIdleTime
+		} elseif {[catch { package require tkXwin }]} {
+		    return
+		} else {
+		    set idle_command tkXwin::idletime
+		}
 	    }
-	}
-	windows {
-	    if {[catch { package require tclWinidle }]} {
+	    windows {
+		if {[catch { package require tclWinidle }]} {
+		    return
+		}
+		set idle_command tclWinidle::idletime
+	    }
+	    default {
 		return
 	    }
-	    set idle_command tclWinidle::idletime
-	}
-	default {
-	    return
 	}
+    } else {
+	set idle_command {tk inactive}
     }
 
     custom::defgroup AutoAway \
 	[::msgcat::mc "Options for module that automatically marks\
-you as away after idle threshold."] \
+		       you as away after idle threshold."] \
 	-group Tkabber
 
     custom::defvar options(awaytime) 5 \
 	[::msgcat::mc "Idle threshold in minutes after that\
-Tkabber marks you as away."] \
+		       Tkabber marks you as away."] \
 	-group AutoAway -type integer
 
     custom::defvar options(xatime) 15 \
 	[::msgcat::mc "Idle threshold in minutes after that\
-Tkabber marks you as extended away."] \
+		       Tkabber marks you as extended away."] \
 	-group AutoAway -type integer
 
     custom::defvar options(status) \
 	[::msgcat::mc "Automatically away due to idle"] \
 	[::msgcat::mc "Text status, which is set when\
-Tkabber is moving to away state."] \
+		       Tkabber is moving to away state."] \
 	-group AutoAway -type string
 
     custom::defvar options(drop_priority) 1 \
@@ -73,13 +77,30 @@ proc autoaway::after_idle {args} {
     if {$::aquaP} {
         set msec 1000
     } else {
-        set msec 100
+        set msec 250
     }
     after $msec [namespace current]::after_idle
 }
 
 proc autoaway::after_idle_cancel {args} {
+    variable options
+    variable savestatus
+    variable savetext
+    variable savepriority
+    global userstatus textstatus userpriority
+
     if {[jlib::connections] == {}} {
+        if {![cequal $savestatus ""]} {
+	    if {$options(drop_priority) && ($userpriority >= 0)} {
+		set userpriority $savepriority
+	    }
+	    set savepriority 0
+	    set textstatus $savetext
+	    set savetext ""
+            set userstatus $savestatus
+            set savestatus ""
+        }
+	
 	after cancel [namespace current]::after_idle
     }
 }
@@ -90,14 +111,15 @@ proc autoaway::after_idle_aux {} {
     variable savetext
     variable savepriority
     global idle_command
-    global userstatus textstatus
-    global userpriority
+    global userstatus textstatus userpriority
 
     if {($options(awaytime) <= 0) && ($options(xatime) <= 0)} {
         return
     }
 
-    if {[set idletime [$idle_command]] < [expr {$options(awaytime)*60*1000}]} {
+    set idletime [eval $idle_command]
+
+    if {$idletime < [expr {$options(awaytime)*60*1000}]} {
         if {![cequal $savestatus ""]} {
 	    if {$options(drop_priority) && ($userpriority >= 0)} {
 		set userpriority $savepriority
@@ -119,7 +141,9 @@ proc autoaway::after_idle_aux {} {
             set savestatus $userstatus
 	    set savetext $textstatus
 	    set savepriority $userpriority
-            set textstatus $options(status)
+	    if {![cequal $options(status) ""]} {
+	       set textstatus $options(status)
+	    }
             if {$idletime >= [expr {$options(xatime)*60*1000}]} {
 		if {$options(drop_priority) && ($userpriority >= 0)} {
 		    set userpriority 0
@@ -137,7 +161,9 @@ proc autoaway::after_idle_aux {} {
             if {(![cequal $savestatus ""]) && \
 		    ($idletime >= [expr {$options(xatime)*60*1000}])} {
 		set savepriority $userpriority
-		set textstatus $options(status)
+		if {![cequal $options(status) ""]} {
+		    set textstatus $options(status)
+		}
 		if {$options(drop_priority) && ($userpriority >= 0)} {
 		    set userpriority 0
 		}
