@@ -1,4 +1,4 @@
-# $Id: avatars.tcl,v 1.4 2006/01/20 03:08:00 aleksey Exp $
+# $Id: avatars.tcl 1060 2007-03-21 18:29:58Z sergei $
 
 if {![cequal $::interface tk]} return
 
@@ -21,7 +21,6 @@ proc ::avatar::setup_menu {} {
 	#    }
 	#}
 	set m [.mainframe getmenu plugins]
-	set ind [$m index end]
 
 	set mm .avatar_menu
 	menu $mm -tearoff $ifacetk::options(show_tearoffs)
@@ -32,7 +31,7 @@ proc ::avatar::setup_menu {} {
 	$mm add command -label [::msgcat::mc "Send to server"] \
 	    -command {avatar::store_on_server}
 
-	$m insert $ind cascade -label [::msgcat::mc "Avatar"] \
+	$m add cascade -label [::msgcat::mc "Avatar"] \
 	    -menu $mm
     }
 }
@@ -60,18 +59,19 @@ proc ::avatar::load_file {filename} {
 
 ##############################################################################
 
-proc ::avatar::get_presence_x {acc connid status} {
+proc ::avatar::get_presence_x {varname connid status} {
     variable avatar
     variable options
+    upvar 2 $varname var
 
     if {$options(announce) && [info exists avatar(userhash)]} {
 	set children [jlib::wrapper:createtag hash -chdata $avatar(userhash)]
-	lappend acc [jlib::wrapper:createtag x \
+	lappend var [jlib::wrapper:createtag x \
 			 -vars [list xmlns $::NS(xavatar)] \
 			 -subtags [list $children]]
 
     }
-    return $acc
+    return
 }
 
 hook::add presence_xlist_hook ::avatar::get_presence_x
@@ -244,7 +244,7 @@ proc ::avatar::store_on_server {args} {
 
 ##############################################################################
 
-proc ::avatar::iq_reply {connid from child} {
+proc ::avatar::iq_reply {connid from lang child} {
     variable avatar
     variable options
 
