@@ -1,4 +1,6 @@
-# $Id: headlines.tcl,v 1.6 2006/02/09 20:08:51 aleksey Exp $
+# $Id: headlines.tcl 1042 2007-03-10 16:18:58Z sergei $
+
+#############################################################################
 
 namespace eval headlines {
     variable headid 0
@@ -19,7 +21,7 @@ namespace eval headlines {
 
     custom::defvar options(multiple) 0 \
 	[::msgcat::mc "Display headlines in single/multiple windows."] \
-	-group Messages -type radio -layout vertical \
+	-group Messages -type options \
 	-values [list 0 [::msgcat::mc "Single window"] \
 		      1 [::msgcat::mc "One window per bare JID"] \
 		      2 [::msgcat::mc "One window per full JID"]]
@@ -32,10 +34,18 @@ namespace eval headlines {
 	[::msgcat::mc "Format of timestamp in headline tree view. Set to\
 		       empty string if you don't want to see timestamps."] \
 	-group Messages -type string
+
+    custom::defvar options(show_balloons) 0 \
+	[::msgcat::mc "Show balloons with headline messages over tree nodes."] \
+	-group Messages -type boolean
 }
 
+#############################################################################
+
 package require md5
 
+#############################################################################
+
 proc headlines::process_message {connid from id type is_subject subject body err thread priority x} {
     switch -- $type {
 	headline {
@@ -49,8 +59,161 @@ proc headlines::process_message {connid from id type is_subject subject body err
 hook::add process_message_hook \
     [namespace current]::headlines::process_message
 
-proc headlines::show {connid from type subject body thread priority x {data {}}} {
+#############################################################################
+
+proc headlines::get_win {connid from} {
+    variable options
+
+    switch -- $options(multiple) {
+        0 { return .headlines }
+        1 { return .headlines_[jid_to_tag [node_and_server_from_jid $from]] }
+        default { return .headlines_[jid_to_tag $from] }
+    }
+}
+
+#############################################################################
+
+proc headlines::get_tree {connid from} {
+    set hw [get_win $connid $from]
+    return $hw.tree
+}
+
+#############################################################################
+
+proc headlines::open_window {connid from} {
     global tcl_platform
+    global font
+    variable options
+    variable trees
+
+    set hw [get_win $connid $from]
+
+    if {[winfo exists $hw]} return
+
+    switch -- $options(multiple) {
+        0 {
+            set title [::msgcat::mc "Headlines"]
+            set tabtitle [::msgcat::mc "Headlines"]
+        }
+        1 {
+            set user [node_and_server_from_jid $from]
+            set title [format [::msgcat::mc "%s Headlines"] $user]
+            set tabtitle [node_from_jid $from]
+        }
+        default {
+            set title [format [::msgcat::mc "%s Headlines"] $from]
+            set tabtitle [node_from_jid $from]/[resource_from_jid $from]
+        }
+    }
+
+    set tw [get_tree $connid $from]
+
+    if {[lsearch -exact $trees $tw] < 0} {
+        lappend trees $tw
+    }
+
+    add_win $hw -title $title -tabtitle $tabtitle \
+		-raisecmd [list focus $tw] \
+		-class JBrowser
+
+    PanedWin $hw.pw -side right -pad 0 -width 8
+    pack $hw.pw -fill both -expand yes
+
+    set uw [PanedWinAdd $hw.pw -weight 1]
+    set dw [PanedWinAdd $hw.pw -weight 1]
+
+    frame $dw.date
+    label $dw.date.label -anchor w -text [::msgcat::mc "Date:"]
+    entry $dw.date.ts -font $font \
+		      -takefocus 0 \
+		      -highlightthickness 0 \
+		      -relief flat
+    pack $dw.date -fill x
+    pack $dw.date.label -side left
+    pack $dw.date.ts -side left -fill x -expand yes
+
+    frame $dw.from
+    label $dw.from.label -anchor w -text [::msgcat::mc "From:"]
+    entry $dw.from.jid -font $font \
+		       -takefocus 0 \
+		       -highlightthickness 0 \
+		       -relief flat
+    pack $dw.from -fill x
+    pack $dw.from.label -side left
+    pack $dw.from.jid -side left -fill x -expand yes
+
+    frame $dw.subject
+    label $dw.subject.lsubj -anchor w -text [::msgcat::mc "Subject:"]
+    text $dw.subject.subj -font $font \
+			  -height 1 \
+			  -takefocus 0 \
+			  -highlightthickness 0 \
+			  -relief flat \
+			  -state disabled \
+			  -background [lindex [$dw.subject configure -background] 4]
+    pack $dw.subject -fill x
+    pack $dw.subject.lsubj -side left
+    pack $dw.subject.subj -side left -fill x -expand yes
+
+    foreach ent [list $dw.date.ts $dw.from.jid] {
+	if {[catch {$ent configure -state readonly}]} {
+	    $ent configure -state disabled
+	}
+    }
+
+    if {![info exists options(seencolor)]} {
+	if {[cequal $tcl_platform(platform) unix]} {
+	    set options(seencolor) [option get $hw disabledForeground JBrowser]
+	} else {
+	    set options(seencolor) [option get $hw nscolor JBrowser]
+	}
+    }
+    if {![info exists options(unseencolor)]} {
+	set options(unseencolor) [option get $hw fill JBrowser]
+    }
+
+    set sw [ScrolledWindow $uw.sw]
+    Tree $tw -deltax 16 -deltay 18 \
+	 -selectcommand [list [namespace current]::update_body \
+			      $dw.date.ts $dw.from.jid $dw.subject.subj $hw.body]
+    $sw setwidget $tw
+    pack $sw -side top -expand yes -fill both
+
+    $tw bindText <ButtonPress-3> [list [namespace current]::select_popup $hw]
+    $tw bindText <Double-ButtonPress-1> \
+		 [list [namespace current]::action browse $hw]
+    balloon::setup $tw -command [list [namespace current]::balloon $hw]
+
+    # HACK
+    bind $tw.c <Return> \
+	 "[namespace current]::action browse $hw \[$tw selection get\]"
+    bind $tw.c <Delete> \
+	 "[namespace current]::action delete $hw \[$tw selection get\]"
+
+    bindscroll $tw.c
+
+    set dsw [ScrolledWindow $dw.sw]
+    text $hw.body -font $font -height 12 -state disabled \
+		  -wrap word -takefocus 1
+    ::richtext::config $hw.body -using url
+    $dsw setwidget $hw.body
+    pack $dsw -expand yes -fill both -anchor nw
+
+    bind $hw.body <ButtonPress-1> [list focus %W]
+
+    foreach ww [list $hw.body $dw.date.ts $dw.from.jid $dw.subject.subj] {
+	bind $ww <Key-Up>    [list Tree::_keynav up    $tw]
+	bind $ww <Key-Down>  [list Tree::_keynav down  $tw]
+	bind $ww <Key-Left>  [list Tree::_keynav left  $tw]
+	bind $ww <Key-Right> [list Tree::_keynav right $tw]
+    }
+
+    hook::run open_headlines_post_hook $hw $tw $uw $dw
+}
+
+#############################################################################
+
+proc headlines::show {connid from type subject body thread priority x {data {}}} {
     variable headid
     variable headlines
     variable trees
@@ -62,7 +225,7 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
 
     set desc ""
     set url ""
-    set seconds [clock seconds]
+    set seconds [jlib::x_delay $x]
     foreach extra $x {
         jlib::wrapper:splitxml $extra tag vars isempty chdata children
 	switch -- [jlib::wrapper:getattr $vars xmlns] {
@@ -78,15 +241,9 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
 		    }
 		}
 	    }
-	    jabber:x:delay {
-		set stamp [jlib::wrapper:getattr $vars stamp]
-		catch { set seconds [clock scan $stamp -gmt 1] }
-	    }
 	}
     }
-    if {[cequal $url ""]} {
-        return
-    }
+
     if {[cequal $subject ""]} {
 	set subject $desc
     } else {
@@ -94,130 +251,36 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
 	    set desc $subject
 	}
     }
-    if {[cequal $subject ""]} {
-	return
+
+    if {$subject == ""} {
+	set dsubject [::msgcat::mc "<none>"]
+    } else {
+	set dsubject $subject
     }
 
-    switch -- $options(multiple) {
-        0 {
-            set hw .headlines
-            set title [::msgcat::mc "Headlines"]
-            set tabtitle [::msgcat::mc "Headlines"]
-        }
-
-        1 {
-            set user [node_and_server_from_jid $from]
-            set hw .headlines_[jid_to_tag $user]
-            set title [format [::msgcat::mc "%s Headlines"] $user]
-            set tabtitle [node_from_jid $from]
-        }
-
-        default {
-            set hw .headlines_[jid_to_tag $from]
-            set title [format [::msgcat::mc "%s Headlines"] $from]
-            set tabtitle [node_from_jid $from]/[resource_from_jid $from]
-        }
-    }
-    if {[lsearch -exact $trees [set tw $hw.tree]] < 0} {
-        lappend trees $tw
+    if {$desc == ""} {
+	set ddesc [::msgcat::mc "<none>"]
+    } else {
+	set ddesc $desc
     }
 
+    set hw [get_win $connid $from]
+
     if {![winfo exists $hw]} {
-        add_win $hw -title $title -tabtitle $tabtitle \
-            -raisecmd "focus [list $hw.tree]
-                       tab_set_updated [list $hw]" -class JBrowser
-
-	PanedWin $hw.pw -side right -pad 0 -width 8
-	pack $hw.pw -fill both -expand yes
-
-	set uw [$hw.pw add -weight 1]
-	set dw [$hw.pw add -weight 1]
-
-	frame $dw.date
-	label $dw.date.label -anchor w -text [::msgcat::mc "Date:"]
-	label $dw.date.ts -font $font -anchor w
-	pack $dw.date -fill x
-	pack $dw.date.label -side left
-	pack $dw.date.ts -side left
-
-	frame $dw.from
-	label $dw.from.label -anchor w -text [::msgcat::mc "From:"]
-	label $dw.from.jid -font $font -anchor w
-	pack $dw.from -fill x
-	pack $dw.from.label -side left
-	pack $dw.from.jid -side left
-
-	frame $dw.subject
-	label $dw.subject.lsubj -anchor w -text [::msgcat::mc "Subject:"]
-	label $dw.subject.subj -font $font -anchor w
-	pack $dw.subject -fill x
-	pack $dw.subject.lsubj -side left
-	pack $dw.subject.subj -side left
-
-	if {![info exists options(seencolor)]} {
-	    if {[cequal $tcl_platform(platform) unix]} {
-		set options(seencolor) [option get $hw disabledForeground JBrowser]
-	    } else {
-		set options(seencolor) [option get $hw nscolor JBrowser]
-	    }
-	}
-	if {![info exists options(unseencolor)]} {
-	    set options(unseencolor) [option get $hw fill JBrowser]
-	}
-
-        set sw [ScrolledWindow $uw.sw]
-	Tree $tw -deltax 16 -deltay 18 \
-	    -selectcommand [list [namespace current]::update_body \
-				 $dw.date.ts $dw.from.jid $dw.subject.subj $hw.body]
-        $sw setwidget $tw
-        pack $sw -side top -expand yes -fill both
-
-        $tw bindText <ButtonPress-3> \
-                [list [namespace current]::select_popup $hw]
-        $tw bindText <Double-ButtonPress-1> \
-                [list [namespace current]::action browse $hw]
-        $tw bindText <Any-Enter>  \
-                [list [namespace current]::balloon $hw enter  %X %Y]
-        $tw bindText <Any-Motion> \
-                [list [namespace current]::balloon $hw motion %X %Y]
-        $tw bindText <Any-Leave>  \
-                [list [namespace current]::balloon $hw leave  %X %Y]
-
-        # HACK
-        bind $tw.c <Return> \
-            "[namespace current]::action browse $hw \[$tw selection get\]"
-        bind $tw.c <Delete> \
-            "[namespace current]::action delete $hw \[$tw selection get\]"
-
-        bindscroll $tw.c
-
-	set dsw [ScrolledWindow $dw.sw]
-	text $hw.body -font $font -height 12 -state disabled \
-	     -wrap word -takefocus 1
-	$dsw setwidget $hw.body
-	pack $dsw -expand yes -fill both -anchor nw
-	$hw.body tag configure emphasized -elide 1
-	$hw.body tag configure nonemphasized -elide 0
-
-	bind $hw.body <ButtonPress-1> [list focus %W]
-
-	bind $hw.body <Key-Up>    [list Tree::_keynav up    $tw]
-	bind $hw.body <Key-Down>  [list Tree::_keynav down  $tw]
-	bind $hw.body <Key-Left>  [list Tree::_keynav left  $tw]
-	bind $hw.body <Key-Right> [list Tree::_keynav right $tw]
-
-	hook::run open_headlines_post_hook $hw
+	open_window $connid $from
     }
 
+    set tw [get_tree $connid $from]
+
     if {$options(multiple) > 1} {
-        set text $subject
+	set text $dsubject
     } else {
         set text $from
     }
     set fnode [str2node $text]
     if {![$tw exists $fnode]} {
         $tw insert end root $fnode -text [string map [list "\n" " "] $text] -open 1 \
-            -image browser/message -font $font \
+            -image browser/headline -font $font \
             -fill $options(seencolor) \
             -data [list type from text $text unseen 0]
     }
@@ -225,11 +288,11 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     if {($options(multiple) > 1) || ([cequal $subject $desc])} {
         set snode $fnode
     } else {
-        set snode $fnode-subject-[str2node $subject]
+        set snode $fnode-subject-[str2node $dsubject]
 
         if {![$tw exists $snode]} {
-            $tw insert end $fnode $snode -text [string map [list "\n" " "] $subject] -open 1 \
-                -image browser/message -font $font \
+            $tw insert end $fnode $snode -text [string map [list "\n" " "] $dsubject] -open 1 \
+                -image browser/headline -font $font \
                 -fill $options(seencolor) \
                 -data [list type subject text $subject unseen 0]
         }
@@ -245,7 +308,7 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     set nodetext \
 	[cconcat \
 	     [clock format $props(seconds) -format $options(timestamp_format)] \
-		    [string map [list "\n" " "] $desc]]
+		    [string map [list "\n" " "] $ddesc]]
     $tw insert end $snode $anode -text $nodetext -open 1 \
         -fill $options(seencolor) -font $font \
         -data [array get props]
@@ -259,6 +322,8 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     tab_set_updated $hw 1 message
 }
 
+#############################################################################
+
 proc headlines::str2node {string} {
     set utf8str [encoding convertto utf-8 $string]
     if {[catch { ::md5::md5 -hex $utf8str } ret]} {
@@ -268,6 +333,8 @@ proc headlines::str2node {string} {
     }
 }
 
+#############################################################################
+
 proc headlines::update_body {wdate wfrom wsubj wbody tw node} {
     variable headlines
     
@@ -286,23 +353,45 @@ proc headlines::update_body {wdate wfrom wsubj wbody tw node} {
 	set date [clock format $props(seconds)]
 	set url $props(url)
     }
-    
-    $wdate configure -text $date
-    $wfrom configure -text $from
-    $wsubj configure -text $subj
-    
+
+    foreach {w s} [list $wdate $date \
+			$wfrom $from] {
+	$w configure -state normal
+	$w delete 0 end
+	$w insert 0 $s
+	if {[catch {$w configure -state readonly}]} {
+	    $w configure -state disabled
+	}
+    }
+
+    $wsubj configure -state normal
+    $wsubj delete 0.0 end
+    $wsubj insert 0.0 $subj
+    $wsubj delete {end - 1 char}
+
+    $wsubj mark set sel_start end
+    $wsubj mark set sel_end 0.0
+
+    $wsubj configure -state disabled
+
     $wbody configure -state normal
     $wbody delete 0.0 end
-    chat::add_emoteiconed_text $wbody "$body\n\n" ""
+    ::richtext::render_message $wbody "$body\n\n" ""
     if {$url != ""} {
-	#chat::add_url $wbody [::msgcat::mc "See more..."] $url
-	chat::add_url $wbody $url $url \
-	    -command [list [namespace code action] markseen \
-			   [winfo parent $tw] $node]
+        ::plugins::urls::render_url $wbody url $url {} \
+	    -title [::msgcat::mc "Read on..."] \
+            -add-command [list [namespace code action] markseen \
+			       [winfo parent $tw] $node]
     }
+
+    $wbody mark set sel_start end
+    $wbody mark set sel_end 0.0
+
     $wbody configure -state disabled
 }
 
+#############################################################################
+
 proc headlines::update_menu {menu num} {
     variable send_jids
 
@@ -319,6 +408,8 @@ proc headlines::update_menu {menu num} {
     }
 }
 
+#############################################################################
+
 namespace eval headlines {
     if {[winfo exists [set m .h1popmenu]]} {
 	destroy $m
@@ -344,7 +435,7 @@ namespace eval headlines {
     $m add command -label [::msgcat::mc "Delete"] \
 	-command "[namespace current]::action delete \
 		    \$[namespace current]::headwindow \$[namespace current]::headnode"
-    update_menu $m 0
+    hook::add finload_hook [list [namespace current]::update_menu $m 0]
 
     if {[winfo exists [set m .h2popmenu]]} {
 	destroy $m
@@ -370,6 +461,8 @@ namespace eval headlines {
 		    \$[namespace current]::headwindow \$[namespace current]::headnode"
 }
 
+#############################################################################
+
 proc headlines::select_popup {hw node} {
     variable headwindow
     variable headnode
@@ -396,6 +489,8 @@ proc headlines::select_popup {hw node} {
     tk_popup $hm [winfo pointerx .] [winfo pointery .]
 }
 
+#############################################################################
+
 proc headlines::action {action hw node} {
     variable headlines
     variable options
@@ -406,7 +501,9 @@ proc headlines::action {action hw node} {
 
     switch -glob -- $props(type)/$action {
         article/browse {
-            browseurl $props(url)
+	    if {$props(url) != ""} {
+		browseurl $props(url)
+	    }
 
             if {$props(unseen)} {
                 set props(unseen) 0
@@ -458,7 +555,24 @@ proc headlines::action {action hw node} {
             $tw itemconfigure $node -fill $options(seencolor) -data [array get props]
             update $tw $node
 
+	    # Deduce the node to select after $node is deleted:
+	    # Next sibling is tried first, then previous, then parent node.
+	    set p [$tw parent $node]
+	    set end [expr {[llength [$tw nodes $p]] - 1}]
+	    set ix [$tw index $node]
+	    if {$ix < $end} {
+		set next [$tw nodes $p [incr ix]]
+	    } elseif {$ix > 0} {
+		set next [$tw nodes $p [incr ix -1]]
+	    } else {
+		set next $p
+	    }
+
             $tw delete $node
+
+	    if {![string equal $next root]} {
+		$tw selection set $next
+	    }
         }
 
         article/deleteseen {
@@ -539,6 +653,8 @@ proc headlines::action {action hw node} {
     }
 }
 
+#############################################################################
+
 proc headlines::update {tw node} {
     variable options
 
@@ -559,6 +675,9 @@ proc headlines::update {tw node} {
         set props(unseen) $unseen
 
         set text $props(text)
+	if {$text == ""} {
+	    set text [::msgcat::mc "<none>"]
+	}
         set myfill $options(seencolor)
         if {$unseen > 0} {
             append text " ($unseen)"
@@ -569,9 +688,17 @@ proc headlines::update {tw node} {
     }
 }
 
-proc headlines::balloon {hw action X Y node} {
-    if {[catch { array set props [$hw.tree itemcget $node -data] }]} {
-        return
+#############################################################################
+
+proc headlines::balloon {hw node} {
+    variable options
+
+    if {!$options(show_balloons)} {
+	return [list $hw:$node ""]
+    }
+
+    if {[catch {array set props [$hw.tree itemcget $node -data]}]} {
+        return [list $hw:$node ""]
     }
 
     set width [expr {[winfo width $hw.tree] * 0.8}]
@@ -582,15 +709,15 @@ proc headlines::balloon {hw action X Y node} {
     switch -- $props(type) {
         article {
             if {![cequal $props(body) ""]} {
-                balloon::default_balloon $hw:$node $action $X $Y $props(body) -width $width
+                return [list $hw:$node $props(body) -width $width]
             }
         }
-
-        default {
-        }
     }
+    return [list $hw:$node ""]
 }
 
+#############################################################################
+
 proc headlines::save {} {
     variable options
     variable trees
@@ -599,7 +726,7 @@ proc headlines::save {} {
         return
     }
 
-    if {[catch { open [set file1 [file join ~ .tkabber headlines1.tcl]] \
+    if {[catch { open [set file1 [file join $::configdir headlines1.tcl]] \
                       { WRONLY CREAT TRUNC } } fd]} {
         debugmsg headlines "unable to open $file: $fd"
         return
@@ -622,13 +749,13 @@ proc headlines::save {} {
     }
 
     set renameP 0
-    if {![file exists [set file [file join ~ .tkabber headlines.tcl]]]} {
+    if {![file exists [set file [file join $::configdir headlines.tcl]]]} {
     } elseif {[file size $file] == 0} {
         catch { file delete -force $file }
     } else {
         set renameP 1
         catch { file rename -force $file \
-                     [set file0 [file join ~ .tkabber headlines0.tcl]] }
+                     [set file0 [file join $::configdir headlines0.tcl]] }
     }
 
     if {![catch { file rename $file1 $file } result]} {
@@ -644,6 +771,8 @@ proc headlines::save {} {
     return
 }
 
+#############################################################################
+
 proc headlines::save_aux {tw node fd} {
     variable headlines
 
@@ -662,21 +791,27 @@ proc headlines::save_aux {tw node fd} {
     }
 }
 
+#############################################################################
+
 proc headlines::restore {} {
     variable options
 
     if {$options(cache)} {
-	if {[file exists [set file [file join ~ .tkabber headlines.tcl]]]} {
-	    set encoding [encoding system]
-	    encoding system utf-8
-	    catch { source $file }
-	    encoding system $encoding
+	if {[file exists [set file [file join $::configdir headlines.tcl]]]} {
+	    catch {
+		set fd [open $file "r"]
+		fconfigure $fd -encoding utf-8
+		uplevel #0 [read $fd]
+		close $fd
+	    }
 	}
     }
 
     return ""
 }
 
+#############################################################################
+
 proc headlines::forward3 {menu to tw node} {
     variable send_jids
 
@@ -695,12 +830,12 @@ proc headlines::forward3 {menu to tw node} {
 					 [jlib::wrapper:createtag desc \
 					      -chdata $props(text)]]]]
     set len [llength $send_jids]
-    set send_jids [update_combo_list $send_jids $to 5]
-    set custom::saved([namespace current]::send_jids) $send_jids
-    custom::store
+    set send_jids [update_combo_list $send_jids $to 10]
     update_menu $menu $len
 }
 
+#############################################################################
+
 proc headlines::forward2 {menu tw node} {
     global forward_hl
     variable send_jids
@@ -710,6 +845,7 @@ proc headlines::forward2 {menu tw node} {
     }
 
     set len [llength $send_jids]
+    set _send_jids $send_jids
     foreach choice [array names forward_hl] {
 	if {$forward_hl($choice)} {
 	    lassign $choice connid to
@@ -723,14 +859,15 @@ proc headlines::forward2 {menu tw node} {
 						 [jlib::wrapper:createtag desc \
 						      -chdata $props(text)]]]] \
 		-connection $connid
-	    set send_jids [update_combo_list $send_jids $to 5]
+	    set _send_jids [update_combo_list $_send_jids $to 10]
 	}
     }
-    set custom::saved([namespace current]::send_jids) $send_jids
-    custom::store
+    set send_jids $_send_jids
     update_menu $menu $len
 }
 
+#############################################################################
+
 proc headlines::forward {menu tw node} {
     global forward_hl
 
@@ -755,12 +892,60 @@ proc headlines::forward {menu tw node} {
     }
 
     CbDialog $gw [::msgcat::mc "Forward headline"] \
-	[list [::msgcat::mc "Send"] "[namespace current]::forward2 [list $menu] [list $tw] [list $node]
+	[list [::msgcat::mc "Send"] "[namespace current]::forward2 [list $menu] \
+								   [list $tw] \
+								   [list $node]
 				     destroy $gw" \
 	      [::msgcat::mc "Cancel"] [list destroy $gw]] \
 	forward_hl $choices $balloons
 }
 
+#############################################################################
+
 hook::add finload_hook [namespace current]::headlines::restore
 hook::add quit_hook    [namespace current]::headlines::save
 
+#############################################################################
+
+proc headlines::restore_window {from connid jid} {
+    open_window $connid $from
+}
+
+#############################################################################
+
+# TODO: Work with changes in options(multiple)
+proc headlines::save_session {vsession} {
+    upvar 2 $vsession session
+    global usetabbar
+
+    # We don't need JID at all, so make it empty (special case)
+    set user     ""
+    set server   ""
+    set resource ""
+
+    # TODO
+    if {!$usetabbar} return
+
+    set prio 0
+    foreach page [.nb pages] {
+	set path [ifacetk::nbpath $page]
+
+	if {[string equal $path .headlines]} {
+	    lappend session [list $prio $user $server $resource \
+		[list [namespace current]::restore_window ""] \
+	    ]
+	}
+	if {[regexp {^.headlines_(.*)} $path -> tag]} {
+	    set jid [tag_to_jid $tag]
+	    lappend session [list $prio $user $server $resource \
+		[list [namespace current]::restore_window $jid] \
+	    ]
+	}
+	incr prio
+    }
+}
+
+hook::add save_session_hook [namespace current]::headlines::save_session
+
+#############################################################################
+
