@@ -1,9 +1,9 @@
-# $Id: message_archive.tcl,v 1.6 2005/11/14 21:40:21 aleksey Exp $
+# $Id: message_archive.tcl 924 2007-02-11 12:40:21Z sergei $
 
 option add *Messages.listheight 10 widgetDefault
 
 namespace eval ::message_archive {
-    variable logdir ~/.tkabber/logs
+    variable logdir [file join $::configdir logs]
 
     if {![file exists $logdir]} {
 	file mkdir $logdir
@@ -16,22 +16,29 @@ namespace eval ::message_archive {
     variable messages
 }
 
+#############################################################################
+
+proc ::message_archive::str_to_log {str} {
+    return [string map {\\ \\\\ \r \\r \n \\n} $str]
+}
+
+#############################################################################
+
+proc ::message_archive::log_to_str {str} {
+    return [string map {\\\\ \\ \\r \r \\n \n} $str]
+}
+
+#############################################################################
+
 proc ::message_archive::log_message {from to subject body x} {
     variable archive_file
 
-    set seconds [clock seconds]
-    foreach xelem $x {
-	jlib::wrapper:splitxml $xelem tag vars isempty chdata children
-	
-	if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:x:delay]} {
-	    set seconds [clock scan [jlib::wrapper:getattr $vars stamp] -gmt 1]
-	}
-    }
+    set seconds [jlib::x_delay $x]
     set ts [clock format $seconds -format "%Y%m%dT%H%M%S"]
 
     set fd [open $archive_file a]
     fconfigure $fd -encoding utf-8
-    puts $fd [list [list timestamp $ts id $ts[random 10000] from $from to $to subject $subject body $body]]
+    puts $fd [str_to_log [list timestamp $ts id $ts[random 10000] from $from to $to subject $subject body $body]]
     close $fd
 }
 
@@ -48,14 +55,13 @@ proc ::message_archive::show_archive {} {
 
     add_win $w -title [::msgcat::mc "Messages"] \
 	    -tabtitle [::msgcat::mc "Messages"] \
-	    -class Messages \
-	    -raisecmd "tab_set_updated [list $w]"
+	    -class Messages
 
     PanedWin $w.pw -side right -pad 0 -width 8
     pack $w.pw -fill both -expand yes
 
-    set uw [$w.pw add -weight 0 -minsize 100]
-    set dw [$w.pw add -weight 1 -minsize 100]
+    set uw [PanedWinAdd $w.pw -weight 0 -minsize 100]
+    set dw [PanedWinAdd $w.pw -weight 1 -minsize 100]
 
     frame $dw.title
     label $dw.title.label -font $font -text $label(from)
@@ -72,11 +78,10 @@ proc ::message_archive::show_archive {} {
     pack $dw.subject.subj -side left
 
     set body [ScrolledWindow $dw.sw]
-    text $body.body -height 20 -state disabled
+    text $body.body -height 20 -state disabled -wrap word
     pack $body -expand yes -fill both -anchor nw
     $body setwidget $body.body
-    $body.body tag configure emphasized -elide 1
-    $body.body tag configure nonemphasized -elide 0
+    ::richtext::config $body.body -using {url emoticon stylecode}
 
     set sww [ScrolledWindow $w.items]
 
@@ -140,17 +145,17 @@ proc ::message_archive::fill_list {l} {
 	return
     }
 
-    set fd [open $archive_file r]
-    fconfigure $fd -encoding utf-8
-    set hist [read $fd]
-    close $fd
-
     foreach i {N timestamp dir fromto subject} {
 	$l column configure $i -width [string length [$l column cget $i -label]]
     }
-    foreach vars $hist {
-	fill_row $l $vars
+
+    set hist {}
+    set fd [open $archive_file r]
+    fconfigure $fd -encoding utf-8
+    while {[gets $fd line] > 0} {
+	catch {fill_row $l [log_to_str $line]}
     }
+    close $fd
 }
 
 proc ::message_archive::fill_row {l var} {
@@ -289,6 +294,6 @@ proc ::message_archive::select_and_print_body {w l index} {
     $w.sw.body configure -state normal
     $w.sw.body delete 0.0 end
     #$w.sw.body insert end $messages($id,body)
-    ::chat::add_emoteiconed_text $w.sw.body $messages($id,body) ""
+    ::richtext::render_message $w.sw.body $messages($id,body) ""
     $w.sw.body configure -state disabled
 }
