@@ -1,4 +1,4 @@
-# $Id: stats.tcl,v 1.7 2004/09/25 20:55:49 aleksey Exp $
+# $Id: stats.tcl 1038 2007-03-10 14:09:40Z sergei $
 
 namespace eval stats {}
 
@@ -82,14 +82,18 @@ proc stats::add_line {jid node name} {
 			     $n]]
     grid $b -row $n -column 6 -sticky w
 
-    set s [SpinBox $f.spin$n -range {0 1000000000 1} \
-	       -width 4 \
-	       -textvariable \
-	       [namespace current]::data(tmpperiod,$jid,$node,$name)]
+    set s [Spinbox $f.spin$n 0 1000000000 1 \
+		   [namespace current]::data(tmpperiod,$jid,$node,$name) \
+		   -width 4]
+    trace variable [namespace current]::data(tmpperiod,$jid,$node,$name) w \
+	  [list [namespace current]::unset_timer $n $jid $node $name]
     grid $s -row $n -column 7 -sticky w
 
+    catch {unset data(period,$jid,$node,$name)}
+
     set b [button $f.bsettimer$n -text [::msgcat::mc "Set"] \
-	       -command [list [namespace current]::set_timer \
+	       -relief raised \
+	       -command [list [namespace current]::toggle_timer \
 			     $n $jid $node $name]]
     grid $b -row $n -column 8 -sticky w
 
@@ -105,6 +109,7 @@ proc stats::query_list {jid node args} {
     jlib::send_iq get [jlib::wrapper:createtag query \
 			   -vars $vars] \
 	-to $jid \
+	-connection [jlib::route $jid] \
 	-command [list [namespace current]::recv_query_list_result $jid $node]
 }
 
@@ -142,6 +147,7 @@ proc stats::request_value {jid node name} {
 			   -subtags [list [jlib::wrapper:createtag stat \
 					       -vars [list name $name]]]] \
 	-to $jid \
+	-connection [jlib::route $jid] \
 	-command [list [namespace current]::recv_values_result $jid $node]
 }
 
@@ -195,21 +201,44 @@ proc stats::remove_line {n} {
     }
 }
 
-proc stats::set_timer {n jid node name} {
+proc stats::unset_timer {n jid node name args} {
     variable data
     variable f
 
-    set data(period,$jid,$node,$name) $data(tmpperiod,$jid,$node,$name)
+    if {[info exists data(period,$jid,$node,$name)]} {
+	unset data(period,$jid,$node,$name)
+	$f.bsettimer$n configure -relief raised
+    }
+}
 
-    timer $n $jid $node $name
+proc stats::toggle_timer {n jid node name} {
+    variable data
+    variable f
+
+    if {![info exists data(period,$jid,$node,$name)]} {
+	if {[string is integer -strict $data(tmpperiod,$jid,$node,$name)] && \
+		$data(tmpperiod,$jid,$node,$name) > 0} {
+	    set data(period,$jid,$node,$name) $data(tmpperiod,$jid,$node,$name)
+	    $f.bsettimer$n configure -relief sunken
+
+	    timer $n $jid $node $name
+	}
+    } else {
+	unset data(period,$jid,$node,$name)
+	$f.bsettimer$n configure -relief raised
+    }
 }
 
 proc stats::timer {n jid node name} {
     variable data
     variable f
 
+    if {![winfo exists $f.spin$n]} return
+
     request_value $jid $node $name
 
+    if {![info exists data(period,$jid,$node,$name)]} return
+
     set p $data(period,$jid,$node,$name)
 
     after cancel \
