@@ -1,4 +1,4 @@
-# $Id: subscribe_gateway.tcl,v 1.5 2006/05/27 20:53:23 aleksey Exp $
+# $Id: subscribe_gateway.tcl 758 2006-10-11 19:45:24Z sergei $
 
 namespace eval gateway {
     variable msgid 0
@@ -55,6 +55,8 @@ proc gateway::subscribe_dialog {service args} {
     pack $mw.prompt -side top -anchor w -in $mw.frame
 
     variable $mw.prompt fulljid
+    bind $mw <Destroy> [list catch [list unset [namespace current]::$mw.prompt]]
+
     jlib::send_iq get \
         [jlib::wrapper:createtag query \
              -vars {xmlns jabber:iq:gateway}] \
@@ -75,8 +77,8 @@ proc gateway::subscribe_dialog {service args} {
     ScrolledWindow $mw.sw
     pack $mw.sw -side top -fill both -expand yes -in $mw.frame
 
-    text $mw.body -width 60 -height 8 -wrap word
-    $mw.body insert 0.0 "I would like to add you to my roster."
+    textUndoable $mw.body -width 60 -height 8 -wrap word
+    $mw.body insert 0.0 [::msgcat::mc "I would like to add you to my roster."]
     pack $mw.body -side top -fill both -expand yes -in $mw.sw
     $mw.sw setwidget $mw.body
 
@@ -139,8 +141,6 @@ proc gateway::send_subscribe {mw service args} {
 		-connection $connid
 	}
     }
-
-    unset $mw.prompt
 }
 
 proc gateway::gw_send_subscribe {mw fallback connid res child} {
