@@ -1,7 +1,6 @@
-# $Id: browse.tcl,v 1.4 2004/01/14 21:55:27 aleksey Exp $
+# $Id: browse.tcl 722 2006-09-18 21:01:30Z sergei $
 
-
-proc iq_browse {connid from child} {
+proc iq_browse {connid from lang child} {
     browser::parse_items $from $child
 
     return {}
@@ -10,8 +9,7 @@ proc iq_browse {connid from child} {
 iq::register_handler set {} jabber:iq:browse \
     [namespace current]::iq_browse
 
-
-proc iq_browse_reply {connid from child} {
+proc iq_browse_reply {connid from lang child} {
     set restags {}
     foreach ns $::iq::supported_ns {
 	lappend restags [jlib::wrapper:createtag ns \
@@ -19,10 +17,10 @@ proc iq_browse_reply {connid from child} {
     }
     
     set res [jlib::wrapper:createtag query \
-		 -vars {xmlns jabber:iq:browse \
+		 -vars {xmlns    jabber:iq:browse \
 			category user \
-			type client \
-			name Tkabber} \
+			type     client \
+			name     Tkabber} \
 		 -subtags $restags]
     
     return [list result $res]
@@ -31,4 +29,3 @@ proc iq_browse_reply {connid from child} {
 iq::register_handler get {} jabber:iq:browse \
     [namespace current]::iq_browse_reply
 
-
