@@ -1,4 +1,4 @@
-# $Id: dtcp.tcl,v 1.9 2004/09/05 20:16:09 aleksey Exp $
+# $Id: dtcp.tcl 722 2006-09-18 21:01:30Z sergei $
 
 namespace eval ::dtcp {}
 namespace eval ::dtcp::active {}
@@ -297,7 +297,7 @@ proc ::dtcp::readable {key chan} {
     }
 }
 
-proc ::dtcp::iq_set_handler {connid from child} {
+proc ::dtcp::iq_set_handler {connid from lang child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
 
     if {$tag == "query"} {
