@@ -1,4 +1,4 @@
-# $Id: systray.tcl,v 1.7 2006/01/28 20:45:07 aleksey Exp $
+# $Id: systray.tcl 1043 2007-03-10 19:29:19Z sergei $
 
 # Freedesktop systray icon support.
 # Requires Tray package
@@ -72,10 +72,7 @@ proc systray::create {icon} {
 
     bind $icon <ButtonRelease-1> ifacetk::systray::toggle_state
     bind $icon <ButtonRelease-3> [list tk_popup $m %X %Y]
-
-    bind $icon <Any-Enter>  [list ifacetk::systray::wrap_balloon $icon enter  %X %Y]
-    bind $icon <Any-Motion> [list ifacetk::systray::wrap_balloon $icon motion %X %Y]
-    bind $icon <Any-Leave>  [list ifacetk::systray::wrap_balloon $icon leave  %X %Y]
+    balloon::setup $icon -command [list ifacetk::systray::balloon $icon]
 }
 
 ##########################################################################
@@ -83,7 +80,9 @@ proc systray::create {icon} {
 proc systray::configure {icon status} {
     variable s2p
 
-    configureti $icon -pixmap $s2p($status)
+    if {![cequal $icon ""] && [winfo exists $icon]} {
+	configureti $icon -pixmap $s2p($status)
+    }
 }
 
 ##########################################################################
