@@ -1,4 +1,19 @@
-# $Id: presence.tcl,v 1.69 2006/06/12 20:38:16 aleksey Exp $
+# $Id: presence.tcl 1094 2007-04-10 08:19:45Z sergei $
+
+###############################################################################
+
+array set long_statusdesc [list \
+    available   [::msgcat::mc "is available"] \
+    chat        [::msgcat::mc "is free to chat"] \
+    away        [::msgcat::mc "is away"] \
+    xa          [::msgcat::mc "is extended away"] \
+    dnd         [::msgcat::mc "doesn't want to be disturbed"] \
+    invisible   [::msgcat::mc "is invisible"] \
+    unavailable [::msgcat::mc "is unavailable"]]
+
+proc get_long_status_desc {status} {
+    set ::long_statusdesc($status)
+}
 
 ###############################################################################
 
@@ -51,13 +66,7 @@ proc client:presence {connid from type x args} {
 	subscribe {}
 	subscribed {}
 	unsubscribe {}
-	unsubscribed {
-	    MessageDlg .unsubscribed -aspect 50000 -icon info \
-		-title [format [::msgcat::mc "Unsubscribed from %s"] $from] \
-		-message [format \
-			      [::msgcat::mc "We unsubscribed from %s"] $from] \
-		-type user -buttons {ok} -default 0 -cancel 0
-	}
+	unsubscribed {}
 	probe {}
 	default {
 	    set type available
@@ -279,17 +288,6 @@ custom::defvar userstatus available [::msgcat::mc "Stored user status."] \
 custom::defvar textstatus "" [::msgcat::mc "Stored user text status."] \
     -type string -group Hidden
 
-proc save_user_status {} {
-    global userpriority userstatus textstatus
-
-    set custom::saved(::::userpriority) $userpriority
-    set custom::saved(::::userstatus) $userstatus
-    set custom::saved(::::textstatus) $textstatus
-    custom::store
-}
-
-hook::add quit_hook save_user_status
-
 set userstatusdesc [::msgcat::mc "Not logged in"]
 
 set statusdesc(available)   [::msgcat::mc "Available"]
@@ -400,24 +398,22 @@ proc change_our_presence {name1 name2 op} {
 		      -connection $connid
     }
 
-    foreach chatid [array names chat::opened] {
+    foreach chatid [lfilter chat::is_groupchat [chat::opened]] {
 	set connid [chat::get_connid $chatid]
 	set group [chat::get_jid $chatid]
-	if {[cequal $chat::chats(type,$chatid) groupchat]} {
-	    set nick [get_our_groupchat_nick $chatid]
+	set nick [get_our_groupchat_nick $chatid]
 
-	    if {$newstatus == "invisible"} {
-		set newst available
-	    } else {
-		set newst $newstatus
-	    }
-
-	    send_presence $newst \
-			  -to $group/$nick \
-			  -stat $status \
-			  -pri $userpriority \
-			  -connection $connid
+	if {$newstatus == "invisible"} {
+	    set newst available
+	} else {
+	    set newst $newstatus
 	}
+
+	send_presence $newst \
+		      -to $group/$nick \
+		      -stat $status \
+		      -pri $userpriority \
+		      -connection $connid
     }
 
     set curuserstatus $newstatus
@@ -517,8 +513,8 @@ proc send_presence {status args} {
 	lappend newargs -stat $stat
     }
 
-    set x [hook::foldl presence_xlist_hook $xlist $connid $stat]
-    lappend newargs -xlist $x
+    hook::run presence_xlist_hook xlist $connid $stat
+    lappend newargs -xlist $xlist
 
     switch -- $status {
 	available   { set command [list jlib::send_presence] }
@@ -548,6 +544,22 @@ proc normalize_show {show} {
 
 ###############################################################################
 
+proc add_presence_to_popup_info {infovar connid jid} {
+    upvar 0 $infovar info
+
+    set bjid [node_and_server_from_jid $jid]
+    if {[chat::is_groupchat [chat::chatid $connid $bjid]]} return
+
+    set priority [get_jid_presence_info priority $connid $jid]
+    if {$priority != ""} {
+	append info [format "\n\t[::msgcat::mc {Priority:}] %s" $priority]
+    }
+}
+
+hook::add roster_user_popup_info_hook add_presence_to_popup_info 20
+
+###############################################################################
+
 proc clear_presence_info {connid} {
     global presence
 
@@ -596,6 +608,7 @@ proc custom_presence_menu {m connid jid} {
 
 hook::add chat_create_user_menu_hook custom_presence_menu 43
 hook::add roster_jid_popup_menu_hook custom_presence_menu 43
+hook::add roster_service_popup_menu_hook custom_presence_menu 43
 
 ###############################################################################
 
@@ -654,3 +667,4 @@ hook::add roster_service_popup_menu_hook service_login_logout_menu_item 20
 
 ###############################################################################
 
+# vim:ts=8:sw=4:sts=4:noet
