@@ -1,4 +1,4 @@
-# $Id: privacy.tcl,v 1.16 2006/04/27 20:29:35 aleksey Exp $
+# $Id: privacy.tcl 1079 2007-04-01 12:42:18Z sergei $
 #
 # Privacy lists support (RFC 3921, section 10)
 #
@@ -7,32 +7,38 @@ namespace eval privacy {
     variable options
 
     array set req_messages \
-	[list ignore    [::msgcat::mc "Requesting ignore list: %s"] \
-	      invisible [::msgcat::mc "Requesting invisible list: %s"] \
-	      visible   [::msgcat::mc "Requesting visible list: %s"]]
+	[list ignore     [::msgcat::mc "Requesting ignore list: %s"] \
+	      invisible  [::msgcat::mc "Requesting invisible list: %s"] \
+	      visible    [::msgcat::mc "Requesting visible list: %s"] \
+	      conference [::msgcat::mc "Requesting conference list: %s"]]
 
     array set send_messages \
-	[list ignore    [::msgcat::mc "Sending ignore list: %s"] \
-	      invisible [::msgcat::mc "Sending invisible list: %s"] \
-	      visible   [::msgcat::mc "Sending visible list: %s"]]
+	[list ignore       [::msgcat::mc "Sending ignore list: %s"] \
+	      invisible    [::msgcat::mc "Sending invisible list: %s"] \
+	      visible      [::msgcat::mc "Sending visible list: %s"] \
+	      conference   [::msgcat::mc "Sending conference list: %s"] \
+	      subscription [::msgcat::mc "Changing accept messages from roster only: %s"]]
 
     array set edit_messages \
-	[list ignore    [::msgcat::mc "Edit ignore list"] \
-	      invisible [::msgcat::mc "Edit invisible list"] \
-	      visible   [::msgcat::mc "Edit visible list"]]
+	[list ignore     [::msgcat::mc "Edit ignore list"] \
+	      invisible  [::msgcat::mc "Edit invisible list"] \
+	      visible    [::msgcat::mc "Edit visible list"] \
+	      conference [::msgcat::mc "Edit conference list"]]
 
     array set menu_messages \
-	[list ignore    [::msgcat::mc "Ignore list"] \
-	      invisible [::msgcat::mc "Invisible list"] \
-	      visible   [::msgcat::mc "Visible list"]]
+	[list ignore     [::msgcat::mc "Ignore list"] \
+	      invisible  [::msgcat::mc "Invisible list"] \
+	      visible    [::msgcat::mc "Visible list"]]
+
+    variable accept_from_roster 0
 
     custom::defgroup Privacy \
 	[::msgcat::mc "Blocking communication (XMPP privacy lists) options."] \
 	-group Tkabber
 
     custom::defvar options(activate_at_startup) 1 \
-	[::msgcat::mc "Activate visible/invisible/ignore lists before sending\
-		       initial presence."] \
+	[::msgcat::mc "Activate visible/invisible/ignore/conference lists\
+		       before sending initial presence."] \
 	-type boolean -group Privacy
 }
 
@@ -68,9 +74,9 @@ proc privacy::on_destroy_dialog {w W} {
 }
 
 proc privacy::open_dialog {connid res child} {
-    if {![cequal $res OK]} {
+    if {[cequal $res ERR]} {
 	MessageDlg .privacy_err -aspect 50000 -icon error \
-	    -message [format [::msgcat::mc "Requesting privacy rules: %s"] \
+	    -message [::msgcat::mc "Requesting privacy rules: %s" \
 			  [error_to_string $child]] \
 	    -type user -buttons ok -default 0 -cancel 0
 	return
@@ -266,9 +272,9 @@ proc privacy::edit_list {connid lf ln} {
 
 
 proc privacy::edit_list_dialog {connid ln name res child} {
-    if {![cequal $res OK]} {
+    if {[cequal $res ERR]} {
 	MessageDlg .privacy_list_err -aspect 50000 -icon error \
-	    -message [format [::msgcat::mc "Requesting privacy list: %s"] \
+	    -message [::msgcat::mc "Requesting privacy list: %s" \
 			  [error_to_string $child]] \
 	    -type user -buttons ok -default 0 -cancel 0
 	set child [jlib::wrapper:createtag query \
@@ -721,7 +727,7 @@ proc privacy::send_default_or_active_list {name tag args} {
 
 ###############################################################################
 #
-# Visible, invisible, ignore list block
+# Visible, invisible, ignore, conference list block
 #
 
 proc privacy::edit_special_list {name args} {
@@ -748,7 +754,7 @@ proc privacy::edit_special_list_dialog {connid name res child} {
     variable req_messages
     variable edit_messages
 
-    if {![cequal $res OK]} {
+    if {[cequal $res ERR]} {
 	if {[error_type_condition $child] != {cancel item-not-found}} {
 	    MessageDlg .privacy_list_err -aspect 50000 -icon error \
 		-message [format $req_messages($name) [error_to_string $child]] \
@@ -810,6 +816,7 @@ proc privacy::edit_special_list_dialog {connid name res child} {
 
 
 proc privacy::edit_special_list_enddialog {connid w name} {
+    $w itemconfigure 0 -state disabled
     send_special_list $connid $name [$w.fields get 0 end]
     destroy $w
 }
@@ -818,6 +825,10 @@ proc privacy::send_special_list {connid name items} {
     variable special_list
     variable cboxes
 
+    if {![is_supported $connid]} {
+	return
+    }
+
     if {![info exists special_list($connid,$name)]} {
 	set special_list($connid,$name) {}
     }
@@ -859,6 +870,11 @@ proc privacy::send_special_list {connid name items} {
 	    set action allow
 	    set postitems $newitems
 	}
+	conference {
+	    set children {}
+	    set action allow
+	    set postitems {}
+	}
     }
 
     set items1 {}
@@ -874,11 +890,60 @@ proc privacy::send_special_list {connid name items} {
     }
 
     send_list_iq "$name-list" $items1 \
-	-command [list [namespace current]::update_tkabber_lists \
-		       $connid $name $items $postitems] \
+	-command [list [namespace current]::get_answer $connid]\
 	-connection $connid
+
+    # We have to vwait because all privacy lists should be updated
+    # before sending next stanzas
+    lassign [wait_for_answer $connid] res child
+
+    update_tkabber_lists $connid $name $items $postitems $res $child
 }
 
+# subscription-list is responsible for blocking all messages
+# not from the roster.
+proc privacy::send_subscription_list {connid} {
+    variable accept_from_roster_only
+
+    if {![is_supported $connid]} {
+	return
+    }
+
+    if {$accept_from_roster_only} {
+	set items [list [jlib::wrapper:createtag item \
+			     -vars [list type subscription \
+					 value none \
+					 action deny \
+					 order 0]]]
+    } else {
+	set items {}
+    }
+
+    # If items aren't empty, we'll never send unavailable presence to
+    # all users to whom directed presence was sent. Bug?
+    send_list_iq "subscription-list" $items \
+	-command [list [namespace current]::get_answer $connid] \
+	-connection $connid
+
+    # We have to vwait because all privacy lists should be updated
+    # before sending next stanzas
+    lassign [wait_for_answer $connid] res child
+
+    update_tkabber_lists $connid subscription $items {} $res $child
+}
+
+proc privacy::on_accept_from_roster_only_change {args} {
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -connection { set connid $val }
+	}
+    }
+    if {![info exists connid]} {
+	set connid [jlib::route ""]
+    }
+
+    send_subscription_list $connid
+}
 
 proc privacy::update_tkabber_lists {connid name items postitems res child} {
     global userstatus textstatus statusdesc
@@ -886,16 +951,25 @@ proc privacy::update_tkabber_lists {connid name items postitems res child} {
     variable special_list
     variable cboxes
 
-    # Workaround for servers without privacy list support/push
-    if {$res == "OK"} {
-	set special_list($connid,$name) $items
-    }
-    array unset cboxes $connid,$name,*
-    foreach jid $special_list($connid,$name) {
-	set cboxes($connid,$name,$jid) 1
+    if {$res == "DISCONNECT"} return
+
+    switch -- $name {
+	subscription {
+	    # Subscription list doesn't contain JIDs
+	}
+	default {
+	    # Workaround for servers without privacy list support/push
+	    if {$res == "OK"} {
+		set special_list($connid,$name) $items
+	    }
+	    array unset cboxes $connid,$name,*
+	    foreach jid $special_list($connid,$name) {
+		set cboxes($connid,$name,$jid) 1
+	    }
+	}
     }
 
-    if {$res != "OK"} {
+    if {$res == "ERR"} {
 	MessageDlg .privacy_list_err -aspect 50000 -icon error \
 	    -message [format $send_messages($name) [error_to_string $child]] \
 	    -type user -buttons ok -default 0 -cancel 0
@@ -903,23 +977,32 @@ proc privacy::update_tkabber_lists {connid name items postitems res child} {
     }
 
     switch -- $name {
-	ignore {
-	    join_lists $connid "i-am-visible-list" {ignore-list invisible-list} \
-		{allow {}}
-	    join_lists $connid "i-am-invisible-list" {ignore-list visible-list} \
-		[list deny [list [jlib::wrapper:createtag presence-out]]]
+	ignore -
+	conference -
+	subscription {
+	    # Some inefficiency here. We load three lists twice.
+	    join_lists $connid "i-am-visible-list" \
+		       {ignore-list invisible-list conference-list subscription-list} \
+		       {allow {} {}}
+	    join_lists $connid "i-am-invisible-list" \
+		       {ignore-list visible-list conference-list subscription-list} \
+		       [list deny {} [list [jlib::wrapper:createtag presence-out]]]
 	}
 	invisible {
-	    join_lists $connid "i-am-visible-list" {ignore-list invisible-list} \
-		[list allow [list]]
+	    join_lists $connid "i-am-visible-list" \
+		       {ignore-list invisible-list conference-list subscription-list} \
+		       {allow {} {}}
 	}
 	visible {
-	    join_lists $connid "i-am-invisible-list" {ignore-list visible-list} \
-		[list deny [list [jlib::wrapper:createtag presence-out]]]
+	    join_lists $connid "i-am-invisible-list" \
+		       {ignore-list visible-list conference-list subscription-list} \
+		       [list deny {} [list [jlib::wrapper:createtag presence-out]]]
 	}
     }
 
-    # TODO: wait for iq reply before sending presence?
+    # ejabberd behaves correctly and applies privacy lists before
+    # routing any subsequent packet, so we haven't to wait for iq reply
+    # before sending presence. What about other servers?
     if {$userstatus == "invisible"} {
 	set status available
     } else {
@@ -938,21 +1021,33 @@ proc privacy::update_tkabber_lists {connid name items postitems res child} {
 }
 
 
-proc privacy::join_lists {connid name lists fallback args} {
+proc privacy::join_lists {connid name lists fallbacks args} {
     variable litems
 
     set items {}
     set i 0
+
+    # Appending myself to the list to make sure we can communicate
+    # between own resources
+    lappend items [jlib::wrapper:createtag item \
+		       -vars [list type jid \
+				   value [jlib::connection_bare_jid $connid] \
+				   action allow \
+				   order $i]]
+    incr i
+
     foreach ln $lists {
 	jlib::send_iq get \
 	    [jlib::wrapper:createtag query \
 		 -vars [list xmlns $::NS(privacy)] \
 		 -subtags [list [jlib::wrapper:createtag list \
 				     -vars [list name $ln]]]] \
-	    -command [list [namespace current]::get_items] \
+	    -command [list [namespace current]::get_items $connid] \
 	    -connection $connid
-	vwait [namespace current]::litems
-	foreach item $litems {
+
+	vwait [namespace current]::litems($connid)
+
+	foreach item $litems($connid) {
 	    jlib::wrapper:splitxml $item tag vars isempty chdata children
 	    catch { array unset tmp }
 	    array set tmp $vars
@@ -963,18 +1058,23 @@ proc privacy::join_lists {connid name lists fallback args} {
 	    incr i
 	}
     }
-    lappend items [jlib::wrapper:createtag item \
-		       -vars [list action [lindex $fallback 0] order $i] \
-		       -subtags [lindex $fallback 1]]
+    unset litems($connid)
+
+    foreach {action vars subtags} $fallbacks {
+	lappend items [jlib::wrapper:createtag item \
+			   -vars [concat [list action $action order $i] $vars] \
+			   -subtags $subtags]
+    }
+
     eval { send_list_iq $name $items -connection $connid } $args
 }
 
 
-proc privacy::get_items {res child} {
+proc privacy::get_items {connid res child} {
     variable litems
 
     if {$res != "OK"} {
-	set litems {}
+	set litems($connid) {}
 	return
     }
 
@@ -982,15 +1082,15 @@ proc privacy::get_items {res child} {
     jlib::wrapper:splitxml [lindex $children 0] tag vars isempty chdata children
 
     if {$tag == "list"} {
-	set litems $children
+	set litems($connid) $children
     } else {
-	set litems {}
+	set litems($connid) {}
     }
 }
 
 
 proc privacy::dropcmd {target source X Y op type data} {
-    add_special_jid $target [lindex $data 0]
+    add_special_jid $target [lindex $data 1]
 }
 
 
@@ -1069,9 +1169,6 @@ proc privacy::add_special_jid {f item} {
 
 proc privacy::activate_privacy_list {depth connid} {
     variable options
-    variable answer
-
-    if {!$options(activate_at_startup)} return
 
     set_status [::msgcat::mc "Waiting for activating privacy list"]
     debugmsg privacy "requested privacy list activation"
@@ -1079,14 +1176,15 @@ proc privacy::activate_privacy_list {depth connid} {
 	-command [list [namespace current]::get_answer $connid] \
 	-connection $connid
 
-    vwait [namespace current]::answer($connid,res)
+    lassign [wait_for_answer $connid] res child
 
-    switch -- $answer($connid,res) {
+    switch -- $res {
 	"OK" {
 	    set_status [::msgcat::mc "Privacy list is activated"]
+	    set_supported $connid
 	}
 	"ERR" {
-	    switch -- [lindex $answer($connid,error) 1] {
+	    switch -- [lindex [error_type_condition $child] 1] {
 		feature-not-implemented {
 		    # Privacy lists aren't implemented
 		    # Give up
@@ -1107,22 +1205,22 @@ proc privacy::activate_privacy_list {depth connid} {
 			# TODO: error message
 			return
 		    }
-		    # There's no "i-am-visible-list" privacy list
+		    # There's no required privacy list
 		    # Create it
 		    set_status \
 			[::msgcat::mc "Creating default privacy list"]
 
 		    join_lists $connid "i-am-visible-list" \
-			{ignore-list invisible-list} \
-			[list allow [list]] \
-			-command [list [namespace current]::get_answer \
-				       $connid]
+			{ignore-list invisible-list conference-list subscription-list} \
+			{allow {} {}} \
+			-command [list [namespace current]::get_answer $connid]
 
-		    vwait [namespace current]::answer($connid,res)
+		    lassign [wait_for_answer $connid] res child
 
-		    switch -- $answer($connid,res) {
+		    switch -- $res {
 			"OK" {
 			    # Activate newly created list
+			    set_supported $connid
 			    return [activate_privacy_list [expr {$depth + 1}] \
 							  $connid]
 			}
@@ -1134,16 +1232,15 @@ proc privacy::activate_privacy_list {depth connid} {
 			    NonmodalMessageDlg .privacy_list_error$connid \
 				-aspect 50000 -icon error \
 				-title [::msgcat::mc "Privacy lists error"] \
-				-message [format \
+				-message \
 				    [::msgcat::mc \
 					"Creating default privacy list failed:\
 					 %s\n\nTry to reconnect. If problem\
 					 persists, you may want to disable privacy\
-					 list activation at start"] \
-				    $answer($connid,errormsg)]
+					 list activation at start" \
+				    [error_to_string $child]]
 
 			    logout $connid
-			    clear_answer $connid
 
 			    # Break connected_hook
 			    return stop
@@ -1152,7 +1249,6 @@ proc privacy::activate_privacy_list {depth connid} {
 			    # "DISCONNECT"
 			    set_status \
 				[::msgcat::mc "Privacy list is not created"]
-			    clear_answer $connid
 
 			    # Break connected_hook
 			    return stop
@@ -1168,16 +1264,15 @@ proc privacy::activate_privacy_list {depth connid} {
 		    NonmodalMessageDlg .privacy_list_error$connid \
 			-aspect 50000 -icon error \
 			-title [::msgcat::mc "Privacy lists error"] \
-			-message [format \
+			-message \
 			    [::msgcat::mc \
 				"Activating privacy list failed:\
 				 %s\n\nTry to reconnect. If problem\
 				 persists, you may want to disable privacy\
-				 list activation at start"] \
-			    $answer($connid,errormsg)]
+				 list activation at start" \
+			    [error_to_string $child]]
 
 		    logout $connid
-		    clear_answer $connid
 
 		    # Break connected_hook
 		    return stop
@@ -1187,38 +1282,72 @@ proc privacy::activate_privacy_list {depth connid} {
 	default {
 	    # "DISCONNECT"
 	    set_status [::msgcat::mc "Privacy list is not activated"]
-	    clear_answer $connid
 
 	    # Break connected_hook
 	    return stop
 	}
     }
+}
 
-    clear_answer $connid
+##########################################################################
+
+proc privacy::activate_privacy_list_at_startup {connid} {
+    variable options
+
+    if {$options(activate_at_startup)} {
+	activate_privacy_list 0 $connid
+    }
 }
 
 hook::add connected_hook \
-    [list [namespace current]::privacy::activate_privacy_list 0] 5
+    [namespace current]::privacy::activate_privacy_list_at_startup 1
+
+##########################################################################
 
 proc privacy::get_answer {connid res child} {
     variable answer
     debugmsg privacy "got privacy list answer $connid $res $child"
 
-    if {$res == "ERR"} {
-	set answer($connid,error) [error_type_condition $child]
-	set answer($connid,errormsg) [error_to_string $child]
-    }
-    set answer($connid,res) $res
+    set answer($connid) [list $res $child]
 }
 
-proc privacy::clear_answer {connid} {
+proc privacy::wait_for_answer {connid} {
     variable answer
 
-    array unset answer $connid,*
+    vwait [namespace current]::answer($connid)
+    set res $answer($connid)
+    unset answer($connid)
+    return $res
 }
 
 ###############################################################################
 
+proc privacy::is_supported {connid} {
+    variable supported
+
+    expr {[info exists supported($connid)] && $supported($connid)}
+}
+
+proc privacy::set_supported {connid} {
+    variable supported
+
+    set supported($connid) 1
+}
+
+proc privacy::clear_supported {connid} {
+    variable supported
+
+    if {$connid == ""} {
+	catch { array unset supported * }
+    } else {
+	catch { array unset supported $connid }
+    }
+}
+
+hook::add disconnected_hook [namespace current]::privacy::clear_supported
+
+###############################################################################
+
 proc privacy::create_menu {m connid jid} {
     variable menu_messages
     variable special_list
@@ -1230,6 +1359,13 @@ proc privacy::create_menu {m connid jid} {
  	set rjid [node_and_server_from_jid $jid]
     }
 
+    if {![is_supported $connid] || \
+	    [chat::is_groupchat [chat::chatid $connid $rjid]]} {
+	set state disabled
+    } else {
+	set state normal
+    }
+
     set mm [menu $m.privacy_menu -tearoff 0]
 
     foreach name {invisible ignore} {
@@ -1243,11 +1379,13 @@ proc privacy::create_menu {m connid jid} {
 
 	$mm add checkbutton -label $menu_messages($name) \
 	    -variable [namespace current]::cboxes($connid,$name,$rjid) \
-	    -command [list [namespace current]::add_to_special_list \
+	    -command [list [namespace current]::update_special_list \
 			   $connid $name $rjid]
     }
 
-    $m add cascad -label [::msgcat::mc "Privacy rules"] -menu $mm
+    $m add cascad -label [::msgcat::mc "Privacy rules"] \
+		  -menu $mm \
+		  -state $state
 }
 
 hook::add chat_create_user_menu_hook \
@@ -1258,11 +1396,22 @@ hook::add roster_jid_popup_menu_hook \
 	  [namespace current]::privacy::create_menu 79
 
 
+###############################################################################
+
+proc privacy::update_special_list {connid name jid} {
+    variable cboxes
+
+    if {[info exists cboxes($connid,$name,$jid)] && $cboxes($connid,$name,$jid)} {
+	add_to_special_list $connid $name $jid
+    } else {
+	remove_from_special_list $connid $name $jid
+    }
+}
+
 ###############################################################################
 
 proc privacy::add_to_special_list {connid name jid} {
     variable special_list
-    variable cboxes
 
     if {![info exists special_list($connid,$name)]} {
 	set special_list($connid,$name) {}
@@ -1270,23 +1419,32 @@ proc privacy::add_to_special_list {connid name jid} {
 
     set idx [lsearch -exact $special_list($connid,$name) $jid]
 
-    if {[info exists cboxes($connid,$name,$jid)] && \
-	    ($cboxes($connid,$name,$jid) && $idx < 0)} {
+    if {$idx < 0} {
 	send_special_list $connid $name \
 			  [linsert $special_list($connid,$name) 0 $jid]
-	return
-    }
-    if {![info exists cboxes($connid,$name,$jid)] || \
-	    (!$cboxes($connid,$name,$jid) && $idx >= 0)} {
-	send_special_list $connid $name \
-			  [lreplace $special_list($connid,$name) $idx $idx]
-	return
     }
 }
 
 ###############################################################################
 
-proc privacy::process_iq {connid from child} {
+proc privacy::remove_from_special_list {connid name jid} {
+    variable special_list
+
+    if {![info exists special_list($connid,$name)]} {
+	set special_list($connid,$name) {}
+    }
+
+    set idx [lsearch -exact $special_list($connid,$name) $jid]
+
+    if {$idx >= 0} {
+	send_special_list $connid $name \
+			  [lreplace $special_list($connid,$name) $idx $idx]
+    }
+}
+
+###############################################################################
+
+proc privacy::process_iq {connid from lang child} {
     if {$from != "" && \
 	    !([string equal -nocase $from [jlib::connection_server $connid]] || \
 	      [string equal -nocase $from [jlib::connection_bare_jid $connid]] || \
@@ -1311,6 +1469,12 @@ proc privacy::process_iq {connid from child} {
 		    ignore-list {
 			reload_special_list $connid ignore
 		    }
+		    conference-list {
+			reload_special_list $connid conference
+		    }
+		    subscription-list {
+			reload_subscription_list $connid
+		    }
 		}
 	    }
 	}
@@ -1338,12 +1502,38 @@ proc privacy::clear_list_vars {connid} {
 
 hook::add disconnected_hook [namespace current]::privacy::clear_list_vars
 
+###############################################################################
+# Conference list should be loaded before any join group attempt is made
+
+proc privacy::get_conference_list {connid} {
+    jlib::send_iq get \
+	[jlib::wrapper:createtag query \
+	     -vars [list xmlns $::NS(privacy)] \
+	     -subtags [list [jlib::wrapper:createtag list \
+				 -vars [list name "conference-list"]]]] \
+	-command [list [namespace current]::get_answer $connid] \
+	-connection $connid
+
+    lassign [wait_for_answer $connid] res child
+
+    if {($res == "OK") || \
+	    ($res == "ERR" && \
+	     [lindex [error_type_condition $child] 1] == "item-not-found")} {
+	set_supported $connid
+    }
+
+    store_special_list $connid conference $res $child
+}
+
+hook::add connected_hook [namespace current]::privacy::get_conference_list 2
+
 ###############################################################################
 
 proc privacy::get_list_vars {connid} {
     foreach name {invisible visible ignore} {
 	reload_special_list $connid $name
     }
+    reload_subscription_list $connid
 }
 
 hook::add connected_hook [namespace current]::privacy::get_list_vars
@@ -1386,3 +1576,77 @@ proc privacy::store_special_list {connid name res child} {
 
 ###############################################################################
 
+proc privacy::reload_subscription_list {connid} {
+    jlib::send_iq get \
+	[jlib::wrapper:createtag query \
+	     -vars [list xmlns $::NS(privacy)] \
+	     -subtags [list [jlib::wrapper:createtag list \
+				 -vars [list name "subscription-list"]]]] \
+	-command [list [namespace current]::store_subscription_list $connid] \
+	-connection $connid
+}
+
+proc privacy::store_subscription_list {connid res child} {
+    variable accept_from_roster_only
+
+    set accept_from_roster_only 0
+
+    if {$res != "OK"} return
+
+    jlib::wrapper:splitxml $child tag vars isempty chdata children
+    jlib::wrapper:splitxml [lindex $children 0] tag vars isempty chdata children
+
+    if {$tag != "list"} return
+
+    foreach item $children {
+	jlib::wrapper:splitxml $item tag1 vars1 isempty1 chdata1 children1
+	if {[jlib::wrapper:getattr $vars1 type] == "subscription" && \
+		[jlib::wrapper:getattr $vars1 value] == "none" && \
+		[jlib::wrapper:getattr $vars1 action] == "deny"} {
+	    set accept_from_roster_only 1
+	}
+    }
+}
+
+###############################################################################
+
+proc privacy::enable_menu {connid} {
+    set c [::jlib::route ""]
+    if {![is_supported $c]} {
+	return
+    }
+
+    set m [.mainframe getmenu privacy]
+    if {$::ifacetk::options(show_tearoffs)} {
+	set start 1
+    } else {
+	set start 0
+    }
+    for {set i $start} {$i <= [$m index end]} {incr i} {
+	catch {$m entryconfigure $i -state normal}
+    }
+}
+
+proc privacy::disable_menu {connid} {
+    set c [::jlib::route ""]
+    if {[llength [jlib::connections]] > 0 && [is_supported $c]} return
+
+    set m [.mainframe getmenu privacy]
+    if {$::ifacetk::options(show_tearoffs)} {
+	set start 1
+    } else {
+	set start 0
+    }
+    for {set i $start} {$i <= [$m index end]} {incr i} {
+	catch {$m entryconfigure $i -state disabled}
+    }
+    $m entryconfigure [$m index [::msgcat::mc "Activate lists at startup"]] \
+       -state normal
+}
+
+hook::add connected_hook [namespace current]::privacy::enable_menu
+hook::add disconnected_hook [namespace current]::privacy::disable_menu
+hook::add finload_hook [list [namespace current]::privacy::disable_menu {}]
+
+###############################################################################
+
