@@ -1,24 +1,18 @@
-# $Id: roster.tcl,v 1.137 2006/04/09 19:45:37 aleksey Exp $
+# $Id: roster.tcl 1087 2007-04-08 17:27:04Z sergei $
 
 namespace eval roster {
     variable undef_group_name [::msgcat::mc "Undefined"]
     variable chats_group_name [::msgcat::mc "Active Chats"]
-    array set long_statusdesc [list \
-	available   [::msgcat::mc "%s is available"] \
-	chat        [::msgcat::mc "%s is free to chat"] \
-	away        [::msgcat::mc "%s is away"] \
-	xa          [::msgcat::mc "%s is extended away"] \
-	dnd         [::msgcat::mc "%s doesn't want to be disturbed"] \
-	invisible   [::msgcat::mc "%s is invisible"] \
-	unavailable [::msgcat::mc "%s is unavailable"]]
+    variable own_resources_group_name [::msgcat::mc "My Resources"]
 }
 
-proc roster::process_item {connid jid name groups subsc ask category subtype} {
+proc roster::process_item {connid jid name groups subsc ask} {
     variable roster
     variable undef_group_name
     variable chats_group_name
+    variable own_resources_group_name
 
-    debugmsg roster "ROSTER_ITEM: $jid; $name; $groups; $subsc; $ask; $category; $subtype"
+    debugmsg roster "ROSTER_ITEM: $connid; $jid; $name; $groups; $subsc; $ask"
 
     set jid [tolower_node_and_domain $jid]
 
@@ -27,7 +21,7 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 	    lappend roster(jids,$connid) $jid
 	}
 	set groups [lrmdups $groups]
-	foreach group [list "" $undef_group_name $chats_group_name] {
+	foreach group [list "" $undef_group_name $chats_group_name $own_resources_group_name] {
 	    set ind [lsearch -exact $groups $group]
 	    if {$ind >= 0} {
 		set groups [lreplace $groups $ind $ind]
@@ -37,12 +31,9 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 	set roster(name,$connid,$jid)     $name
 	set roster(subsc,$connid,$jid)    $subsc
 	set roster(ask,$connid,$jid)      $ask
-	set roster(category,$connid,$jid) $category
-	set roster(subtype,$connid,$jid)  $subtype
 
 	catch {unset roster(cached_category_and_subtype,$connid,$jid)}
-	lassign [get_category_and_subtype $connid $jid] ccategory csubtype
-	set roster(isuser,$connid,$jid) [cequal $ccategory user]
+	get_category_and_subtype $connid $jid
     } else {
 	lvarpop roster(jids,$connid) [lsearch -exact $roster(jids,$connid) $jid]
 
@@ -50,9 +41,6 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 	catch {unset roster(name,$connid,$jid)}
 	catch {unset roster(subsc,$connid,$jid)}
 	catch {unset roster(ask,$connid,$jid)}
-	catch {unset roster(category,$connid,$jid)}
-	catch {unset roster(subtype,$connid,$jid)}
-	catch {unset roster(isuser,$connid,$jid)}
 	catch {unset roster(cached_category_and_subtype,$connid,$jid)}
     }
 }
@@ -60,14 +48,12 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 hook::add roster_item_hook [namespace current]::roster::process_item
 hook::add roster_push_hook [namespace current]::roster::process_item
 
-proc client:roster_item {connid jid name groups subsc ask category subtype} {
-    hook::run roster_item_hook \
-	$connid $jid $name $groups $subsc $ask $category $subtype
+proc client:roster_item {connid jid name groups subsc ask} {
+    hook::run roster_item_hook $connid $jid $name $groups $subsc $ask 
 }
 
-proc client:roster_push {connid jid name groups subsc ask category subtype} {
-    hook::run roster_push_hook \
-	$connid $jid $name $groups $subsc $ask $category $subtype
+proc client:roster_push {connid jid name groups subsc ask} {
+    hook::run roster_push_hook $connid $jid $name $groups $subsc $ask
     ::redraw_roster
 }
 
@@ -210,13 +196,13 @@ proc roster::itemconfig {connid jid args} {
 	    -ask      {set param ask}
 	    -category {set param category}
 	    -subtype  {set param subtype}
-	    -isuser   {set param isuser}
-	    default {return}
+	    -isuser   {
+		return [cequal [lindex [get_category_and_subtype $connid $jid] 0] "user"]
+	    }
+	    default   {return -code error "Illegal option"}
 	}
 	if {[info exists roster($param,$connid,$jid)]} {
 	    return $roster($param,$connid,$jid)
-	} elseif {$param == "isuser"} {
-	    return 0
 	} else {
 	    return ""
 	}
@@ -229,8 +215,7 @@ proc roster::itemconfig {connid jid args} {
 		-ask      {set param ask}
 		-category {set param category}
 		-subtype  {set param subtype}
-		-isuser   {set param isuser}
-		default   {set param ""}
+		default   {return -code error "Illegal option"}
 	    }
 	    set roster($param,$connid,$jid) $val
 	}
@@ -238,7 +223,6 @@ proc roster::itemconfig {connid jid args} {
 }
 
 proc roster::on_change_jid_presence {connid jid type x args} {
-    variable long_statusdesc
     variable roster
 
     switch -- $type {
@@ -257,8 +241,8 @@ proc roster::on_change_jid_presence {connid jid type x args} {
 	if {$category == "user"} {
 	    set status [get_user_status $connid $rjid]
 	    set label [get_label $connid $rjid]
-	    if {![catch {set desc $long_statusdesc($status)}]} {
-		set_status [format $desc $label]
+	    if {![catch {set desc [::get_long_status_desc $status]}]} {
+		set_status [format "%s $desc" $label]
 	    }
 	    hook::run on_change_user_presence_hook $label $status
 	}
@@ -271,14 +255,18 @@ hook::add client_presence_hook roster::on_change_jid_presence 60
 proc roster::find_jid {connid jid} {
     variable roster
 
-    if {[info exists roster(category,$connid,$jid)]} {
+    if {![info exists roster(jids,$connid)]} {
+	return ""
+    }
+
+    if {[lsearch -exact $roster(jids,$connid) $jid] >= 0} {
 	return $jid
     }
 
     lassign [get_category_and_subtype $connid $jid] category subtype
     if {$category == "user"} {
 	set rjid [node_and_server_from_jid $jid]
-	if {[info exists roster(category,$connid,$rjid)]} {
+	if {[lsearch -exact $roster(jids,$connid) $rjid] >= 0} {
 	    lassign [get_category_and_subtype $connid $rjid] rcategory rsubtype
 	    if {$category == $rcategory} {
 		return $rjid
@@ -297,72 +285,111 @@ proc roster::get_label {connid jid} {
     }
 }
 
+proc roster::override_category_and_subtype {connid jid category subtype} {
+    variable roster
+
+    set roster(overridden_category_and_subtype,$connid,$jid) \
+	[list $category $subtype]
+}
+
 proc roster::get_category_and_subtype {connid jid} {
     variable roster
 
+    if {[info exists roster(overridden_category_and_subtype,$connid,$jid)]} {
+	return $roster(overridden_category_and_subtype,$connid,$jid)
+    }
+
+    set server [server_from_jid $jid]
+    if {[info exists roster(overridden_category_and_subtype,$connid,$server)]} {
+	catch { unset roster(cached_category_and_subtype,$connid,$jid) }
+	set cs [heuristically_get_category_and_subtype $connid $jid]
+	set roster(overridden_category_and_subtype,$connid,$jid) $cs
+	return $cs
+    }
+
     if {[info exists roster(cached_category_and_subtype,$connid,$jid)]} {
 	return $roster(cached_category_and_subtype,$connid,$jid)
     }
 
-    if {[info exists roster(category,$connid,$jid)] && \
-	    $roster(category,$connid,$jid) != ""} {
-	return [list $roster(category,$connid,$jid) $roster(subtype,$connid,$jid)]
-    }
-    
-    return [heuristically_get_category_and_subtype $connid $jid]
+    catch { plugins::cache_categories::request_category_and_subtype $connid $jid }
+
+    set cs [heuristically_get_category_and_subtype $connid $jid]
+    set roster(cached_category_and_subtype,$connid,$jid) $cs
+    return $cs
 }
 
 proc roster::heuristically_get_category_and_subtype {connid jid} {
     variable roster
 
     set node [node_from_jid $jid]
-    set updomain [lindex [split [server_from_jid $jid] .] 0]
+    set server [server_from_jid $jid]
+    set resource [resource_from_jid $jid]
 
-    if {$node == ""} {
+    if {$node == "" && $resource == ""} {
+	set updomain [lindex [split $server .] 0]
 	set category service
 
-	if {[lcontain {aim icq irc jabber jud msn pager rss serverlist \
-			   sms smtp yahoo} $updomain]} {
-	    set subtype $updomain
-	} elseif {[cequal icqv7 $updomain]} {
-	    set subtype icq
-	} elseif {[cequal gg $updomain]} {
-	    set subtype x-gadugadu
-	} elseif {([cequal pogoda $updomain]) \
-		      || ([cequal weather $updomain])} {
-	    set subtype x-weather
-	} else {
-	    set subtype ""
+	switch -- $updomain {
+	    aim        -
+	    icq        -
+	    irc        -
+	    jabber     -
+	    jud        -
+	    msn        -
+	    mrim       -
+	    pager      -
+	    rss        -
+	    serverlist -
+	    sms	       -
+	    smtp       -
+	    yahoo {
+		set subtype $updomain
+	    }
+	    gg {
+		set subtype gadu-gadu
+	    }
+	    pogoda -
+	    weather {
+		set subtype x-weather
+	    }
+	    default {
+		set subtype ""
+	    }
 	}
 
-	set roster(cached_category_and_subtype,$connid,$jid) [list $category $subtype]
 	return [list $category $subtype]
     }
 
+    if {$node == ""} {
+	return [get_category_and_subtype $connid $server]
+    }
+
     if {[resource_from_jid $jid] == ""} {
-	switch -- $updomain {
-	    tach -
-	    conference {
-		set category conference
-		set subtype ""
-	    }
-	    irc {
-		set category user
-		set subtype ""
-		if {[string first "%" $node] != -1} {
+	lassign [get_category_and_subtype $connid $server] scategory ssubtype
+
+	switch -glob -- $scategory/$ssubtype {
+	    conference/irc {
+		if {[string first "%" $node] >= 0} {
 		    set category conference
 		    set subtype irc
+		} else {
+		    set category user
+		    set subtype ""
 		}
 	    }
+	    conference/* {
+		set category conference
+		set subtype ""
+	    }
 	    default {
 		set category user
 		set subtype ""
 	    }
 	}
-	set roster(cached_category_and_subtype,$connid,$jid) [list $category $subtype]
+
 	return [list $category $subtype]
     }
-    set roster(cached_category_and_subtype,$connid,$jid) {user client}
+
     return {user client}
 }
 
@@ -376,8 +403,8 @@ proc roster::clean {} {
     array unset roster ask,*
     array unset roster category,*
     array unset roster subtype,*
-    array unset roster isuser,*
     array unset roster cached_category_and_subtype,*
+    array unset roster overridden_category_and_subtype,*
     ::redraw_roster
 }
 
@@ -391,28 +418,26 @@ proc roster::clean_connection {connid} {
     array unset roster ask,$connid,*
     array unset roster category,$connid,*
     array unset roster subtype,$connid,*
-    array unset roster isuser,$connid,*
     array unset roster cached_category_and_subtype,$connid,*
+    array unset roster overridden_category_and_subtype,$connid,*
 
     ::redraw_roster
 }
 
-proc roster::is_user {connid jid} {
-    return [cequal [lindex [get_category_and_subtype $connid $jid] 0] "user"]
-}
-
 ###############################################################################
 
 proc roster::item_to_xml {connid jid} {
     variable roster
     variable undef_group_name
     variable chats_group_name
+    variable own_resources_group_name
 
     set grtags {}
     foreach group $roster(group,$connid,$jid) {
 	if {![cequal $group ""] && \
 		![cequal $group $undef_group_name] && \
-		![cequal $group $chats_group_name]} {
+		![cequal $group $chats_group_name] && \
+		![cequal $group $own_resources_group_name]} {
 	    lappend grtags [jlib::wrapper:createtag group -chdata $group]
 	}
     }
@@ -453,21 +478,26 @@ proc roster::remove_item_fallback {connid jid} {
 
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
-	     -vars {xmlns jabber:iq:roster} \
+	     -vars [list xmlns $::NS(roster)] \
 	     -subtags [list [jlib::wrapper:createtag item \
 				 -vars [list jid $jid \
-					    subscription remove]]]] \
+					     subscription remove]]]] \
 	-connection $connid
 	
     jlib::send_presence -to $jid -type unsubscribe -connection $connid
+    jlib::send_presence -to $jid -type unsubscribed -connection $connid
 
     lassign [get_category_and_subtype $connid $jid] category subtype
 
-    if {($category == "service") && ($jid != [jlib::connection_server $connid])} {
+    if {(($category == "service") || \
+	 ($category == "server") || \
+	 ($category == "gateway")) && \
+	[string compare -nocase [node_and_server_from_jid $jid] \
+				[jlib::connection_server $connid]]} {
 	jlib::send_iq set \
 	    [jlib::wrapper:createtag query \
-		-vars {xmlns jabber:iq:register} \
-		-subtags [list [jlib::wrapper:createtag remove]]] \
+		 -vars [list xmlns $::NS(register)] \
+		 -subtags [list [jlib::wrapper:createtag remove]]] \
 	    -to $jid \
 	    -connection $connid
     }
@@ -605,7 +635,7 @@ proc roster::add_group_by_jid_regexp {name regexp} {
 	set items {}
 
 	foreach jid $roster(jids,$connid) {
-	    if {[regexp $regexp $jid]} {
+	    if {[regexp -- $regexp $jid]} {
 		set idx [lsearch -exact $roster(group,$connid,$jid) $name]
 		lappend roster(group,$connid,$jid) $name
 		set roster(group,$connid,$jid) \
@@ -630,7 +660,7 @@ proc roster::export_to_file {connid} {
     variable roster
 
     set filename [tk_getSaveFile \
-		      -initialdir ~/.tkabber/ \
+		      -initialdir $::configdir \
 		      -initialfile [jlib::connection_user $connid].roster \
 		      -filetypes [list \
 				      [list [::msgcat::mc "Roster Files"] \
@@ -654,7 +684,7 @@ proc roster::import_from_file {connid} {
     variable roster
 
     set filename [tk_getOpenFile \
-		      -initialdir ~/.tkabber/ \
+		      -initialdir $::configdir \
 		      -initialfile [jlib::connection_user $connid].roster \
 		      -filetypes [list \
 				      [list [::msgcat::mc "Roster Files"] \
@@ -676,3 +706,6 @@ proc roster::import_from_file {connid} {
     }
 }
 
+###############################################################################
+
+# vim:ts=8:sw=4:sts=4:noet
