@@ -1,4 +1,4 @@
-# $Id: userinfo.tcl,v 1.64 2006/05/28 14:54:24 aleksey Exp $
+# $Id: userinfo.tcl 1038 2007-03-10 14:09:40Z sergei $
 
 
 namespace eval userinfo {
@@ -19,7 +19,7 @@ proc userinfo::show_info_dialog {} {
     set connid [lindex [jlib::connections] 0]
     set show_info_connid [jlib::connection_jid $connid]
 
-    Dialog $gw -title [::msgcat::mc "Show user info"] -separator 1 -anchor e \
+    Dialog $gw -title [::msgcat::mc "Show user or service info"] -separator 1 -anchor e \
 	    -default 0 -cancel 1
 
     set gf [$gw getframe]
@@ -29,7 +29,7 @@ proc userinfo::show_info_dialog {} {
 
     label $gf.ljid -text [::msgcat::mc "JID:"]
     ecursor_entry [ComboBox $gf.jid -textvariable [namespace current]::show_info_jid \
-	    -values $show_info_list -width 35].e
+	    -values [linsert $show_info_list 0 ""] -width 35].e
     
     grid $gf.ljid -row 0 -column 0 -sticky e
     grid $gf.jid  -row 0 -column 1 -sticky ew
@@ -70,8 +70,6 @@ proc userinfo::show_info {gw} {
     }
 
     set show_info_list [update_combo_list $show_info_list $show_info_jid 10]
-    set custom::saved([namespace current]::show_info_list) $show_info_list
-    custom::store
     userinfo::open $show_info_jid -connection $connid
 }
 
@@ -101,9 +99,9 @@ proc userinfo::pack_entry {jid g row name text} {
 #        set bgcolor [lindex [$g.$name configure -background] 4]
         set bgcolor [option get $g background Notebook]
 	if {[info tclversion] >= 8.4} {
-    	    $g.$name configure -state readonly -highlightcolor $bgcolor -takefocus 0
+    	    $g.$name configure -state readonly -relief flat -highlightcolor $bgcolor -takefocus 0
 	} else {
-	    $g.$name configure -state disabled -background $bgcolor
+	    $g.$name configure -state disabled -relief flat -background $bgcolor
 	}
     }
 
@@ -121,9 +119,9 @@ proc userinfo::pack_text_entry {jid g row name text} {
     set w [w_from_jid $jid]
 
     label $g.l$name -text $text
-    text $g.$name -height 1 -state disabled -background [option get $g background Notebook]
-    $g.$name tag configure emphasized -elide 1
-    $g.$name tag configure nonemphasized -elide 0
+    text $g.$name -height 1 -state disabled -relief flat \
+		  -background [option get $g background Notebook]
+    ::richtext::config $g.$name -using url
     fill_user_description $g.$name userinfo($name,$jid) 0
     
     grid $g.l$name -row $row -column 0 -sticky e
@@ -132,8 +130,9 @@ proc userinfo::pack_text_entry {jid g row name text} {
     trace variable [namespace current]::userinfo($name,$jid) w \
 	[list userinfo::fill_user_description $g.$name userinfo($name,$jid) 0]
     bind $g.$name <Destroy> \
-	[list trace vdelete [namespace current]::userinfo($name,$jid) w \
-		  [list userinfo::fill_user_description $g.$name userinfo($name,$jid) 0]]
+	+[list trace vdelete [namespace current]::userinfo($name,$jid) w \
+	       [list userinfo::fill_user_description $g.$name \
+		     userinfo($name,$jid) 0]]
 }
 
 proc userinfo::pack_spinbox {jid g row col name low high text} {
@@ -206,7 +205,7 @@ proc userinfo::open {jid args} {
     if {$editable} {
 	set bbox [ButtonBox $w.bbox -spacing 10 -padx 10 -default 0]
 	$bbox add -text [::msgcat::mc "OK"] -command "
-		userinfo::send_vcard [list $jid]
+		userinfo::send_vcard [list $connid] [list $jid]
 		destroy [list $w]
 	"
 	bind $w <Return>  "ButtonBox::invoke $bbox default"
@@ -317,10 +316,13 @@ proc userinfo::open {jid args} {
 
     set a [pack_frame $about.about [string trim [::msgcat::mc "About "]]]
     set sw [ScrolledWindow $a.sw -scrollbar vertical]
-    text $a.text -font $font -height 12 -wrap word
+    if {!$editable} {
+	text $a.text -font $font -height 12 -wrap word
+	::richtext::config $a.text -using {url emoticon}
+    } else {
+	textUndoable $a.text -font $font -height 12 -wrap word
+    }
     $sw setwidget $a.text
-    $a.text tag configure emphasized -elide 1
-    $a.text tag configure nonemphasized -elide 0
     bind $a.text <Key-Return> [bind Text <Key-Return>]
     bind $a.text <Key-Return> +break
     bind $a.text <Control-Key-Return> "
@@ -331,10 +333,12 @@ proc userinfo::open {jid args} {
     pack $sw -fill both -expand yes
     pack $a -fill both -expand yes
     trace variable [namespace current]::userinfo(desc,$jid) w \
-	[list userinfo::fill_user_description $a.text userinfo(desc,$jid) $editable]
+	[list userinfo::fill_user_description $a.text \
+	      userinfo(desc,$jid) $editable]
     bind $a.text <Destroy> \
-	[list trace variable [namespace current]::userinfo(desc,$jid) w \
-		  [list userinfo::fill_user_description $a.text userinfo(desc,$jid) $editable]]
+	+[list trace vdelete [namespace current]::userinfo(desc,$jid) w \
+	       [list userinfo::fill_user_description $a.text \
+		     userinfo(desc,$jid) $editable]]
     set userinfo(descfield,$jid) $a.text
 
     manage_focus $jid about $b.bday[expr {$editable ? "year" : ""}] $editable
@@ -367,6 +371,7 @@ proc userinfo::open {jid args} {
 	if {![info exists userinfo(photo_use,$jid)]} {
 	    set userinfo(photo_use,$jid) none
 	}
+
 	radiobutton $p.use_url -text [::msgcat::mc "URL"] \
 	    -value url -variable userinfo::userinfo(photo_use,$jid) \
 	    -command [list [namespace current]::enable_active_photo $p $jid]
@@ -379,21 +384,27 @@ proc userinfo::open {jid args} {
 	entry $p.photo_url -textvariable userinfo::userinfo(photo_extval,$jid)
 	label $p.photo -image $photo_img
 
-	grid $p.use_url -row 0 -column 0 -sticky w
-	grid $p.photo_url  -row 0 -column 1 -sticky we -columnspan 2
-	grid $p.use_image -row 1 -column 0 -sticky w
-	grid $p.photo  -row 1 -column 1 -sticky we
-	grid $p.use_none -row 2 -column 0 -sticky w
+	grid $p.use_url -row 1 -column 0 -sticky w
+	grid $p.photo_url  -row 1 -column 1 -sticky we
+	grid $p.use_image -row 2 -column 0 -sticky w
+	grid $p.photo  -row 3 -column 1 -sticky we
+	grid $p.use_none -row 0 -column 0 -sticky w
 
 	button $p.loadimage -text [::msgcat::mc "Load Image"] \
 	    -command [list userinfo::load_photo $jid $p.photo]
-	grid $p.loadimage -row 1 -column 2 -sticky e
+	grid $p.loadimage -row 2 -column 1 -sticky w
 	grid columnconfig $p 1 -weight 1 -minsize 0
 	#grid rowconfig $p 0 -weight 1
 	grid rowconfig $p 1
 	#grid rowconfig $p 1 -weight 1
 	enable_active_photo $p $jid
-	manage_focus $jid photo $p.use_url $editable
+	manage_focus $jid photo $p.use_none $editable
+
+	trace variable [namespace current]::userinfo(photo_use,$jid) w \
+	    [list userinfo::enable_active_photo $p $jid]
+	bind $p <Destroy> \
+	    +[list trace vdelete [namespace current]::userinfo(photo_use,$jid) w \
+		   [list userinfo::enable_active_photo $p $jid]]
     }
 
 
@@ -461,7 +472,7 @@ proc userinfo::client_page {tab connid jid editable} {
 
 hook::add userinfo_hook [namespace current]::userinfo::client_page
 
-proc userinfo::enable_active_photo {p jid} {
+proc userinfo::enable_active_photo {p jid args} {
     switch -- $userinfo::userinfo(photo_use,$jid) {
 	url {
 	    $p.photo_url configure -state normal
@@ -491,7 +502,8 @@ proc userinfo::fill_user_description {txt descvar editable args} {
 	if {$editable} {
 	    $txt insert 0.0 [set $descvar]
 	} else {
-	    ::chat::add_emoteiconed_text $txt [set $descvar] ""
+	    ::richtext::render_message $txt [set $descvar] ""
+	    $txt delete {end - 1 char}
 	}
 	$txt configure -state $state
     }
@@ -660,6 +672,7 @@ proc userinfo::parse_vcard_photo_item {jid items} {
 		catch {		
 		    set userinfo(photo_binval,$jid) [base64::decode $chdata]
 		    set userinfo(photo_use,$jid) image
+		    photo_$jid blank
 		    photo_$jid put $chdata
 		    catch { reconfigure_photo $jid }
 		}
@@ -902,7 +915,7 @@ proc userinfo::append_tag {varname tag chdata {subtags {}}} {
     }
 }
 
-proc userinfo::send_vcard {jid} {
+proc userinfo::send_vcard {connid jid} {
     variable userinfo
 
     set w [w_from_jid $jid]
@@ -938,8 +951,8 @@ proc userinfo::send_vcard {jid} {
     jlib::send_iq set \
 	[jlib::wrapper:createtag vCard \
 	     -vars [list xmlns vcard-temp] \
-	     -subtags $tags]
-
+	     -subtags $tags] \
+	-connection $connid
 }
 
 proc userinfo::make_n_item {jid} {
@@ -1078,3 +1091,4 @@ hook::add postload_hook \
     [list disco::browser::register_feature_handler jabber:iq:version userinfo::open_client \
 	 -desc [list * [::msgcat::mc "Version"]]]
 
+#  vim:ts=8:sw=4:sts=4:noet
